<?php

namespace App\Controllers;

use App\baseClasses\KCBase;
use App\baseClasses\KCRequest;
use App\models\KCAppointment;
use App\models\KCBill;
use App\models\KCClinicSession;
use App\models\KCPatientEncounter;
use App\models\KCAppointmentServiceMapping;
use App\models\KCBillItem;
use App\models\KCClinic;
use  App\models\KCReceptionistClinicMapping;
use App\controllers\KCPaymentController;
use DateTime;
use Exception;

class KCAppointmentController extends KCBase {

	public $db;

	private $request;

	public function __construct() {

		global $wpdb;

		$this->db = $wpdb;

		$this->request = new KCRequest();

	}

	public function index() {

		if ( ! kcCheckPermission( 'appointment_list' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => $this->permission_message,
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();
		$users_table        = $this->db->base_prefix . 'users';
		$appointments_table = $this->db->prefix . 'kc_appointments';
		$clinics_table      = $this->db->prefix . 'kc_clinics';
		$start_date         = esc_sql($request_data['start_date']);
		$end_date           = esc_sql($request_data['end_date']);
		$appointments_service_table = $this->db->prefix . 'kc_appointment_service_mapping';
		$service_table = $this->db->prefix . 'kc_services';
		$query = "
			SELECT {$appointments_table}.*,
		       doctors.display_name  AS doctor_name,
		       patients.display_name AS patient_name,
		       {$clinics_table}.name AS clinic_name
			FROM  {$appointments_table}
		       LEFT JOIN {$users_table} doctors
		              ON {$appointments_table}.doctor_id = doctors.id
		       LEFT JOIN {$users_table} patients
		              ON {$appointments_table}.patient_id = patients.id
		       LEFT JOIN {$clinics_table}
		              ON {$appointments_table}.clinic_id = {$clinics_table}.id
            WHERE {$appointments_table}.appointment_start_date >= '{$start_date}' AND {$appointments_table}.appointment_start_date <= '{$end_date}' ";

        $current_user_id = get_current_user_id();

        $current_user_role = $this->getLoginUserRole();

		if ($current_user_role ===  $this->getDoctorRole() ) {
			$query .= " AND {$appointments_table}.doctor_id = " . $current_user_id;
		} elseif ( $current_user_role === $this->getPatientRole() ) {
			$query .= " AND {$appointments_table}.patient_id = " . $current_user_id;
		}

        if(isKiviCareProActive()){
            if($current_user_role == $this->getClinicAdminRole()){
                $clinic_id = kcGetClinicIdOfClinicAdmin();
                $query .= " AND {$appointments_table}.clinic_id={$clinic_id} ";
            }elseif ($current_user_role == $this->getReceptionistRole()) {
                $clinic_id = kcGetClinicIdOfReceptionist();
                $query .= " AND {$appointments_table}.clinic_id={$clinic_id} ";
            }
        }

		$appointments     = collect($this->db->get_results( $query ))->unique('id')->values();
        if ( ! count( $appointments ) ) {
            echo json_encode( [
                'status'  => false,
                'message' => esc_html__('No appointments found', 'kc-lang'),
                'data'    => []
            ] );
            wp_die();
        }

        $new_appointments = $telemed_data = $googlemeet_data = [];
        $all_appointment_id = $appointments->pluck('id')->implode(",");

        $all_service_query = "SELECT {$appointments_table}.id AS appointment_id,GROUP_CONCAT({$service_table}.name) AS service_name FROM {$appointments_table}
				LEFT JOIN {$appointments_service_table} ON {$appointments_table}.id = {$appointments_service_table}.appointment_id JOIN {$service_table} 
				ON {$appointments_service_table}.service_id = {$service_table}.id WHERE 0 = 0 AND {$appointments_service_table}.appointment_id IN ({$all_appointment_id}) GROUP BY {$appointments_table}.id ";

        $all_service_results = collect($this->db->get_results($all_service_query))->keyBy('appointment_id')->toArray();

        if(isKiviCareTelemedActive()){
            $zoom_mapping_table = $this->db->prefix.'kc_appointment_zoom_mappings';
            $telemed_query = "SELECT {$zoom_mapping_table}.* FROM {$zoom_mapping_table} JOIN {$appointments_table} ON {$appointments_table}.id = {$zoom_mapping_table}.appointment_id WHERE 0=0 AND {$zoom_mapping_table}.appointment_id IN ({$all_appointment_id})";
            $telemed_data = collect($this->db->get_results($telemed_query))->keyBy('appointment_id')->toArray();
        }
        if(isKiviCareGoogleMeetActive()){
            $googlemeet_mapping_table = $this->db->prefix.'kc_appointment_google_meet_mappings';
            $telemed_query = "SELECT {$googlemeet_mapping_table}.* FROM {$googlemeet_mapping_table} JOIN {$appointments_table} ON {$appointments_table}.id = {$googlemeet_mapping_table}.appointment_id WHERE 0=0 AND {$googlemeet_mapping_table}.appointment_id IN ({$all_appointment_id})";
            $googlemeet_data = collect($this->db->get_results($telemed_query))->keyBy('appointment_id')->toArray();
        }
        foreach ( $appointments as $key => $appointment ) {

            $new_appointments[ $key ]['id']                     = (int)$appointment->id;
            $new_appointments[ $key ]['date']                   = $appointment->appointment_start_date . ' ' . $appointment->appointment_start_time;
            $new_appointments[ $key ]['endDate']                = $appointment->appointment_end_date . ' ' . $appointment->appointment_end_time;
            $new_appointments[ $key ]['appointment_start_date'] = $appointment->appointment_start_date;
            $new_appointments[ $key ]['appointment_start_time'] = date( 'h:i A', strtotime( $appointment->appointment_start_time ) );
            $new_appointments[ $key ]['visit_type']             = $appointment->visit_type;
            $new_appointments[ $key ]['description']            = $appointment->description;
            $new_appointments[ $key ]['title']                  = ($current_user_role === $this->getPatientRole()) ? $appointment->doctor_name : $appointment->patient_name;
            $new_appointments[ $key ]['clinic_id']              = [
                'id'    => (int)$appointment->clinic_id,
                'label' => $appointment->clinic_name
            ];
            $new_appointments[ $key ]['doctor_id']              = [
                'id'    => (int)$appointment->doctor_id,
                'label' => $appointment->doctor_name
            ];
            $new_appointments[ $key ]['patient_id']             = [
                'id'   => (int)$appointment->patient_id,
                'label' => $appointment->patient_name
            ];
            $new_appointments[ $key ]['clinic_name']            = $appointment->clinic_name;
            $new_appointments[ $key ]['doctor_name']            = $appointment->doctor_name;
            $new_appointments[ $key ]['status']                 = $appointment->status;
            $new_appointments[ $key ]['all_services']= !empty($all_service_results[$appointment->id]->service_name) ? $all_service_results[$appointment->id]->service_name : '';
            $new_appointments[ $key ]['color'] = '#3490dc';
            if ( $appointment->status === '0' ) {
                $new_appointments[ $key ]['color'] = '#f5365c';
            } elseif ($appointment->status === '3') {
                $new_appointments[ $key ]['color'] = '#23a359';
            }elseif ($appointment->status === '4') {
                $new_appointments[ $key ]['color'] = '#f68685';
            }
            $new_appointments[ $key ]['telemed_service'] = false;
            $new_appointments[ $key ]['telemed_meeting_link'] = '';
            if(!empty($telemed_data[$appointment->id])){
                $new_appointments[ $key ]['telemed_service'] = true;
                if(in_array($current_user_role,[$this->getPatientRole(),$this->getDoctorRole()])){
                    if($this->getLoginUserRole() === $this->getPatientRole()){
                        $new_appointments[ $key ]['telemed_meeting_link'] = $telemed_data[$appointment->id]->join_url;
                    }else{
                        $new_appointments[ $key ]['telemed_meeting_link'] = $telemed_data[$appointment->id]->start_url;
                    }
                }
            }
            if(!empty($googlemeet_data[$appointment->id])){
                $new_appointments[ $key ]['telemed_service'] = true;
                if(in_array($current_user_role,[$this->getPatientRole(),$this->getDoctorRole()])){
                    $new_appointments[ $key ]['telemed_meeting_link'] = $googlemeet_data[$appointment->id]->url;
                }
            }

        }

		echo json_encode( [
			'status'  => true,
			'message' => esc_html__('Appointment list','kc-lang'),
			'data'    => $new_appointments
		] );
	}

	public function save() {

		global $wpdb;

		if ( ! kcCheckPermission( 'appointment_add' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => $this->permission_message,
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();
		$rules = [
			'appointment_start_date' => 'required',
			'appointment_start_time' => 'required',
			'clinic_id'              => 'required',
			'doctor_id'              => 'required',
			'patient_id'             => 'required',
			'status'                 => 'required',
		];

		$message = [
			'status'     => esc_html__('Status is required', 'kc-lang'),
			'patient_id' => esc_html__('Patient is required','kc-lang'),
			'clinic_id'  => esc_html__('Clinic is required','kc-lang'),
			'doctor_id'  => esc_html__('Doctor is required','kc-lang'),
		];

		$errors = kcValidateRequest( $rules, $request_data, $message );

		if ( count( $errors ) ) {
			echo json_encode( [
				'status'  => false,
				'message' => $errors[0]
			] );
			die;
		}

        $proPluginActive = isKiviCareProActive();
        $telemedZoomPluginActive = isKiviCareTelemedActive();
        $telemedGooglemeetPluginActive = isKiviCareGoogleMeetActive();

        //check if service is single or multiple, if single create array
        if(empty(array_filter($request_data['visit_type'], 'is_array'))){
            $request_data['visit_type'] = [$request_data['visit_type']];
        };

        $current_user_role = $this->getLoginUserRole();
        if($proPluginActive){
            if($current_user_role == $this->getClinicAdminRole()){
                $request_data['clinic_id']['id'] = kcGetClinicIdOfClinicAdmin();
            }elseif ($current_user_role == $this->getReceptionistRole()) {
                $request_data['clinic_id']['id'] = kcGetClinicIdOfReceptionist();
            }
        }
        $notification = '';
        $appointment_day = esc_sql(strtolower(date('l', strtotime($request_data['appointment_start_date'])))) ;
        $day_short = esc_sql(substr($appointment_day, 0, 3));
        $query = "SELECT time_slot FROM {$wpdb->prefix}kc_clinic_sessions  WHERE `doctor_id` = ".(int)$request_data['doctor_id']['id']." AND `clinic_id` = ".(int)$request_data['clinic_id']['id']."  AND ( `day` = '{$day_short}' OR `day` = '{$appointment_day}') ";
        $clinic_session_time_slots = $wpdb->get_var($query);
		$time_slot             = !empty($clinic_session_time_slots) ? $clinic_session_time_slots : 15;
		$end_time             = strtotime( "+" . $time_slot . " minutes", strtotime($request_data['appointment_start_time'])  );
		$appointment_end_time = date( 'H:i:s', $end_time );
		$appointment_date     = date( 'Y-m-d', strtotime( $request_data['appointment_start_date'])  );
        $appointment_start_date = esc_sql($appointment_date);
        $appointment_start_time = esc_sql(date( 'H:i:s', strtotime( $request_data['appointment_start_time']) ) );
		$appointment_status = esc_sql($request_data['status']);
        $temp = [
			'appointment_start_date' => $appointment_start_date,
			'appointment_start_time' => $appointment_start_time,
			'appointment_end_date'   => esc_sql($appointment_date),
			'appointment_end_time'   => esc_sql($appointment_end_time),
			'clinic_id'              => (int)$request_data['clinic_id']['id'],
			'doctor_id'              => (int)$request_data['doctor_id']['id'],
			'patient_id'             => (int)$request_data['patient_id']['id'],
			'description'            => esc_sql($request_data['description']),
			'status'                 => $appointment_status,
		];

        $appointment_table_name = $this->db->prefix . 'kc_appointments';
        if(isset($request_data['file']) && is_array($request_data['file']) && count($request_data['file']) > 0){
            kcUpdateFields($appointment_table_name,[ 'appointment_report' => 'longtext NULL']);
            $temp['appointment_report'] = json_encode($request_data['file']);
        }

		$appointment = new KCAppointment();
        $oldMappingServiceDelete = false;
		if ( !empty( $request_data['id'] )) {
            $appointment_id = (int)$request_data['id'];
            $beforeUpdateAppointmentData =  $this->db->get_row("SELECT * FROM {$appointment_table_name} WHERE id={$appointment_id}");
            $appointment->update( $temp, array( 'id' => (int)$request_data['id'] ) );
			(new KCPatientEncounter())->update([
				'encounter_date' => $appointment_date,
				'patient_id'             => (int)$request_data['patient_id']['id'],
				'doctor_id'              => (int)$request_data['doctor_id']['id'],
				'clinic_id'              => (int)$request_data['clinic_id']['id'],
				'description'            => esc_sql($request_data['description']),
			], ['appointment_id' => $appointment_id]);
            if (isset($request_data['custom_fields_data']) && $request_data['custom_fields_data'] !== []) {
                kvSaveCustomFields('appointment_module',$appointment_id, $request_data['custom_fields_data']);
            }
			$message = esc_html__('Appointment has been updated successfully', 'kc-lang');
            $reminder_setting = get_option(KIVI_CARE_PREFIX . 'email_appointment_reminder', true);
            if(gettype($reminder_setting) !== 'boolean' && !empty($reminder_setting['time'])){
                $msg_reminder_table = $wpdb->prefix . "kc_appointment_reminder_mapping";
                $temp = [
                    'sms_status' => 0,
                    'email_status' => 0,
                    'whatsapp_status'=> 0
                ];
                $wpdb->update($msg_reminder_table, $temp, ['id' => (int)$request_data['id']]);
            }
            if(!empty($appointment_id) && $appointment_id !== 0) {
                // hook for appointment update
                do_action( 'kc_appointment_update', $appointment_id );
            }

            if($beforeUpdateAppointmentData->appointment_start_date == $appointment_start_date
              && $beforeUpdateAppointmentData->appointment_start_time == $appointment_start_time &&
                $beforeUpdateAppointmentData->status == $appointment_status){
                echo json_encode([
                    'status'  => true,
                    'message' => $message,
                    'notification' =>$notification,
                ]);
                die;
            }

		} else {

			$temp['created_at'] = current_time('Y-m-d H:i:s');
			$appointment_id = (int)$appointment->insert( $temp );

			// if appointment is not successfully created. (WP Error handle) 
			if(is_wp_error($appointment_id) || empty($appointment_id) ) {
				$message = __('Appointment booking Failed. Please try again.','kc-lang');
				echo json_encode([
					'status'  => false,
					'message' => $message,
				]); 
				wp_die();
			}
            $message = esc_html__('Appointment is Successfully booked.','kc-lang');
			if (isset($request_data['custom_fields_data']) && $request_data['custom_fields_data'] !== []) {
                kvSaveCustomFields('appointment_module',$appointment_id, $request_data['custom_fields_data']);
            }
		}

        $telemed_service_include = false;
		if (gettype($request_data['visit_type']) === 'array') {

			foreach ($request_data['visit_type'] as $key => $value) {

			    $service = strtolower($value['name']);

				// generate zoom link request (Telemed AddOn filter) 
			    if ($service == 'telemed') {

                    if ($telemedZoomPluginActive || $telemedGooglemeetPluginActive) {

                        $request_data['appointment_id'] = $appointment_id;
                        $request_data['time_slot'] = $time_slot;

                        if(kcCheckDoctorTelemedType($appointment_id) == 'googlemeet'){
                            $res_data = apply_filters('kcgm_save_appointment_event', ['appoinment_id' => $appointment_id,'service' => kcServiceListFromRequestData($request_data)]);
                        }else{
                            $res_data = apply_filters('kct_create_appointment_meeting', $request_data);
                        }
						// if zoom meeting is not created successfully
						if(empty($res_data['status'])) {
                            if(empty($request_data['id'])){
                                ( new KCAppointmentServiceMapping() )->delete( [ 'appointment_id' => (int)$appointment_id] );
                                ( new KCAppointment() )->delete( [ 'id' =>  (int)$appointment_id] );
                            }
							echo json_encode([
								'status'  => false,
								'message' => esc_html__('Failed to generate Video Meeting.', 'kc-lang'),
                                'error' => $res_data
							]); wp_die();
						}
                        $telemed_service_include = true;
                    }
                }

				if(!empty($appointment_id) && $appointment_id !== 0){
                    if(!$oldMappingServiceDelete){
                        ( new KCAppointmentServiceMapping() )->delete( [ 'appointment_id' => $appointment_id ] );
                        $oldMappingServiceDelete = true;
                    }
					(new KCAppointmentServiceMapping())->insert([
						'appointment_id' => (int)$appointment_id,
						'service_id' => (int)$value['service_id'],
						'created_at' => current_time('Y-m-d H:i:s'),
						'status'=> 1
					]);
				}
			}

		}

        if ( in_array((string)$request_data['status'],['2','4']) ) {
            KCPatientEncounter::createEncounter($appointment_id);
            KCBillItem::createAppointmentBillItem($appointment_id);
        }

        if(!empty($appointment_id) && $appointment_id !== 0) {
            // hook for appointment booked
            do_action( 'kc_appointment_book', $appointment_id );
        }

        switch($request_data['payment_mode']){
            case 'paymentWoocommerce':
                $woocommerce_response  = kcWoocommerceRedirect($appointment_id, $request_data);
                if(isset($woocommerce_response['status']) && $woocommerce_response['status']) {
                    if(!empty($woocommerce_response['woocommerce_cart_data'])) {
                        echo json_encode($woocommerce_response);
                        wp_die();
                    }
                }
                break;
            case 'paymentPaypal':
                $this->db->update($this->db->prefix."kc_appointments",['status' => 0],['id' => $appointment_id]);
                $paypal_response = (new KCPaymentController())->makePaypalPayment($request_data,$appointment_id);
                if(empty($paypal_response['status'])) {
                    (new KCAppointment())->delete(['id' => $appointment_id]);
                    (new KCAppointmentServiceMapping())->delete(['appointment_id' => $appointment_id]);
                }
                $paypal_response['appointment_id'] = $appointment_id;
                $paypal_response['data'] = $request_data;
                echo json_encode($paypal_response);
                wp_die();
                break;
            case 'paymentOffline':
                if($request_data['status'] != 0){
                    if($proPluginActive || $telemedZoomPluginActive || $telemedGooglemeetPluginActive){
                        $notification = kcProAllNotification($appointment_id,$request_data,$telemed_service_include);
                    }else{
                        $notification = kivicareCommonSendEmailIfOnlyLitePluginActive($request_data,$appointment_id);
                    }
                }
                break;
        }
		if(!empty($appointment_id) && $appointment_id !== 0) {

			echo json_encode([
				'status'  => true,
				'message' => $message,
                'notification' =>$notification,
			]); 
		} else {
			$message = esc_html__('Appointment booking Failed. Please try again.', 'kc-lang');
			echo json_encode([
				'status'  => false,
				'message' => $message,
                'notification' =>$notification,
			]); 
		}
        wp_die();
		
	}

	public function delete() {

		if ( ! kcCheckPermission( 'appointment_delete' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => $this->permission_message,
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();

		try {

			if ( ! isset( $request_data['id'] ) ) {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}

			$id = (int)$request_data['id'];
            $appointmentData   = ( new KCAppointment() )->get_by(['id' => $id ], '=', true);
            //appointment cancel mail to patient
            kcAppointmentCancelMail($appointmentData);

            //zoom telemed entry delete
            if (isKiviCareTelemedActive()) {
                apply_filters('kct_delete_appointment_meeting', $request_data);
            }

            //google calendar event delete
            if(kcCheckGoogleCalendarEnable()){
                apply_filters('kcpro_remove_appointment_event', ['appoinment_id'=>$id]);
            }

            //googlemeet telemed entry delete
            if(isKiviCareGoogleMeetActive()){
                apply_filters('kcgm_remove_appointment_event',['appoinment_id' => $id]);
            }

            //appointment encounter delete
            $this->appointmentEncounterDelete($id);
            // hook for appointment before cancelled
            do_action( 'kc_appointment_cancel', $id );

            ( new KCAppointmentServiceMapping() )->delete( [ 'appointment_id' => $id ] );
            $results = ( new KCAppointment() )->delete( [ 'id' => $id ] );

			if ( $results ) {
                // hook for appointment after cancelled
				echo json_encode( [
					'status'  => true,
					'message' => esc_html__('Appointment is deleted successfully', 'kc-lang'),
				] );
			} else {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}

		} catch ( Exception $e ) {

			$code    = $e->getCode();
			$message = $e->getMessage();

			header( "Status: $code $message" );

			echo json_encode( [
				'status'  => false,
				'message' => $e->getMessage()
			] );
		}
	}

	public function updateStatus() {

		if ( ! kcCheckPermission( 'appointment_edit' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => $this->permission_message,
				'data'        => []
			] );
			wp_die();
		}


		$request_data = $this->request->getInputs();

		$rules  = [
			'appointment_id'     => 'required',
			'appointment_status' => 'required',

		];
		$errors = kcValidateRequest( $rules, $request_data );

		if ( count( $errors ) ) {
			echo json_encode( [
				'status'  => false,
				'message' => $errors[0]
			] );
			die;
		}


		try {

            $request_data['appointment_id'] = (int)$request_data['appointment_id'];
            $request_data['appointment_status'] = (string)$request_data['appointment_status'];
			if ( in_array($request_data['appointment_status'],[ '2','4']) ) {
				KCPatientEncounter::createEncounter( $request_data['appointment_id'] );
				KCBillItem::createAppointmentBillItem($request_data['appointment_id']);
			}
			if (in_array($request_data['appointment_status'] ,['3','0']) ) {
				KCPatientEncounter::closeEncounter( $request_data['appointment_id'],$request_data['appointment_status'] );
			}

            ( new KCAppointment() )->update( [ 'status' => $request_data['appointment_status'] ], array( 'id' => $request_data['appointment_id'] ) );

            do_action( 'kc_appointment_status_update', $request_data['appointment_id'] , $request_data['appointment_status'] );

			echo json_encode( [
				'status'  => true,
				'message' => esc_html__('Appointment status is updated successfully', 'kc-lang')
			] );


		} catch ( Exception $e ) {

			$code    = $e->getCode();
			$message = $e->getMessage();

			header( "Status: $code $message" );

			echo json_encode( [
				'status'  => false,
				'message' => $message
			] );
		}
	}

	public function getAppointmentSlots() {

		$request_data = $this->request->getInputs();

		if($this->getLoginUserRole() == $this->getDoctorRole()) {
			$request_data['doctor_id'] = get_current_user_id();
		}

        if(isKiviCareProActive()){
            if($this->getLoginUserRole() == $this->getReceptionistRole()) {
                $request_data['clinic_id'] = kcGetClinicIdOfReceptionist();
            }
            if($this->getLoginUserRole() == $this->getClinicAdminRole()){
                $request_data['clinic_id'] = kcGetClinicIdOfClinicAdmin();
            }
        }

		$rules = [
			'date'      => 'required',
			'clinic_id' => 'required',
			'doctor_id' => 'required',

		];

		$message = [
			'clinic_id' => esc_html__('Clinic is required', 'kc-lang'),
			'doctor_id' => esc_html__('Doctor is required', 'kc-lang'),
		];

		$errors = kcValidateRequest( $rules, $request_data, $message );

		if ( count( $errors ) ) {
			echo json_encode( [
				'status'  => false,
				'message' => $errors[0]
			] );
			die;
		}

		try {

			$slots = kvGetTimeSlots( $request_data );
			echo json_encode( [
				'status'  => true,
				'message' => esc_html__('Appointment slots', 'kc-lang'),
				'data'    => $slots
			] );
			
		} catch ( Exception $e ) {

			$code    = $e->getCode();
			$message = $e->getMessage();

			header( "Status: $code $message" );

			echo json_encode( [
				'status'  => false,
				'message' => $message
			] );
		}
	}

	public function getAppointmentQueue() {
		$request_data = $this->request->getInputs();
		$filterData = isset( $request_data['filterData'] ) ? stripslashes( $request_data['filterData'] ) : [];
		$filterData =  json_decode($filterData, true);
		$request_data['filterData'] =  kcRecursiveSanitizeTextField($filterData);
		$proPluginActive =isKiviCareProActive();
		$appointments_table = $this->db->prefix . 'kc_appointments';
		$users_table   = $this->db->base_prefix . 'users';
		$clinics_table = $this->db->prefix . 'kc_clinics';
		$static_data   = $this->db->prefix . 'kc_static_data';
        $encounter_table = $this->db->prefix . 'kc_patient_encounters';
        $data_filter = '';
		if (isset( $request_data['start']) && isset( $request_data['end'])) {
            $start_date = esc_sql(( new DateTime( $request_data['start'] ) )->format( 'Y-m-d' ));
            $end_date =  esc_sql(( new DateTime( $request_data['end'] ) )->format( 'Y-m-d' ));
    		$data_filter  = " AND {$appointments_table}.appointment_start_date BETWEEN '{$start_date}' AND '{$end_date}' "  ;
        } elseif ( !empty($request_data['filterData']['date']) && !empty($request_data['filterData']['status']) && $request_data['filterData']['status'] !== 'all') {
			if(!empty($request_data['filterData']['date']['start']) && !empty($request_data['filterData']['date']['end'])) {
				$start_date =  esc_sql(( new DateTime( $request_data['filterData']['date']['start'] ) )->format( 'Y-m-d' ));
				$end_date =  esc_sql(( new DateTime( $request_data['filterData']['date']['end'] ) )->format( 'Y-m-d' ));
				$data_filter  = " AND {$appointments_table}.appointment_start_date BETWEEN '{$start_date}' AND '{$end_date}' "  ;
			} else {
				if(isset($request_data['filterData']['status']) && $request_data['filterData']['status'] == 1){
					$date =  esc_sql(( new DateTime( $request_data['filterData']['date'] ) )->format( 'Y-m-d' ));
					$data_filter  = " AND {$appointments_table}.appointment_start_date >= '{$date}' " ;
				}
			}
        }
        elseif(!empty($request_data['filterData']['status']) && $request_data['filterData']['status'] == 'past'){
            $data_filter  = " AND {$appointments_table}.appointment_start_date < CURDATE() " ;
        } elseif (isset($request_data['filterData']['date']) && $request_data['filterData']['date']!= null) {
            if(!empty($request_data['filterData']['date']['start']) && !empty($request_data['filterData']['date']['end'])) {
                $start_date =  esc_sql(( new DateTime( $request_data['filterData']['date']['start'] ) )->format( 'Y-m-d' ));
                $end_date =  esc_sql(( new DateTime( $request_data['filterData']['date']['end'] ) )->format( 'Y-m-d' ));
                $data_filter  = " AND {$appointments_table}.appointment_start_date BETWEEN '{$start_date}' AND '{$end_date}' "  ;
            }else{
                $date =  esc_sql(( new DateTime( $request_data['filterData']['date'] ) )->format( 'Y-m-d' ));
                $data_filter  = " AND {$appointments_table}.appointment_start_date = '{$date}' " ;
            }
        }

		$query = " SELECT DISTINCT {$appointments_table}.*,  
		       doctors.display_name  AS doctor_name,
		       patients.display_name AS patient_name,
		       static_data.label AS type_label,
		       CONCAT({$clinics_table}.address, ', ', {$clinics_table}.city,', '
		           ,{$clinics_table}.postal_code,', ',{$clinics_table}.country) AS clinic_full_address,
		       {$clinics_table}.name AS clinic_name,
               {$clinics_table}.extra AS clinic_extra,
               {$encounter_table}.id as encounter_id
			FROM  {$appointments_table}
		       LEFT JOIN {$users_table} doctors
		              ON {$appointments_table}.doctor_id = doctors.id
			   LEFT JOIN {$encounter_table}
			          ON {$appointments_table}.id={$encounter_table}.appointment_id
		       LEFT JOIN {$users_table} patients
		              ON {$appointments_table}.patient_id = patients.id
		       LEFT JOIN {$clinics_table}
		              ON {$appointments_table}.clinic_id = {$clinics_table}.id
		       LEFT JOIN {$static_data} static_data
		              ON {$appointments_table}.visit_type = static_data.value
			WHERE 0 = 0 " . $data_filter;


        $current_login_user_role = $this->getLoginUserRole();
        $current_login_user_id = get_current_user_id();

        if($this->getClinicAdminRole() == $current_login_user_role){
            $query .= " AND {$appointments_table}.clinic_id = " . kcGetClinicIdOfClinicAdmin();
        }elseif( $this->getReceptionistRole() == $current_login_user_role){
            $query .= " AND {$appointments_table}.clinic_id = " . kcGetClinicIdOfReceptionist();
        }else if ($this->getDoctorRole() == $current_login_user_role ) {
			$query .= " AND {$appointments_table}.doctor_id = " . $current_login_user_id;
		} elseif ($this->getPatientRole() == $current_login_user_role ) {
			$query .= " AND {$appointments_table}.patient_id = " . $current_login_user_id;
		}

        if($this->getDoctorRole() !== $current_login_user_role){
            if(!empty($filterData['doctor_id']['id'])){
                $query .= " AND {$appointments_table}.doctor_id = " . (int)$filterData['doctor_id']['id'];
            }
        }
		
		if ( isset( $filterData['patient_id']['id'] ) && $filterData['patient_id']['id'] !== null ) {
			$query = $query . " AND {$appointments_table}.patient_id = " . (int)$filterData['patient_id']['id'];
		}

		if ( isset( $request_data['filterData']['visit_type'] ) && $request_data['filterData']['visit_type']['id'] !== null ) {
            $request_data['filterData']['visit_type']['id'] = esc_sql($request_data['filterData']['visit_type']['id']);
			$query = $query . " AND {$appointments_table}.visit_type = '{$request_data['filterData']['visit_type']['id']}' ";
		}

		if(isset($request_data['filterData']['clinic_id']['id']) && isKiviCareProActive() ){
			if($request_data['filterData']['clinic_id']['id'] != 0){
                $request_data['filterData']['clinic_id']['id'] = (int)$request_data['filterData']['clinic_id']['id'];
                $query = $query . " AND {$appointments_table}.clinic_id = '{$request_data['filterData']['clinic_id']['id']}' ";
			}
		}
        if ( isset( $filterData['status'])  &&  $filterData['status'] != ""  ) {

            if (!empty($filterData['status']) && ((int)$filterData['status'] === -1 || $filterData['status'] === "all") ) {

				$query = $query . " ORDER BY {$appointments_table}.appointment_start_date ASC, {$appointments_table}.appointment_start_time ASC";
			
			}  elseif (!empty($filterData['status']) && $filterData['status'] == "past") {

                $query = $query . " ORDER BY {$appointments_table}.appointment_start_date ASC , {$appointments_table}.appointment_start_time DESC";

            } elseif (isset($filterData['status']['value'])) {
                $filterData['status']['value'] = esc_sql($filterData['status']['value']);
                $query = $query . " AND {$appointments_table}.status = {$filterData['status']['value']} ORDER BY {$appointments_table}.appointment_start_date ASC , {$appointments_table}.appointment_start_time ASC";
            } else {
				if(!empty($filterData['status']) && $filterData['status'] == 1) {
					$query = $query . " AND {$appointments_table}.status  IN(1,4) ORDER BY {$appointments_table}.appointment_start_date ASC , {$appointments_table}.appointment_start_time ASC";
				}else{
                    $filterData['status'] = esc_sql($filterData['status']);
					$query = $query . " AND {$appointments_table}.status = {$filterData['status']} ORDER BY {$appointments_table}.appointment_start_date ASC , {$appointments_table}.appointment_start_time ASC";
				}
			}

		}else{
            $query = $query . " ORDER BY {$appointments_table}.appointment_start_date ASC ,  {$appointments_table}.appointment_start_time ASC";
        }

		$counts = $this->db->get_results($query);
		$count = !empty($counts) && count($counts) > 0 ? count($counts) : 0;
        $paginationCondition = ' ';
        if(!empty($filterData['pagination'])){
            $limit = 10 ;
            $offset = ((int)$filterData['pagination'] - 1) * $limit;
            $paginationCondition = " LIMIT {$limit} OFFSET {$offset} ";
        }

        $query = $query . $paginationCondition;

		$appCollection = collect( $this->db->get_results( $query, OBJECT ) )->unique( 'id' );

       
        if (count($appCollection)) {

            $appointment_ids = $appCollection->pluck('id')->implode(',');

            $zoom_mappings = apply_filters('kct_get_meeting_list', [
                'appointment_ids' => $appointment_ids
            ]);

            if (isset($zoom_mappings['appointment_ids'])) {
                $zoom_mappings = collect([]);
            }


//            $telemedPluginActive = isKiviCareTelemedActive();
            $googleMeetPluginActive = isKiviCareGoogleMeetActive();
            $is_patient_enable = in_array((string)get_option( KIVI_CARE_PREFIX . 'patient_cal_setting',true),['1','true']) ? 'on' : 'off';
            $enableAppointmentDescription = get_option(KIVI_CARE_PREFIX.'appointment_description_config_data');
            $enableAppointmentDescription = gettype($enableAppointmentDescription) == 'boolean' ? 'on' : $enableAppointmentDescription;
            $currency_detail = kcGetClinicCurrenyPrefixAndPostfix();
            $currency_prefix = !empty($currency_detail['prefix']) ? $currency_detail['prefix'] : '' ;
            $currency_postfix = !empty($currency_detail['postfix']) ? $currency_detail['postfix'] : '' ;
            $current_date = current_time("Y-m-d H:i:s");

            $appointments = $appCollection->map(function ($appointment) use ($current_date,$currency_prefix,$currency_postfix,$zoom_mappings,$is_patient_enable,$current_login_user_role,$googleMeetPluginActive,$proPluginActive,$enableAppointmentDescription) {

                $appointments_table = $this->db->prefix . 'kc_appointments';
                $appointments_service_table = $this->db->prefix . 'kc_appointment_service_mapping';
                $service_table = $this->db->prefix . 'kc_services';
                $service_doctor_table = $this->db->prefix . 'kc_service_doctor_mapping';

                $appointment->appointment_start_time = date('h:i A', strtotime($appointment->appointment_start_time));
                $appointment->appointment_end_time = date('h:i A', strtotime($appointment->appointment_end_time));
                $patient_mobile_number = ' -';
                $patient_user_meta = get_user_meta( $appointment->patient_id, 'basic_data', true );
                if(!empty($patient_user_meta)){
                    $patient_user_meta = json_decode($patient_user_meta);
                    $patient_mobile_number = (!empty($patient_user_meta->mobile_number) ? $patient_user_meta->mobile_number : '  - ');
                }
                $appointment->patient_contact_no = $patient_mobile_number;
                $appointment->clinic_id = [
                    'id' => $appointment->clinic_id,
                    'label' => $appointment->clinic_name
                ];
                $appointment->doctor_id = [
                    'id' => $appointment->doctor_id,
                    'label' => $appointment->doctor_name,
                ];
                $appointment->patient_id = [
                    'id' => $appointment->patient_id,
                    'label' => $appointment->patient_name
                ];

                if(!empty($appointment->encounter_id)){
                    $appointment->encounter_detail = $this->db->get_row("SELECT * FROM {$this->db->prefix}kc_patient_encounters WHERE id={$appointment->encounter_id}");
                }
                $zoom_data = $zoom_mappings->where('appointment_id', (int)$appointment->id)->first();

                if ($googleMeetPluginActive) {
                    $googlemeet_data = $this->db->get_var("SELECT url FROM {$this->db->prefix}kc_appointment_google_meet_mappings WHERE appointment_id=" . (int)$appointment->id);
                    if (!empty($googlemeet_data)) {
                        if(empty($zoom_data)){
                            $zoom_data = (object)[];
                        }
                        $zoom_data->join_url =  $zoom_data->start_url =  $googlemeet_data;
                    }
                }

                $appointment->zoom_data = $zoom_data;
                $appointment->clinic_prefix = $currency_prefix;
                $appointment->clinic_postfix = $currency_postfix;
                $appointment->video_consultation = !empty($zoom_data);

                $get_service_query = "SELECT {$appointments_table}.id,{$service_table}.name AS service_name,{$service_table}.id AS service_id,{$service_doctor_table}.charges FROM {$appointments_table}
				LEFT JOIN {$appointments_service_table} ON {$appointments_table}.id = {$appointments_service_table}.appointment_id JOIN {$service_table} 
				ON {$appointments_service_table}.service_id = {$service_table}.id JOIN {$service_doctor_table} ON {$service_doctor_table}.service_id ={$service_table}.id and {$service_doctor_table}.doctor_id={$appointment->doctor_id['id']} and {$appointments_service_table}.appointment_id = {$appointment->id}  WHERE 0 = 0";
                $services = $this->db->get_results($get_service_query, OBJECT);
                $service_array = $service_list = [];
                $service_charges = 0;

                foreach ($services as $service) {
                    $service_array[] = $service->service_name;
                    $service_list[] = [
                        'service_id' => $service->service_id,
                        'name' => $service->service_name,
                        'charges' => $service->charges
                    ];
                    $service_charges += (int)$service->charges;
                }

                $appointment->descriptionEnable = $enableAppointmentDescription;
                $appointment->all_service_charges = $service_charges;
                $appointment->all_services = implode(", ", $service_array);
                $appointment->visit_type_old = $appointment->visit_type;
                $appointment->visit_type = $service_list;

                $custom_field_data = [];
                if($proPluginActive){
                    $custom_field_data = kcGetCustomFields('appointment_module', $appointment->id, (int)$appointment->doctor_id['id']);
                }
                $appointment->custom_fields = $custom_field_data;

                if (!empty($appointment->appointment_report)) {
                    if (is_array(json_decode($appointment->appointment_report)) && count(json_decode($appointment->appointment_report)) > 0)
                        $appointment->appointment_report = array_map(function ($v) {
                            return wp_get_attachment_url($v);
                        }, json_decode($appointment->appointment_report));
                }

                if ($appointment->status == '1' && $proPluginActive &&
                    $is_patient_enable == 'on' && $this->getPatientRole() === $current_login_user_role ) {

                    $appointment_data = [
                        "clinic_name" => $appointment->clinic_name,
                        "clinic_address" => $appointment->clinic_full_address,
                        "id" => $appointment->id,
                        "start_date" => $appointment->appointment_start_date,
                        "start_time" => $appointment->appointment_start_time,
                        "end_date" => $appointment->appointment_end_date,
                        "end_time" => $appointment->appointment_end_time,
                        "appointment_service" => $appointment->all_services,
                        "extra" => $appointment
                    ];

                    $appointment->calendar_content = kcAddToCalendarContent($appointment_data);
                }

                $date = date("Y-m-d H:i:s", strtotime(date("c", strtotime($appointment->appointment_start_date . $appointment->appointment_end_time))));
                $appointment->isEditAble = $date > $current_date;
                return $appointment;

            })->values();

        } else {
            $appointments = [];
        }
		
		echo json_encode( [
			'status'  => true,
			'message' => esc_html__('Appointments', 'kc-lang'),
			'data'    => $appointments,
            'total_rows' => $count,
			'nextPage' => (!empty($request_data['page']) ? (int)$request_data['page'] + 1 :  1 ),
		] );

	}

	public function allAppointment() {

		$request_data = $this->request->getInputs();
		$condition    = '';
		$appointments_table = $this->db->prefix . 'appointments';

		$users_table   = $this->db->base_prefix . 'users';
		$static_data_table  = $this->db->prefix . 'static_data';

		if ( $request_data['searchKey'] && $request_data['searchValue']) {
            $request_data['searchKey'] = esc_sql($request_data['searchKey']);
            $request_data['searchValue'] = esc_sql($request_data['searchValue']);
			$condition = " WHERE {$request_data['searchKey']} LIKE  '%{$request_data['searchValue']}%' ";
		}

        $request_data['limit'] = (int)$request_data['limit'];
        $request_data['offset'] = (int)$request_data['offset'];
		$query = "
			SELECT {$appointments_table}.*,
		       doctors.display_name  AS doctor_name,
		       patients.display_name AS patient_name
		       
			FROM  {$appointments_table}
		       LEFT JOIN {$users_table} doctors
		            ON {$appointments_table}.doctor_id = doctors.id
		       LEFT JOIN {$users_table} patients
		            ON {$appointments_table}.patient_id = patients.id
		       {$condition}  
		       ORDER BY {$appointments_table}.appointment_start_date DESC LIMIT {$request_data['limit']} OFFSET {$request_data['offset']}";

		$appointments = collect( $this->db->get_results( $query, OBJECT ) )->unique( 'id' );

		$appointment_count_query = "SELECT count(*) AS count FROM {$appointments_table}";

		$total_appointment = $this->db->get_results( $appointment_count_query, OBJECT );

		if ( $request_data['searchKey'] && $request_data['searchValue'] ) {
			$total_rows = count( $appointments );
		} else {
			$total_rows = $total_appointment[0]->count;
		}

		if ( $total_rows < 0 ) {
			echo json_encode( [ 		
				'status'  => false,
				'message' => esc_html__('No appointment found', 'kc-lang'),
				'data'    => []
			] );
			wp_die();
		}

		$visit_type_data =  $appointments->pluck('visit_type')->unique()->implode("','");

		$static_data_query = " SELECT * FROM $static_data_table WHERE value IN ('$visit_type_data') ";

		$static_data = collect($this->db->get_results( $static_data_query, OBJECT ))->pluck('label','value')->toArray();

		foreach ($appointments as $key => $appointment) {
			$appointment->type_label = $static_data[$appointment->visit_type];
		}

		echo json_encode( [
			'status'  => true,
			'message' => esc_html__('Appointments', 'kc-lang'),
			'data'    => $appointments,
			'total_rows' => $total_rows
		] );
	}

	public function delete_multiple(){

		if ( ! kcCheckPermission( 'appointment_delete' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => $this->permission_message,
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();
		try {
			if(!empty($request_data)){
                $results = false;
				foreach($request_data['id'] as $id){
					if(!empty($id)){
                        $id = (int)$id;
						if ( ! empty( $id ) ) {
							throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
						}

                        $appointmentData   = ( new KCAppointment() )->get_by(['id' => $id ], '=', true);
                        //APPIONTMENT CANCEL EMAIL
                        kcAppointmentCancelMail($appointmentData);
                        // cancel google calendar event
                        if(kcCheckGoogleCalendarEnable()){
                            apply_filters('kcpro_remove_appointment_event', ['appoinment_id'=>$id]);
                        }
                        // cancel google meet event
                        if(isKiviCareGoogleMeetActive()){
                            $event = apply_filters('kcgm_remove_appointment_event',['appoinment_id' => $id]);
                        }
                        if (isKiviCareTelemedActive()) {
                            //delete zoom meeting
                            apply_filters('kct_delete_appointment_meeting', $id);
                        }

                        $this->appointmentEncounterDelete($id);
                        // hook for appointment before cancelled
                        do_action( 'kc_appointment_cancel', $id );
                        // delete appointment
                        ( new KCAppointmentServiceMapping() )->delete( [ 'appointment_id' => $id ] );
                        $results = ( new KCAppointment() )->delete( [ 'id' => $id ] );
					}
				}
				if ( $results ) {
					// hook for appointment after cancelled
					echo json_encode( [
						'status'  => true,
						'message' => esc_html__('Selected Appointment is deleted successfully', 'kc-lang'),
					] );
				} else {
					throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
				}
			}

		} catch ( Exception $e ) {

			$code    = $e->getCode();
			$message = $e->getMessage();

			header( "Status: $code $message" );

			echo json_encode( [
				'status'  => false,
				'message' => $e->getMessage()
			] );
		}
	}

    public function uploadMedicalReport(){
        $parameters = $this->request->getInputs();
        $attach = [];
        $status = false;
        if(isset($parameters['file_multi']) && $parameters['file_multi'] !== '' && $parameters['file_multi'] != null ) {
            $status =true;
            array_walk($_FILES["file_multi"],function($item1, $key){
                if(is_array($item1)){
                    if($key === 'name'){
                        $item1 = array_map(function ($v){
                            return sanitize_file_name($v);
                        },$item1);
                    }
                    $supported_type = get_allowed_mime_types();
                    if($key === 'type' && !empty($supported_type) &&  is_array($supported_type)){
                        $item1 = array_map(function ($v) use($supported_type){
                            if(!in_array($v,$supported_type)){
                                echo json_encode([
                                    'status'  => false,
                                    'message' => esc_html__( "Failed to upload Medical report.File Type not supported" , 'kc-lang' ),
                                    'data'    => ''
                                ]);
                                die;
                            }
                            return $v;
                        },$item1);
                    }
                }
                return $item1;
            });
            $files = $_FILES["file_multi"];
            foreach ($files['name'] as $key => $value) {
                if ($files['name'][$key]) {
                    $file = array(
                        'name' => $files['name'][$key],
                        'type' => $files['type'][$key],
                        'tmp_name' => $files['tmp_name'][$key],
                        'error' => $files['error'][$key],
                        'size' => $files['size'][$key]
                    );
                    $_FILES = array("upload_file" => $file);
                    $attachment_id = media_handle_upload("upload_file", 0);
                    array_push($attach,$attachment_id);
                    if(is_wp_error($attachment_id)) {
                        foreach ($attach as $att){
                            wp_delete_attachment($att);
                        }
                        $attach = [];
                        echo json_encode([
                            'status'  => false,
                            'message' => esc_html__( "Failed to upload Medical report." , 'kc-lang' ),
                            'data'    => ''
                        ]);
                        die;
                    }
                }
            }
        }

        ob_start();
        ?>
        <div>
            <h6 class="iq-letter-spacing-1"><?php echo esc_html__('Uploaded files', 'kc-lang'); ?></h6>
            <div class="iq-card iq-preview-details">
                <table class="iq-table-border" style="border: 0;">
                    <?php
                    foreach ($attach as $key => $file){
                        ?>
                        <tr>
                            <td>
                                <a href="<?php echo esc_url(wp_get_attachment_url($file)); ?>" target="_blank" alt="<?php echo esc_html(get_the_title($file));?>">
                                    <i class="fas fa-external-link-alt"></i><?php echo ' '.esc_html(get_the_title($file)) ?>
                                </a>
                            </td>
                        </tr>
                        <?php
                    }
                    ?>
                </table>
            </div>
        </div>
        <?php
        $html = ob_get_clean();
        echo json_encode([
            'status'  => $status,
            'message' => $status ?  esc_html__( "Medical report uploaded successfully." , 'kc-lang' ) : esc_html__( "Failed to upload Medical report." , 'kc-lang' ) ,
            'data'    => $attach,
            'html'    => $html
        ]);
        die;

    }

    public function appointmentEncounterDelete( $id){
        $encounter_id = (new KCPatientEncounter())->get_var(['appointment_id' => $id],'id');
        if(!empty($encounter_id)){
            $encounterBill = (new KCBill())->get_var(['encounter_id' => $encounter_id],"id");
            if(!empty($encounterBill)){
                $this->db->query("DELETE FROM {$this->db->prefix}kc_bill_items WHERE bill_id IN ({$encounterBill})");
                (new KCBill())->delete(['id' => $encounterBill]);
            }
            (new KCPatientEncounter())->delete(['id' => $encounter_id]);
        }
    }
}
