<?php

namespace App\Controllers;


use App\baseClasses\KCBase;
use App\baseClasses\KCRequest;
use App\models\KCCustomField;
use Exception;

class KCCustomFieldController extends KCBase {

	public $db;

	/**
	 * @var KCRequest
	 */
	private $request;

	public function __construct() {

		global $wpdb;

		$this->db = $wpdb;

		$this->request = new KCRequest();

	}

	public function index() {

		if ( ! kcCheckPermission( 'custom_field_list' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => $this->permission_message,
				'data'        => []
			] );
			wp_die();
		}


        $conditions = ' ';
        if(!isKiviCareProActive()){
            $conditions = " WHERE module_type !='appointment_module'";
        }

		$custom_field = $this->db->get_results("SELECT * FROM {$this->db->prefix}kc_custom_fields {$conditions} ORDER BY id DESC");

		if ( ! count( $custom_field ) ) {
			echo json_encode( [
				'status'  => false,
				'message' => esc_html__('No custom fields found', 'kc-lang'),
				'data'    => []
			] );
			wp_die();
		}

		echo json_encode( [
			'status'  => true,
			'message' => esc_html__('Custom fields records', 'kc-lang'),
			'data'    => $custom_field
		] );
	}

	public function save() {

		if ( ! kcCheckPermission( 'custom_field_add' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => $this->permission_message,
				'data'        => []
			] );
			wp_die();
		}
        $request_data = $this->request->getInputs();
        $custome_field = new KCCustomField();
        $fields = [];
        $user_id = get_current_user_id();
        if ( count( $request_data['fields'] ) ) {
            foreach ( $request_data['fields'] as $key => $field ) {
                $field['name'] = $field['label'];
                $field['type'] =  $field['type']['id'];
                $field['status'] =  $field['status']['id'];
                $fields[] = $field;
            }
        }
        if($this->getLoginUserRole() == $this->getDoctorRole() && $request_data['module_type']['id'] == 'patient_module'){
            $request_data['module_id'] = $user_id;
        }else if($this->getLoginUserRole() == $this->getDoctorRole() && $request_data['module_type']['id'] == 'patient_encounter_module'){
            $request_data['module_id'] = $user_id;
        }
		else if($this->getLoginUserRole() == $this->getDoctorRole() && $request_data['module_type']['id'] == 'appointment_module'){
            $request_data['module_id'] = $user_id;
        }else{
            $request_data['module_id'] = isset($request_data['module_id']['id']) ? $request_data['module_id']['id'] :0;
        }

        $status = isset($field['status']) ? $field['status'] : 1 ;

        $temp = [
            'module_type' => $request_data['module_type']['id'],
            'module_id' => isset($request_data['module_id']['id']) ? (int)$request_data['module_id']['id'] :(int)$request_data['module_id'] ,
            'fields'      => json_encode( $fields[0] ),
            'status'      => $status
        ];

		
		if ( ! isset( $request_data['id'] ) && gettype($request_data['module_id']) !== 'array' ) {

            $temp['created_at'] = current_time( 'Y-m-d H:i:s' );
            $data = $custome_field->insert( $temp );
            $message = esc_html__('Custom fields has been saved successfully', 'kc-lang') ;

        } else {
            $message = esc_html__('Custom fields has been updated successfully', 'kc-lang') ;
            $custome_field->update( $temp, array( 'id' => (int)$request_data['id'] ) );
        }

        echo json_encode( [
            'status'  => true,
            'message' => $message
        ] );


    }

	public function edit() {
		if ( ! kcCheckPermission( 'custom_field_edit' ) || !kcCheckPermission('custom_field_view') ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => $this->permission_message,
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();

        if (!isset($request_data['id'])) {
            throw new Exception(esc_html__('Data not found', 'kc-lang'), 400);
        }

        $id = (int)$request_data['id'];

        $custom_field = (new KCCustomField())->get_by(['id' => $id], '=', true);

        if ($custom_field->module_id !== null && $custom_field->module_id !== '') {
            $user_data = get_userdata((int)$custom_field->module_id);
            if (isset($user_data->data)) {
                $custom_field->module_id = [
                    'id' => $user_data->data->ID,
                    'label' => $user_data->data->display_name
                ];
            }
        } else {
            $custom_field->module_id = '';
        }

        $fields = json_decode($custom_field->fields);

        $fields->type = [
            'id' => $fields->type,
            'label' => ucfirst($fields->type)
        ];

        $fields->status = [
            'id' => in_array($custom_field->status,[1,'1']) ? 1 : 0 ,
            'label' => in_array($custom_field->status,[1,'1']) ? 'Active' : 'Inactive'
        ];

        if ($custom_field) {

            $temp = [
                'id' => $custom_field->id,
                'module_type' => [
                    'id' => $custom_field->module_type,
                    'label' => str_replace("_", " ", $custom_field->module_type)
                ],
                'module_id' => $custom_field->module_id,
                'fields' => $fields,
                'status' => $custom_field->status
            ];
            echo json_encode([
                'status' => true,
                'message' => esc_html__('Custom field record', 'kc-lang'),
                'data' => $temp
            ]);

        } else {
            throw new Exception(esc_html__('Data not found', 'kc-lang'), 400);
        }

	}

	public function delete() {

		if ( ! kcCheckPermission( 'custom_field_delete' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => $this->permission_message,
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();

		try {

			if ( ! isset( $request_data['id'] ) ) {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}

			$id = (int)$request_data['id'];

			$results = ( new KCCustomField() )->delete( [ 'id' => $id ] );

			if ( $results ) {
				echo json_encode( [
					'status'  => true,
					'message' => esc_html__('Custom field has been deleted successfully', 'kc-lang'),
				] );
			} else {
				throw new Exception( esc_html__('Custom field delete failed', 'kc-lang'), 400 );
			}


		} catch ( Exception $e ) {

			$code    = $e->getCode();
			$message = $e->getMessage();

			header( "Status: $code $message" );

			echo json_encode( [
				'status'  => false,
				'message' => $message
			] );
		}
	}

    public function getCustomFields()
    {
        $request_data = $this->request->getInputs();
        $custom_field_table =   $this->db->prefix . 'kc_custom_fields';
        try {
            if (!isset($request_data['module_type'])) {
                throw new Exception(esc_html__('Data not found', 'kc-lang'), 400);
            }
            $user_id = get_current_user_id();
            $current_user_role = $this->getLoginUserRole();
            $request_data = $this->request->getInputs();
            $module_type = $request_data['module_type'];
            $module_id = $request_data['module_id'] ;

            if($request_data['module_type'] == 'patient_module') {
                if($current_user_role == $this->getDoctorRole()){
                    $module_id = $user_id ;
                    $module_id  = " AND module_id IN($module_id,0) ";
                }
                else{
                    $module_id = (int)$request_data['module_id'] ;
                    $module_id  = " AND module_id = {$module_id} " ;
                }
            }

            if($request_data['module_type'] === 'doctor_module') {
                $module_id = (int)$request_data['module_id'] ;
                $module_id  = " AND module_id = {$module_id} " ;
            }

            if($request_data['module_type'] == 'appointment_module'){
                if($current_user_role == $this->getDoctorRole()){
                    $module_id = $user_id ;
                    $module_id  = " AND module_id IN($module_id,0) ";
                } elseif (!empty($request_data['doctor_id'])) {
                    if(gettype($request_data['doctor_id']) === 'string') {
                        $doctor_id  = (int)$request_data['doctor_id'];
                    } else {
                        $doctor_id  = (int)$request_data['doctor_id']['id'];
                    }

                    $module_id  = " AND module_id IN ($doctor_id,0) " ;

                } else {
                    $module_id  = " AND module_id = 0 " ;
                }
            }

            if( isset($module_id) && $request_data['module_type'] !== 'patient_encounter_module'  ){
                $query = "SELECT * FROM {$custom_field_table} WHERE module_type = '{$module_type}'  $module_id" ;
            }else{
                $module_id_temp = $module_id;
                if($current_user_role == $this->getDoctorRole()){
                    $module_id = $user_id ;
                    $module_id  = " AND p.module_id IN($module_id,0) ";
                } elseif (!empty($request_data['doctor_id'])) {
                    if(gettype($request_data['doctor_id']) === 'string') {
                        $doctor_id  = (int)$request_data['doctor_id'];
                    } else {
                        $doctor_id  = (int)$request_data['doctor_id']['id'];
                    }

                    $module_id  = " AND p.module_id IN ($doctor_id,0) " ;

                } else {
                    $module_id  = " AND p.module_id = 0 " ;
                }
                $query = "SELECT p.*, u.fields_data FROM {$custom_field_table} AS p 
                 LEFT JOIN  (SELECT * FROM {$this->db->prefix}kc_custom_fields_data WHERE module_id = {$module_id_temp} ) AS u ON p.id = u.field_id 
                          WHERE p.module_type ='{$module_type}' {$module_id}";
            }
            $custom_module  = $this->db->get_results( $query );

            $fields = [] ;
            if(count($custom_module) > 0) {
                foreach ($custom_module as $key => $value) {
                    $field_data  = '' ;
                    if(!empty($value->fields_data)){
                        if(json_decode($value->fields)->type != null
                            &&  json_decode($value->fields)->type === 'checkbox'){
                            $value->fields_data = json_decode($value->fields_data);
                        }
                        $field_data = $value->fields_data ;
                    }
                    $fields[] = array_merge(json_decode($value->fields,true), ['field_data'=> $field_data], ['id'=> $value->id]);
                }
            }

            echo json_encode([
                'status' => true,
                'message' => esc_html__('Custom fields', 'kc-lang'),
                'data' => array_values($fields)
            ]);


        } catch (Exception $e) {

            $code =$e->getCode();
            $message = $e->getMessage();

            header("Status: $code $message");

            echo json_encode([
                'status' => false,
                'message' => $message
            ]);
        }

    }
}
