<?php

namespace App\controllers;

use App\baseClasses\KCBase;
use App\baseClasses\KCRequest;

class KCGeneralSettingController extends KCBase
{
    /**
     * @var KCRequest
     */
    public $db;

    private $request;

    public function __construct()
    {
        global $wpdb;

        $this->db = $wpdb;

        $this->request = new KCRequest();
    }

    public function saveRequestHelperStatus() {

        $request_data = $this->request->getInputs();
        $status = false;
        $data = '';

        if(isset($request_data['request_status'])) {
            update_option(KIVI_CARE_PREFIX.'request_helper_status',strval($request_data['request_status']));
            $status = true;
            $data = $request_data['request_status'];
        }

        echo json_encode([
            'status'  => $status,
            'data' => $data
        ]);
    }


    public function getRequestHelperStatus() {

        $request_help = get_option(KIVI_CARE_PREFIX.'request_helper_status',true);

        $request_help = gettype($request_help)  != 'boolean' && $request_help == 'on' ? 'on' : 'off';

        echo json_encode([
            'status'  => true,
            'data' => $request_help
        ] );
    }

    public function saveGoogleRecaptchaSetting(){
        $request_data = $this->request->getInputs();

        update_option(KIVI_CARE_PREFIX . 'google_recaptcha', [
                'site_key' => $request_data['site_key'],
                'secret_key' => $request_data['secret_key'],
                'status' => $request_data['status']
            ]
        );

        echo json_encode( [
            'status'  => true,
            'message' => __("Google Recaptcha Setting Saved Successfully","kc-lang"),
        ] );
        wp_die();
    }

    public function saveFullcalendarSetting(){

        $request_data = $this->request->getInputs();

        update_option(KIVI_CARE_PREFIX . 'fullcalendar_setting', !empty($request_data['fullcalendar_key']) ? $request_data['fullcalendar_key'] : '');

        echo json_encode( [
            'status'  => true,
            'message' => __("Fullcalendar Setting Setting Saved","kc-lang"),
        ] );

        wp_die();

    }

    public function saveLogoutRedirectSetting(){

        $request_data = $this->request->getInputs();

        $rules = [
            'clinic admin redirect url'    => 'required',
            'doctor redirect url'     => 'required',
            'receptionist redirect url' => 'required',
            'patient redirect url'        => 'required',
        ];

        $errors = kcValidateRequest( $rules, $request_data['data'] );

        if ( count( $errors ) ) {
            echo json_encode( [
                'status'  => false,
                'message' => $errors[0]
            ] );
            die;
        }

        update_option(KIVI_CARE_PREFIX.'logout_redirect',$request_data['data']);

        echo json_encode( [
            'status'  => true,
            'message' => __("Logout Redirect Setting Saved","kc-lang"),
        ] );
        wp_die();
    }

    public function saveClinicCurrency(){
        $request_data = $this->request->getInputs();
        $status = false;
        $message = esc_html__("Failed To update Currency Setting","kc-lang");
        if(!empty($request_data['clinic_data'])){
            update_option(KIVI_CARE_PREFIX.'clinic_currency','on');
            $currencyData = $request_data['clinic_data'];
            $currency = [
                'currency_prefix' => (!empty($currencyData['currency_prefix']) && $currencyData['currency_prefix'] !== 'null') ? $currencyData['currency_prefix'] : '',
                'currency_postfix' => (!empty($currencyData['currency_postfix']) && $currencyData['currency_postfix'] !== 'null') ? $currencyData['currency_postfix'] : '',
                // 'decimal_point' => (!empty($currencyData['decimal_point'])) ? $currencyData['decimal_point'] : array( 'id'=> '2', 'label'=>'100.00')
            ];
            global $wpdb;
            $results = $wpdb->query("UPDATE {$wpdb->prefix}kc_clinics SET extra='".json_encode($currency, JSON_UNESCAPED_UNICODE)."'");
            $status = true;
            $message = esc_html__("Currency Setting Saved","kc-lang");
        }
        echo json_encode(['status' => $status,"message" => $message]);
        die;
    }

    public function saveDateFormat(){
        $request_data = $this->request->getInputs();
        $status = false;
        $message = esc_html__("Failed To update date format Setting","kc-lang");
        if(!empty($request_data['data'])){
            update_option(KIVI_CARE_PREFIX.'date_format',$request_data['data']);
            $message = esc_html__("Date Format Setting Saved","kc-lang");
            $status = true;
        }
        echo json_encode(['status' => $status,"message" => $message]);
    }
    public function getAllGeneralSettingsData() {

        if ( !current_user_can('administrator') ) {
            echo json_encode( [
                'status'      => false,
                'status_code' => 403,
                'message'     => $this->permission_message,
                'data'        => []
            ] );
            wp_die();
        }

        $currencyDetail = kcGetClinicCurrenyPrefixAndPostfix();
        $fullCalendarSetting = get_option(KIVI_CARE_PREFIX . 'fullcalendar_setting',true);

        echo json_encode([
            'status' => true,
            'data' => [
                "currency_prefix" =>$currencyDetail['prefix'],
                "currency_postfix" => $currencyDetail['postfix'],
            ],
            'captcha_data' => [
                'site_key' => kcGoogleCaptchaData('site_key'),
                'secret_key' =>  kcGoogleCaptchaData('secret_key'),
                'status' =>   kcGoogleCaptchaData('status')
            ],
            'logout_redirect' => kcGetLogoutRedirectSetting('all'),
            'fullcalendar' => gettype($fullCalendarSetting) !== 'boolean' && !empty($fullCalendarSetting) ? $fullCalendarSetting : '',
            'date_format' => kcGetDateFormat()
        ]);
        die;
    }

    public function getClinicCurrency() {
        // $data = get_option(KIVI_CARE_PREFIX.'clinic_currency',true);
        $currencyData = [
            "currency_prefix" =>'',
            "currency_postfix" => '',
            "decimal_point" => []
        ];
        global $wpdb;
        $result = $wpdb->get_var("SELECT extra FROM {$wpdb->prefix}kc_clinics");
        if(!empty($result)){
            $result = json_decode($result);
            $currencyData['currency_prefix'] = !empty($result->currency_prefix) || $result->currency_prefix !== 'null'? $result->currency_prefix : '';
            $currencyData['currency_postfix'] = !empty($result->currency_postfix) || $result->currency_postfix !== 'null' ? $result->currency_postfix : '';
            $currencyData['decimal_point'] = !empty($result->decimal_point) ? ( gettype($result->decimal_point) == 'string' && gettype($result->decimal_point) != 'object' ? kcRecursiveSanitizeTextField(json_decode(stripslashes($result->decimal_point))) : $result->decimal_point) : '';
        }
        $captchaData = [
          'site_key' => kcGoogleCaptchaData('site_key'),
          'secret_key' =>  kcGoogleCaptchaData('secret_key'),
          'status' =>   kcGoogleCaptchaData('status')
        ];

        $fullCalendarSetting = get_option(KIVI_CARE_PREFIX . 'fullcalendar_setting',true);

        echo json_encode([
            'status' => true,
            'data' => $currencyData,
            'captcha_data' => $captchaData,
            'logout_redirect' => kcGetLogoutRedirectSetting('all'),
            'fullcalendar' => gettype($fullCalendarSetting) !== 'boolean' && !empty($fullCalendarSetting) ? $fullCalendarSetting : ''
        ]);
        die;
    }
}