<?php

namespace App\Controllers;

use App\baseClasses\KCBase;
use App\baseClasses\KCRequest;
use App\models\KCReceptionistClinicMapping;
use App\models\KCClinic;
use Exception;
use WP_User;

class KCReceptionistController extends KCBase {

	public $db;

	private $request;

	public function __construct() {
		$this->request = new KCRequest();
	}

	public function index() {

		global $wpdb;

		if (! kcCheckPermission('receptionist_list')) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have permission to access', 'kc-lang'),
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();
		$table_name = $wpdb->prefix . 'kc_receptionist_clinic_mappings';
		$receptionists_count = get_users([
			'role' => $this->getReceptionistRole(),
		] );

		$receptionists_count = count( $receptionists_count );

		$args['role']           = $this->getReceptionistRole();
		$args['number']         = $request_data['limit'];
		$args['orderby']        = 'ID';
		$args['order']          = 'DESC';
		if(current_user_can('administrator')){
			$receptionists = get_users( $args );
		}else{
			$query = "SELECT `receptionist_id` FROM {$table_name} WHERE `clinic_id` =". kcGetClinicIdOfClinicAdmin();
			$result = collect($wpdb->get_results($query))->unique('receptionist_id')->pluck('receptionist_id');
			$receptionists = get_users( $args );
			$receptionists = collect($receptionists)->whereIn('ID',$result)->toArray();
		}

		if ( ! count( $receptionists ) ) {
			echo json_encode( [
				'status'  => false,
				'message' => esc_html__('No receptionist found', 'kc-lang'),
				'data'    => []
			] );
			wp_die();
		}

		$data = [];

		foreach ( $receptionists as $key => $receptionist ) {

            $receptionist->ID = (int)$receptionist->ID;
			$user_meta = get_user_meta( $receptionist->ID, 'basic_data', true );
			$clinic_mapping = (new KCReceptionistClinicMapping())->get_by([ 'receptionist_id' => $receptionist->ID]);
            $clinic_name =  (new KCClinic())->get_by([ 'id' => (int)$clinic_mapping[0]->clinic_id]);
			$data[ $key ]['ID']              = $receptionist->ID;
			$image_attachment_id = get_user_meta($receptionist->ID,'receptionist_profile_image',true);
			$data[ $key ]['profile_image'] = (!empty($image_attachment_id) && $image_attachment_id != '') ? wp_get_attachment_url($image_attachment_id) : '';
			$data[ $key ]['display_name']    = $receptionist->data->display_name;
			$data[ $key ]['user_email']      = $receptionist->data->user_email;
			$data[ $key ]['user_status']     = $receptionist->data->user_status;
			$data[ $key ]['user_registered'] = $receptionist->data->user_registered;
			$data[$key]['clinic_id'] = $clinic_mapping[0]->clinic_id;
            $data[$key]['clinic_name'] = $clinic_name[0]->name;
            $data[$key]['user_registered_formated']= date("Y-m-d", strtotime($receptionist->data->user_registered));
            $user_deactivate = 'yes';
            $user_deactivate_status = get_user_meta((int)$receptionist->ID,'kivicare_user_account_status',true);
            if(!empty($user_deactivate_status)){
                $user_deactivate = $user_deactivate_status;
            }
            $data[$key]['user_deactivate'] = $user_deactivate;

			if ( $user_meta != null ) {
				$basic_data                    = json_decode( $user_meta );
				$data[ $key ]['mobile_number'] = $basic_data->mobile_number;
				$data[ $key ]['gender']        = $basic_data->gender;
				$data[ $key ]['dob']           = $basic_data->dob;
				$data[ $key ]['address']       = $basic_data->address;
			}
		}

		echo json_encode([
			'status'     => true,
			'message'    => esc_html__('Receptionist list', 'kc-lang'),
			'data'       => array_values($data),
			'total_rows' => $receptionists_count
		]);

	}

	public function save() {

		global $wpdb;

		$is_permission = false;
		$active_domain = $this->getAllActivePlugin();

		if ( kcCheckPermission( 'receptionist_profile' ) || kcCheckPermission( 'receptionist_add' ) || kcCheckPermission( 'receptionist_edit' ) ) {
			$is_permission = true;
		}

		if ( ! $is_permission ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have permission to access', 'kc-lang'),
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();

		$rules = [
			'first_name'    => 'required',
			'user_email'    => 'required|email',
			'mobile_number' => 'required',
			//'dob'           => 'required',
			'gender'        => 'required',
		];

		$errors = kcValidateRequest( $rules, $request_data );

		$username = kcGenerateUsername( $request_data['first_name'] );

		$password = kcGenerateString( 12 );

		$login_user = wp_get_current_user();

		if ( count( $errors ) ) {
			echo json_encode( [
				'status'  => false,
				'message' => $errors[0]
			] );
			die;
		}

		$temp = [
			'mobile_number' => $request_data['mobile_number'],
			'gender'        => $request_data['gender'],
			'dob'           => !empty($request_data['dob']) ? $request_data['dob'] : '',
			'address'       => $request_data['address'],
			'city'          => $request_data['city'],
			'state'         => '',
			'country'       => $request_data['country'],
			'postal_code'   => $request_data['postal_code'],
		];

		$request_data['clinic_id'] = json_decode(stripslashes( $request_data['clinic_id']));
        
        if( gettype($request_data['clinic_id']) == 'object' ){
            $clinic_id = (int)$request_data['clinic_id']->id;
        }else if( gettype($request_data['clinic_id']) == 'array' ){
            $clinic_id = (int)$request_data['clinic_id'][0]->id;
        }
        $clinic_id = (!empty($clinic_id) && gettype($clinic_id) === 'integer' ) ? $clinic_id : kcGetDefaultClinicId();

		if($this->getLoginUserRole() === $this->getClinicAdminRole()) {
            $clinic_id = kcGetClinicIdOfClinicAdmin();
		}

		if ( ! isset( $request_data['ID'] ) ) {

            if(email_exists($request_data['user_email'])){
                echo json_encode([
                    'status'  => false,
                    'message' => esc_html__('Receptionist email already exist.', 'kc-lang')
                ]); die;
            }

			$user = wp_create_user( $username, $password, sanitize_email( $request_data['user_email'] ) );

			$u               = new WP_User( $user );
			$u->display_name = $request_data['first_name'] . ' ' . $request_data['last_name'];
			wp_insert_user( $u );

			$u->set_role( $this->getReceptionistRole() );

			$user_id = $u->ID;
			update_user_meta($user, 'first_name', $request_data['first_name'] );
			update_user_meta($user, 'last_name', $request_data['last_name'] );
			update_user_meta( $user, 'basic_data', json_encode( $temp, JSON_UNESCAPED_UNICODE ) );

			// Insert Doctor Clinic mapping...
			$receptionist_mapping = new KCReceptionistClinicMapping;

			$new_temp = [
				'receptionist_id' => $user_id,
				'clinic_id'       => $clinic_id,
				'created_at'      =>   current_datetime('Y-m-d H:i:s' )
			];

			$receptionist_mapping->insert( $new_temp );

            $user_email_param = kcCommonNotificationUserData($user_id,$password);

            kcSendEmail($user_email_param);
            if(kcCheckSmsOptionEnable()){
                $sms = apply_filters('kcpro_send_sms', [
                    'type' => 'receptionist_register',
                    'user_data' => $user_email_param,
                ]);
            }

			$message = esc_html__('Receptionist saved successfully', 'kc-lang');

		} else {

			$user = email_exists($request_data['user_email']);

			if(!empty($user)) {
				if($request_data['ID'] != $user ) {
					echo json_encode([
						'status'  => false,
						'message' => esc_html__('Receptionist email already exist.', 'kc-lang')
					]); die;
				}
			}

			$receptionist_mapping = new KCReceptionistClinicMapping;

            $request_data['ID'] = (int)$request_data['ID'];
			wp_update_user(
				array(
					'ID'           => $request_data['ID'],
					'user_email'   => sanitize_email( $request_data['user_email']),
					'display_name' => $request_data['first_name'] . ' ' . $request_data['last_name']
				)
			);

			$user_id = $request_data['ID'];
			update_user_meta($user_id, 'first_name', $request_data['first_name']);
			update_user_meta($user_id, 'last_name' , $request_data['last_name']);
			update_user_meta( $user_id, 'basic_data', json_encode( $temp, JSON_UNESCAPED_UNICODE ));

            (new KCReceptionistClinicMapping())->delete(['receptionist_id' => $user_id]);
            $receptionist_mapping->insert([
                'receptionist_id' => $user_id,
                'clinic_id' => isKiviCareProActive() ? $clinic_id : kcGetDefaultClinicId(),
                'created_at' => current_time('Y-m-d H:i:s')
            ]);
            // hook for Receptionist save.
            do_action( 'kc_receptionist_update', $user_id);
			$message = esc_html__('Receptionist has been updated successfully', 'kc-lang');

		}

		if ( $user_id ) {
            // hook for Receptionist save.
            do_action( 'kc_receptionist_save', $user_id );
			$user_table_name = $wpdb->base_prefix . 'users';
			$user_status     = $request_data['user_status'];
			$wpdb->update( $user_table_name, [ 'user_status' => $user_status ], [ 'ID' => $user_id ] );
		}
		if($request_data['profile_image'] != '' && isset($request_data['profile_image']) && $request_data['profile_image'] != null ){
            $attachment_id = media_handle_upload('profile_image', 0);
            update_user_meta( $user_id, 'receptionist_profile_image',  $attachment_id  );
        }
		if ( !empty($user->errors) ) {
			echo json_encode( [
				'status'  => false,
				'message' => $user->get_error_message() ? $user->get_error_message() : esc_html__(' Failed to save Receptionist data', 'kc-lang')
			] );
		} else {
			echo json_encode( [
				'status'  => true,
				'message' => $message
			] );
		}

	}

	public function edit() {

		$is_permission = false;

		if ( kcCheckPermission( 'receptionist_profile' ) || kcCheckPermission( 'receptionist_edit' ) ) {
			$is_permission = true;
		}

		if ( ! $is_permission ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have permission to access', 'kc-lang'),
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();
        try {

			if ( !isset( $request_data['id'] ) ) {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}

            $table_name = collect((new KCClinic)->get_all());

            $request_data['id'] = (int)$request_data['id'];
            $clinic_id =  (new KCReceptionistClinicMapping())->get_by([ 'receptionist_id' =>$request_data['id']]);
            $clinics = collect((new KCReceptionistClinicMapping)->get_by(['receptionist_id' =>$request_data['id']]))->pluck('clinic_id')->toArray();
            $clinics = $table_name->whereIn('id', $clinics);
            $image_attachment_id = get_user_meta($request_data['id'],'receptionist_profile_image',true);
            $user_image_url = wp_get_attachment_url($image_attachment_id);

            $id = $request_data['id'];

			$user = get_userdata( $id );
			unset( $user->user_pass );

			$user_data  = get_user_meta( $id, 'basic_data', true );
            if(!empty($user_data)){
                $user_data = array_map(function ($v){
                    if(is_null($v) || $v == 'null'){
                        $v = '';
                    }
                    return $v;
                },(array)json_decode($user_data));
            }else{
                $user_data = [];
            }
			$first_name = get_user_meta( $id, 'first_name', true );
			$last_name  = get_user_meta( $id, 'last_name', true );

			$data             = (object) array_merge( (array) $user->data, $user_data );
			$data->first_name = $first_name;
			$data->username   = $data->user_login;
			$data->last_name  = $last_name;
            $clinic_id_array = $list = [];
			foreach($clinics as $d ){
                $clinic_id_array[] = (int)$d->id;
                $list[] = [
                    'id'    => $d->id,
                     'label' => $d->name,
                 ];
            }
            $data->clinic_id = $list;
			$data->user_profile =$user_image_url;
			if ( $data ) {
                $current_user_role = $this->getLoginUserRole();
                $validate = true;
                if ($current_user_role == $this->getClinicAdminRole()) {
                    $clinic_id = kcGetClinicIdOfClinicAdmin();
                    if (!in_array($clinic_id,$clinic_id_array)) {
                        $validate = false;
                    }
                } elseif ($current_user_role == $this->getReceptionistRole()) {
                    if (get_current_user_id() !== $id) {
                        $validate = false;
                    }
                }
                if (!$validate) {
                    echo json_encode( [
                        'status' => false,
                        'status_code' => 403,
                        'message' => esc_html__('You don\'t have permission to access', 'kc-lang'),
                        'data' => []
                    ]);
                    die;
                }
				echo json_encode( [
					'status'    => true,
					'message'   => 'Receptionist data found',
					'id'        => $id,
					'user_data' => $user_data,
					'data'      => $data
				] );
			} else {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}


		} catch ( Exception $e ) {

			$code    = $e->getCode();
			$message = $e->getMessage();

			header( "Status: $code $message" );

			echo json_encode( [
				'status'  => false,
				'message' => $e->getMessage()
			] );
		}
	}

	public function delete() {

		if ( ! kcCheckPermission( 'receptionist_delete' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have permission to access', 'kc-lang'),
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();

		try {

			if ( ! isset( $request_data['id'] ) ) {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}

			$id = (int)$request_data['id'];

            // hook for Receptionist delete
            do_action( 'kc_receptionist_delete', $id );

			delete_user_meta( $id, 'basic_data' );
			delete_user_meta( $id, 'first_name' );
			delete_user_meta( $id, 'last_name' );

            (new KCReceptionistClinicMapping())->delete(['receptionist_id' => $id]);

			$results = wp_delete_user( $id );

			if ( $results ) {
				echo json_encode( [
					'status'  => true,
					'message' => esc_html__('Receptionist deleted successfully', 'kc-lang'),
				] );
			} else {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}


		} catch ( Exception $e ) {

			$code    = $e->getCode();
			$message = $e->getMessage();

			header( "Status: $code $message" );

			echo json_encode( [
				'status'  => false,
				'message' => $message
			] );
		}

	}

	public function changeEmail() {

		$request_data = $this->request->getInputs();

		echo json_encode( [
			'status'  => true,
			'data'    => $request_data,
			'message' => esc_html__('Email has been changed', 'kc-lang'),
		] );

	}

}