<?php

namespace App\Controllers;

use App\baseClasses\KCBase;
use App\baseClasses\KCRequest;
use App\models\KCAppointmentServiceMapping;
use App\models\KCService;
use App\models\KCServiceDoctorMapping;
use Automattic\WooCommerce\Blocks\RestApi;
use App\models\KCReceptionistClinicMapping;
use App\models\KCClinic;
use Exception;
use WP_User;
class KCServiceController extends KCBase {

	public $db;
	/**
	 * @var KCRequest
	 */
	private $request;

	public function __construct() {

		global $wpdb;

		$this->db = $wpdb;

		$this->request = new KCRequest();

	}

	public function index() {

		$request_data      = $this->request->getInputs();
		$service_table     = $this->db->prefix . 'kc_services';
		$service_doctor_mapping  = $this->db->prefix . 'kc_service_doctor_mapping' ;
		$users_table       = $this->db->base_prefix . 'users';
		$clinic_doctor_mapping = $this->db->prefix.'kc_doctor_clinic_mappings';

        //current login user role
        $current_login_user_role = $this->getLoginUserRole();

        //current login user id
        $current_login_user_id = get_current_user_id();

        //default query condition value
        $search_condition = $doctor_condition  = $clinic_condition = $clinic_join = $doctor_googlemeet = "";
        $doctor_telemed_enable = false;

        $request_from_new_appointment_widget = !empty($request_data['widgetType']) && $request_data['widgetType'] === 'phpWidget';

        //search query condition
        //check request is from new appointment book shortcode/widget
        if($request_from_new_appointment_widget){
            if (!empty($request_data['searchKey'])){
                $request_data['searchKey'] = esc_sql($request_data['searchKey']);
                $searchKey = $request_data['searchKey'];
                $search_condition = " AND ({$service_table}.name LIKE '%{$searchKey}%' OR {$service_table}.type LIKE '%{$searchKey}%' OR {$service_doctor_mapping}.charges LIKE '%{$searchKey}%')";
            }
        }else{
            if ( !empty($request_data['searchKey']) && !empty($request_data['searchValue'])) {
                $search_condition = " AND {$service_table}.{$request_data['searchKey']}  LIKE  '%{$request_data['searchValue']}%' ";
            }
        }

        //check if login user is doctor or request data have valid doctor id
		if(($this->getDoctorRole() === $current_login_user_role) || (isset($request_data['doctor_id']) && !in_array($request_data['doctor_id'],[0,'0']))) {

            //doctor id
            $doctor_id = $this->getDoctorRole() === $current_login_user_role ? $current_login_user_id :  (int)$request_data['doctor_id'];

            //doctor query condition
            $doctor_condition = " AND {$service_doctor_mapping}.doctor_id = {$doctor_id} " ;

            //check doctor  telemed enable
            $doctor_telemed_enable = kcDoctorTelemedServiceEnable($doctor_id);

        }
        //check current login user role is clinic-admin or receptionist
        elseif(in_array($current_login_user_role,[$this->getClinicAdminRole(),$this->getReceptionistRole()])){
            //check pro plugin active
            if(isKiviCareProActive()){
                //clinic join query
				$clinic_join = " JOIN  {$clinic_doctor_mapping} ON {$clinic_doctor_mapping}.doctor_id = {$service_doctor_mapping}.doctor_id";
                //get clinic id based on current login user role
                $clinic_id = $this->getReceptionistRole() === $current_login_user_role ? kcGetClinicIdOfReceptionist(): kcGetClinicIdOfClinicAdmin() ;
            }
            else{
                $clinic_join = " JOIN  {$clinic_doctor_mapping} ON {$clinic_doctor_mapping}.doctor_id = {$service_doctor_mapping}.doctor_id";
                //get default clinic id
                $clinic_id = kcGetDefaultClinicId();
            }
            //clinic id query condition
            $clinic_condition = " AND {$clinic_doctor_mapping}.clinic_id = {$clinic_id}" ;
        }


        // get only active service list in appointment book
        $active_services = '' ;
        if((isset($request_data['module_type']) && $request_data['module_type'] === 'appointment_form') || ($request_from_new_appointment_widget)) {
			$active_services = " AND {$service_table}.status = '1' AND {$service_doctor_mapping}.status = '1' ";
		}

        //check if doctor zoom or googlemeet telemed is enable
		if($doctor_telemed_enable) {
            //get telemed service
            $telemed_condition = '';
		}else{
            // ignore telemed service condition
            $telemed_condition = " AND {$service_table}.type != 'system_service' ";
            //check request is from new appointment book shortcode/widget and check doctor id empty or not valid id
            if($request_from_new_appointment_widget &&
                (empty($request_data['doctor_id']) || in_array($request_data['doctor_id'],[0,'0']))){
                //check if telemed or googlemeet plugin active
                if(isKiviCareTelemedActive() || isKiviCareGoogleMeetActive()){
                    //empty telemed service condition ,to get telemed service
                    $telemed_condition = " ";
                }
            }
		}

        //query for service list
        $query = "SELECT {$service_doctor_mapping}.*,{$service_table}.price as service_base_price,
                  CONCAT({$service_table}.name,{$service_table}.type,{$service_doctor_mapping}.doctor_id) AS full_service_name,
                  {$service_table}.name AS name, {$service_table}.type AS service_type, {$service_table}.created_at AS created_at,
                  {$users_table}.display_name AS doctor_name  FROM {$service_doctor_mapping}
                  JOIN {$service_table} ON {$service_doctor_mapping}.service_id = {$service_table}.id
                  JOIN {$users_table} ON {$users_table}.ID = {$service_doctor_mapping}.doctor_id
                  {$clinic_join}
                  WHERE 0 = 0 {$doctor_condition} {$clinic_condition} {$active_services} {$telemed_condition} {$search_condition} 
                  ORDER BY {$service_doctor_mapping}.id  DESC" ;

        //get unique service (full_service_name = service_name + service_category_name + service_doctor_id)
		$services = collect($this->db->get_results( $query, OBJECT ))->values();

        if($request_from_new_appointment_widget){
            //get clinic id wise service list
            if(!empty($request_data['clinic_id']) && !in_array($request_data['clinic_id'],['0',0]) && isKiviCareProActive()){
                $request_data['clinic_id'] = (int)$request_data['clinic_id'];
                //get doctor list of requested clinic
                $clinic_doctor = collect($this->db->get_results("SELECT `doctor_id` FROM {$this->db->prefix}kc_doctor_clinic_mappings WHERE `clinic_id` = {$request_data['clinic_id']}"))->unique('doctor_id')->pluck('doctor_id');
                if(!empty($clinic_doctor)){
                    //get services of clinic doctors
                    $services = $services->whereIn('doctor_id',$clinic_doctor)->values();
                }
            }
        }

		$services = $services->map( function ( $services )use($request_data,$request_from_new_appointment_widget)  {
            //service image
            $services->image = !empty($services->image) ? wp_get_attachment_url($services->image) : '';
			//service category name format
            $services->service_type = isset( $services->service_type ) ? str_replace( '_', ' ', $services->service_type) : "";
            //check if service name is telemed
            if(in_array($services->name,['telemed','Telemed'])){
                //get category name of telemed service (updated category name of telemed service)
                $services->service_type = !empty($services->service_name_alias) ? str_replace("_"," ",strtolower($services->service_name_alias)) :  $services->service_type;
            }

            if($request_from_new_appointment_widget){
                //check doctor id empty or not valid id
                if(empty($request_data['doctor_id']) || in_array($request_data['doctor_id'],[0,'0'])){
                    //change service charges as base service price
                    $services->charges = $services->service_base_price;
                }
            }

            return $services;
		} );

        if (empty($service) && !count( $services ) ) {
			echo json_encode( [
				'status'  => false,
				'message' => esc_html__('No services found', 'kc-lang'),
				'data'    => []
			] );
			wp_die();
		}
        $htmldata = '';
        if($request_from_new_appointment_widget){
            $htmldata = $this->kcCreateServiceListHtml($services,$request_data);
        }
		echo json_encode( [
			'status'     => true,
			'message'    => esc_html__('Service list', 'kc-lang'),
			'data'       => $services,
			'total_rows' => count( $services ),
            'html' => $htmldata
		] );

	}

	public function save() {

		global $wpdb;

		if ( ! kcCheckPermission( 'service_add' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have permission to access', 'kc-lang'),
				'data'        => []
			] );
			wp_die();
		}

        $request_data = $this->request->getInputs();
        $doctor_service_table = $this->db->prefix . 'kc_service_doctor_mapping';
        $service_table =  $this->db->prefix .'kc_services';
        //add new column in table
        kcUpdateFields($doctor_service_table,[
            'service_name_alias' => 'varchar(191)',
            'multiple' => 'varchar(191)',
            'image' => 'bigint']
        );

        //current login user role
        $current_login_user_role = $this->getLoginUserRole();

        //current login user id
        $current_login_user_id = get_current_user_id();

        //remove slash from request data
        $request_data['status'] = json_decode(stripslashes($request_data['status']),true);
        $request_data['doctor_id'] = json_decode(stripslashes($request_data['doctor_id']),true);
        $request_data['type'] = json_decode(stripslashes($request_data['type']),true);
        $request_data['multiservice'] = json_decode(stripslashes($request_data['multiservice']),true);

        //service data
        $temp = [
			'name'   => $request_data['name'],
			'price'  => (int)$request_data['price'],
			'type'   => str_replace(' ', '_', strtolower($request_data['type']['label'])),
			'status' => $request_data['status']['id'],
		];
        $telemedTemp =$temp;

        //change category name of telemed service to default
        if(in_array($request_data['name'],['telemed','Telemed'])){
            $temp['type'] = 'system_service';
        }

        //get doctor id from request data
        if($current_login_user_role === $this->getDoctorRole()){
            $doctor_id = [$current_login_user_id];
        }else{
            $doctor_id = collect($request_data['doctor_id'])->pluck('id')->map(function($v){
                return (int)$v;
            })->toArray();
        }

        //check if same service exists with same doctor
        $validationQuery = "SELECT map.id FROM {$doctor_service_table} AS map LEFT JOIN {$service_table} AS ser ON ser.id=map.service_id WHERE 0=0 ";
        //implode doctor id for query
        $implode_doctor_id = implode(',',$doctor_id);

        //check if service update or save
        if(!empty($request_data['id'])){
            $request_data['id'] = (int)$request_data['id'];
            $validationRow = $wpdb->get_var($validationQuery ." AND map.id = {$request_data['id']}  AND map.doctor_id IN ({$implode_doctor_id}) AND ser.type ='{$temp['type']}' AND ser.name='{$request_data['name']}'");
            if($validationRow == $request_data['id']){
                $validationRow = '';
            }
        }else{
            $validationRow = $wpdb->get_var($validationQuery." AND map.doctor_id IN ({$implode_doctor_id}) AND ser.type ='{$temp['type']}' AND ser.name='{$request_data['name']}'");
        }


        if(!empty($validationRow)){
            echo json_encode( [
                'status'      => false,
                'message'     => esc_html__('Same Service Already Exists,Please select Different category or service name', 'kc-lang'),
                'data'        => []
            ] );
            wp_die();
        }

		$service = new KCService();

		$service_doctor_mapping = new KCServiceDoctorMapping();

        $attachment_id = 0;
        if ( ! function_exists( 'wp_handle_upload' ) ) {
            require_once( ABSPATH . 'wp-admin/includes/file.php' );
        }
        if($request_data['profile_image'] != '' && isset($request_data['profile_image']) && $request_data['profile_image'] != null ) {
            $attachment_id = media_handle_upload('profile_image', 0);
            if(is_wp_error($attachment_id)) {
                echo json_encode([
                    'status' => false,
                    'message' => esc_html__('Failed to upload Service image.', 'kc-lang')
                ]); die;
            }
        }
		if (!isset( $request_data['id'])) {

			$temp['created_at'] = current_time( 'Y-m-d H:i:s' );
            $service_id = $wpdb->get_var("SELECT id FROM {$service_table} WHERE type='{$temp['type']}' AND name='{$temp['name']}'");
            if(empty($service_id)){
                $service_id = $service->insert( $temp );
            }

            if ($service_id) {

                foreach ($doctor_id as $doctor){

                    $service_mapping_data = [
                        'service_id' => (int)$service_id,
                        'clinic_id'  => kcGetDefaultClinicId(),
                        'doctor_id'  => (int)$doctor,
                        'charges'    => (int)$request_data['price'],
                        'status'     => (int)$request_data['status']['id'],
                        'image'       => $attachment_id,
                        'multiple'  =>$request_data['multiservice']['id']
                    ];

                    $service_doctor_mapping->insert($service_mapping_data);
                    // hook for service add.
                    do_action( 'kc_service_add', $service_mapping_data );
                }
            }

			$message  = esc_html__('Service saved successfully', 'kc-lang');

		} else {
            $request_data['id'] = (int)$request_data['id'];
            $service_id = $wpdb->get_var("SELECT id FROM {$service_table} WHERE type='{$temp['type']}' AND name='{$temp['name']}'");
            if(empty($service_id)){
                $temp['created_at'] = current_time( 'Y-m-d H:i:s' );
                $request_data['service_id'] = $service->insert( $temp );
            }else{
                $request_data['service_id'] = (int)$request_data['service_id'];
                $service->update([
                    'name' => $request_data['name']
                ], array('id' => $request_data['service_id']));
            }

			$service_mapping_update_data  = [
				'service_id' => $request_data['service_id'],
				'doctor_id' => (int)$request_data['doctor_id']['id'],
				'charges'    => (int)$request_data['price'],
                'status'    => (int)$request_data['status']['id'],
                'multiple'  =>$request_data['multiservice']['id']
			];

            if(!empty($attachment_id)){
                $service_mapping_update_data['image'] = $attachment_id;
            }

            if(in_array($request_data['name'],['telemed','Telemed'])){
                $service_mapping_update_data['service_name_alias'] = $telemedTemp['type'];
            }

            $old_service_id_before_update = $this->db->get_var("SELECT service_id FROM {$doctor_service_table} WHERE id={$request_data['id']}");

            if($request_data['service_id'] !== (int)$old_service_id_before_update){
                (new KCAppointmentServiceMapping())->update(['service_id' => $request_data['service_id']],['service_id' => (int)$old_service_id_before_update]);
            }

            $service_doctor_mapping->update($service_mapping_update_data, array('id' => $request_data['id']));

            $product_id = $this->getProductIdOfService($request_data['id']);
            if(!empty($product_id)  &&  get_post_status( $product_id )){
                update_post_meta($product_id,'_price', $request_data['price']);
                update_post_meta($product_id,'_sale_price', $request_data['price']);
                $my_post = array(
                    'ID'           => $product_id,
                    'post_title'   => $request_data['name'],
                );
                wp_update_post( $my_post );
            }

			$service_mapping_update_data['id'] = $request_data['id'];

			do_action( 'kc_service_update', $service_mapping_update_data);

            $message = esc_html__('Service updated successfully', 'kc-lang');

		}

		echo json_encode( [
			'status'  => true,
			'message' => $message
		] );

	}

	public function edit() {

		
		if ( ! kcCheckPermission( 'service_edit' ) || ! kcCheckPermission( 'service_view' ) ) {
			
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have permission to access', 'kc-lang'),
				'data'        => []
			] );
			
			wp_die();
		}

		$request_data = $this->request->getInputs();

		try {

			if ( ! isset( $request_data['id'] ) ) {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}

			$edit_id = (int)$request_data['id'];
			$service_table     = $this->db->prefix . 'kc_services';
			$service_doctor_mapping  = $this->db->prefix . 'kc_service_doctor_mapping' ;
			$users_table       = $this->db->base_prefix . 'users';
			
			$query = " SELECT {$service_doctor_mapping}.id AS mapping_id, {$service_table}.id AS service_id, 
                              {$service_doctor_mapping}.*, {$service_table}.* , {$users_table}.display_name AS doctor_name, 
                              {$service_doctor_mapping}.charges AS doctor_charges, 
                              {$service_doctor_mapping}.status AS mapping_status
                       FROM  {$service_doctor_mapping} 
					   JOIN  {$users_table} ON {$users_table}.ID = {$service_doctor_mapping}.doctor_id  
					   JOIN  {$service_table} ON {$service_table}.id = {$service_doctor_mapping}.service_id  
					   WHERE {$service_doctor_mapping}.id = {$edit_id} ";


			$service = $this->db->get_row( $query );

			if ( count( $service ) ) {

                $status =  new \stdClass();
				$status->id =  0 ;
				$status->label = 'Inactive' ;
				
				if((int) $service->mapping_status === 1) {
					$status->id = 1 ;
					$status->label = 'Active' ;
				}

				$temp = [
					'id'     => $service->mapping_id,
					'service_id' => $service->service_id,
					'name'   => $service->name,
					'price'  => $service->doctor_charges,
                    'doctor_id' =>  [
						'id' 	=>  $service->doctor_id,
						'label' =>  $service->doctor_name
					],
					'type'   => [
						'id'    => $service->type,
						'label' => str_replace('_'," ",$service->type)
					],
					'status' => $status,
                    'image' => !empty($service->image) ? wp_get_attachment_url($service->image) : ''
				];

                if(!empty($service->multiple)){
                    $temp['multiservice'] = [
                        'id' =>  strtolower($service->multiple),
                        'label' =>  $service->multiple
                    ];
                }else{
                    $temp['multiservice'] = [
                        'id' =>  'yes' ,
                        'label' =>  __("yes","kc-lang")
                    ];
                }
                if(in_array($service->name,['telemed','Telemed'])){
                    if(!empty($service->service_name_alias)){
                        $temp['type'] = [
                            'id' =>  str_replace(" ","_",strtolower($service->service_name_alias)) ,
                            'label' =>  $service->service_name_alias
                        ];
                    }else{
                        $temp['type'] = [
                            'id' =>  $service->type ,
                            'label' =>  str_replace('_'," ",$service->type)
                        ];
                    }
                }

                echo json_encode( [
					'status'  => true,
					'message' => esc_html__('Service data', 'kc-lang'),
					'data'    => $temp
				] );

			} else {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}


		} catch ( Exception $e ) {
			$code    = $e->getCode();
			$message = $e->getMessage();
			header( "Status: $code $message" );
			echo json_encode( [
				'status'  => false,
				'message' => $e->getMessage()
			] );
		}
	}

	public function delete() {

		if ( ! kcCheckPermission( 'service_delete' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have permission to access', 'kc-lang'),
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();

		try {

			if ( ! isset( $request_data['id'] ) ) {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}

            $service_doctor_mapping = new KCServiceDoctorMapping();

            $request_data['id'] = (int)$request_data['id'];
            $product_id = $this->getProductIdOfService($request_data['id']);
            if(!empty($product_id) && get_post_status( $product_id )){
				do_action( 'kc_woocoomerce_service_delete', $product_id );
                wp_delete_post($product_id);
            }

			$id = $request_data['id'];
			$results = $service_doctor_mapping->delete( [ 'id' => $id ] );

			if ( $results ) {
				do_action( 'kc_service_delete', $id);
				echo json_encode( [
					'status'  => true,
					'message' => esc_html__('Service has been deleted successfully', 'kc-lang'),
				] );
			} else {
				throw new Exception( esc_html__('Failed to delete Service', 'kc-lang'), 400 );
			}


		} catch ( Exception $e ) {

			$code    = $e->getCode();
			$message = $e->getMessage();

			header( "Status: $code $message" );

			echo json_encode( [
				'status'  => false,
				'message' => $message
			] );
		}
	}

	public function clinicService () {
	    $table = $this->db->prefix  . 'kc_services' ;
        $query = "SELECT `id`, `type`, `name`, `price` FROM {$table} " ;
        $services = $this->db->get_results( $query, OBJECT );
        echo json_encode([
            'status' => true,
            'message' => esc_html__('Clinic service list', 'kc-lang'),
            'data' => $services
        ]);
    }

	public function getProductIdOfService($id){
        $id = (int)$id;
		$product_id = '';
		$appointments_service_table =  $this->db->prefix . 'kc_service_doctor_mapping';
		$data = $this->db->get_var('select extra from '.$appointments_service_table.' where id='.$id);
        if($data != null){
			$data = json_decode($data);
            $product_id = $data->product_id;
		}	
		return $product_id;
	}

    public function kcCreateServiceListHtml($services,$request_data){
        $services = $services->sortBy('service_type')->groupBy('service_type');
        $showServicetImageStatus = kcGetSingleWidgetSetting('showServiceImage');
        $showServicetypeStatus = kcGetSingleWidgetSetting('showServicetype');
        $showServicePriceStatus = kcGetSingleWidgetSetting('showServicePrice');
        $clinicCurrenySetting = kcGetClinicCurrenyPrefixAndPostfix();
        $clinic_prefix = !empty($clinicCurrenySetting['prefix']) ? $clinicCurrenySetting['prefix'] : '';
        $clinic_postfix = !empty($clinicCurrenySetting['postfix']) ? $clinicCurrenySetting['postfix'] : '';
        ob_start();
        foreach( $services as $key => $main_services){
            ?>
            <div class="d-flex flex-column gap-1 pt-2">
                <?php if($showServicetypeStatus){
                    ?>
                    <h5 class="iq-color-secondary iq-letter-spacing-1 pl-1"><?php echo esc_html($key); ?></h5>
                    <?php
                } ?>
                <div class="text-center iq-category-list" >
                    <?php
                    foreach ($main_services as $service) {
                        $singleServiceClass = '';
                        if(!empty($service->multiple) && $service->multiple == 'no'){
                            $singleServiceClass = ' selected-service-single';
                        }
                        if(count($services) == 1){
                            $singleServiceClass = ' selected-service-single';
                        }
                        $image = !empty($service->image) ? $service->image : KIVI_CARE_DIR_URI .'assets/images/kc-demo-img.png' ;
                        ?>
                        <div class="iq-client-widget">
                            <input type="checkbox" class="card-checkbox selected-service <?php echo esc_html($singleServiceClass);?>" name="card_main"
                                   id="service_<?php echo esc_html($service->id); ?>"
                                   value="<?php echo esc_html($service->id); ?>"
                                   service_prefix="<?php echo esc_html($clinic_prefix); ?>"
                                   service_postfix="<?php echo esc_html($clinic_postfix); ?>"
                                   service_id="<?php echo esc_html($service->service_id); ?>"
                                   service_name="<?php echo esc_html($service->name); ?>"
                                   service_price="<?php echo esc_html($service->charges); ?>"
                                   doctor_id="<?php echo esc_html($service->doctor_id); ?>"
                                   clinic_id="<?php echo esc_html($service->clinic_id); ?>"
                                   status="<?php echo esc_html($service->status); ?>"
                                   created_at="<?php echo esc_html($service->created_at); ?>"
                                   doctor_name="<?php echo esc_html($service->doctor_name); ?>"
                                   service_type="<?php echo esc_html($service->service_type); ?>"
                                   multipleService="<?php echo esc_html(!empty($service->multiple) && $service->multiple == 'no' ? 'no' : 'yes')?>" >
                            <label class="btn-border01 service-content" for="service_<?php echo esc_html($service->id); ?>">
                                <div class="iq-card iq-card-border iq-fancy-design service-content gap-1 kc-service-card">
                                    <?php
                                    if($showServicetImageStatus){
                                        ?>
                                        <div class="d-flex align-items-center justify-content-center">
                                            <div class="avatar-70 avatar icon-img">
                                                <img src="<?php echo esc_url($image); ?>" alt="service_image" class="avatar-70 rounded-circle"/>
                                            </div>
                                        </div>
                                    <?php } ?>
                                    <div class="d-flex flex-column">
                                        <div class="kc-service-name">
                                            <h6><?php echo esc_html($service->name); ?></h6>
                                        </div>
                                        <?php
                                        if($showServicePriceStatus){
                                            ?>
                                            <p class="iq-dentist-price">
                                                <?php
                                                if(empty($request_data['doctor_id']) || in_array($request_data['doctor_id'],[0,'0'])){
                                                    echo esc_html__("Base Price: ","kc-lang"). esc_html($clinic_prefix . $service->charges . $clinic_postfix);
                                                }else{
                                                    echo esc_html($clinic_prefix . $service->charges . $clinic_postfix);
                                                }
                                                ?>
                                            </p>
                                            <?php
                                        }
                                        ?>
                                    </div>
                                </div>
                            </label>
                        </div>
                        <?php
                    }
                    ?>
                </div>
            </div>
            <?php
        }
        return ob_get_clean();
    }
}


