<?php

namespace App\Controllers;

use App\baseClasses\KCBase;
use App\baseClasses\KCRequest;
use App\models\KCClinic;
use App\models\KCPatientClinicMapping;
use App\models\KCReceptionistClinicMapping;
use App\models\KCStaticData;
use Exception;
use WP_User;

class KCStaticDataController extends KCBase {

	public $db;

	public $table_name;

	public $db_config;

	/**
	 * @var KCRequest
	 */
	private $request;

	public function __construct() {
		global $wpdb;

		$this->db = $wpdb;

		$this->table_name = $wpdb->prefix . 'kc_static_data';

		$this->db_config = [
			'user' => DB_USER,
			'pass' => DB_PASSWORD,
			'db'   => DB_NAME,
			'host' => DB_HOST
		];

		$this->request = new KCRequest();

	}

	public function index() {

		if ( ! kcCheckPermission( 'static_data_list' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have permission to access', 'kc-lang'),
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();
		$static_data_table = $this->db->prefix . 'kc_' . 'static_data';
		$total_rows = 0 ;
		$condition = '';
		$static_data_count_query = "
			SELECT count(*) AS count FROM {$static_data_table} ";
		$total_static_data = $this->db->get_results( $static_data_count_query, OBJECT );

		if($request_data['searchKey'] && $request_data['searchValue']) {
			$search_key = $request_data['searchKey'];
			$search_value = $request_data['searchValue'];
			$condition = " WHERE {$static_data_table}.{$search_key} LIKE  '%{$search_value}%' " ;
		}

		$static_data_query = "
			SELECT *, REPLACE(type,'_',' ') as type
			FROM  {$static_data_table}  {$condition} ORDER BY id DESC ";

		$static_data = $this->db->get_results( $static_data_query, OBJECT );

		if($request_data['searchKey'] && $request_data['searchValue']) {
			$total_rows = count($static_data);
		} else {
			$total_rows = $total_static_data[0]->count ;
		}

		if ( ! count( $static_data ) ) {
			echo json_encode( [
				'status'  => false,
				'message' => esc_html__('No services found', 'kc-lang'),
				'data'    => []
			] );
			wp_die();
		}

		echo json_encode( [
			'status'  => true,
			'message' => __('Service list','kc-lang'),
			'data'    => $static_data,
			'total_rows' =>  $total_rows
		] );
	}

	public function save() {

//        bug
//		if ( ! kcCheckPermission( 'static_data_add' ) ) {
//			echo json_encode( [
//				'status'      => false,
//				'status_code' => 403,
//				'message'     => esc_html__('You don\'t have permission to access', 'kc-lang'),
//				'data'        => []
//			] );
//			wp_die();
//		}

        $message = '';
        $insert_id = '';
		$request_data = $this->request->getInputs();

		$value = str_replace(' ', '_', strtolower($request_data['label']));

		$temp = [
			'label' => $request_data['label'],
			'type' => (isset($request_data['type']['id']) ? $request_data['type']['id'] : $request_data['type']),
			'value' => $value,
			'status' => (isset($request_data['status']['id']) ? (int)$request_data['status']['id'] :  $request_data['status'])
		];

		$check_condition = '';
		if(!empty($request_data['id'])){
			$check_condition = " AND id !=".(int)$request_data['id'];
		}
        $existItem = $this->db->get_var("SELECT id FROM {$this->db->prefix}kc_static_data WHERE `type`='{$temp['type']}' AND `value`='{$value}' {$check_condition }");

        if(!empty($existItem)){
            echo json_encode([
                'status' => false,
                'message' => esc_html__('Similar value already exist in database', 'kc-lang')
            ]);
            die;
        }

		$static_data = new KCStaticData;

		if (!isset($request_data['id'])) {
            $temp['created_at'] = current_time('Y-m-d H:i:s');
            $insert_id = $static_data->insert($temp);
            $message = esc_html__('Static data saved successfully', 'kc-lang');
		} else {
			$static_data->update($temp, array( 'id' => (int)$request_data['id'] ));
			$message = esc_html__('Listing data updated successfully', 'kc-lang');
		}

		echo json_encode([
			'status' => true,
			'message' => $message,
            'insert_id' => $insert_id
		]);

	}

	public function edit() {

		if ( ! kcCheckPermission( 'static_data_edit' ) || ! kcCheckPermission('static_data_view') ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have permission to access', 'kc-lang'),
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();

		try {

			if (!isset($request_data['id'])) {
				throw new Exception(esc_html__('Data not found', 'kc-lang'), 400);
			}

			$id = $request_data['id'];

			$static_data = new KCStaticData;

			$results = $static_data->get_by(['id' => (int)$id], '=',true);

			$results->status = [
				'id' => 0,
				'label' => 'Inactive'
			] ;

			if( (int) $results->status === 1) {
				$results->status = [
					'id' => 1,
					'label' => 'Active'
				] ;
			}
			
			$results->type = [
				'id' => $results->type,
				'type' => str_replace('_', ' ', $results->type),
			] ;

			if ($results) {
				echo json_encode([
					'status' => true,
					'message' => esc_html__('Static data', 'kc-lang'),
					'data' => $results
				]);
			} else {
				throw new Exception(esc_html__('Data not found', 'kc-lang'), 400);
			}

		} catch (Exception $e) {

			$code    = $e->getCode();
			$message = $e->getMessage();

			header("Status: $code $message");

			echo json_encode([
				'status' => false,
				'message' => $message
			]);
		}
	}

	public function delete() {

		if ( ! kcCheckPermission( 'static_data_delete' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have permission to access', 'kc-lang'),
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();

		try {

			if (!isset($request_data['id'])) {
				throw new Exception(esc_html__('Data not found', 'kc-lang'), 400);
			}

			$id = $request_data['id'];

			$static_data = new KCStaticData;

			$results = $static_data->delete(['id' => (int)$id]);

			if ($results) {
				echo json_encode([
					'status' => true,
					'tableReload' => true,
					'message' => esc_html__('Static data deleted successfully', 'kc-lang'),
				]);
			} else {
				throw new Exception(esc_html__('Data not found', 'kc-lang'), 400);
			}


		} catch (Exception $e) {

			$code    = $e->getCode();
			$message = $e->getMessage();

			header("Status: $code $message");

			echo json_encode([
				'status' => false,
				'message' => $message
			]);
		}
	}

    public function getStaticData()
    {
        global $wpdb;
        $request_data = $this->request->getInputs();
        $active_domain =$this->getAllActivePlugin();

        if($active_domain === $this->kiviCareProOnName() && $request_data['data_type'] == 'clinic_list' ) {

            $table_name = $wpdb->prefix . 'kc_' . 'clinics';
            $response = apply_filters('kcpro_get_all_clinic', []);
            echo json_encode($response);

        }else{
            $data = [
                'status' => false,
                'message' => esc_html__('Datatype not found', 'kc-lang')
            ];

            if (isset($request_data['data_type']) || isset($request_data['type'])) {
                global $wpdb;
                $table_name = $wpdb->prefix . 'kc_' . 'static_data';
                $type = !empty($request_data['data_type']) ? $request_data['data_type'] : $request_data['type'];

                switch ($type) {
                    case "static_data":
                        $static_data_type = $request_data['static_data_type'];
                        $query = "SELECT id, label FROM $table_name WHERE type = '$static_data_type' AND status = '1' GROUP BY $table_name.`value`";
                        $results = $wpdb->get_results($query, OBJECT);
                        break;

                    case "static_data_with_label":
                        $static_data_type = $request_data['static_data_type'];
                        $query = "SELECT `value` as id, label FROM $table_name WHERE type = '$static_data_type' AND status = '1' GROUP BY $table_name.`value` ";
                        $results = collect($wpdb->get_results($query, OBJECT))->unique('id')->toArray();
                        break;

                    case "static_data_types":
                        $query = "SELECT `type` as id, REPLACE(type, '_' , ' ') AS `type` FROM $table_name WHERE status = 1 GROUP BY `type`";
                        $results = $wpdb->get_results($query, OBJECT);
                        break;

                    case "clinics":
                        $table_name = $wpdb->prefix . 'kc_' . 'clinics';
                        $condition = '';
                        $clinic_condition = ' ';
                        if(!$this->isKcProActivated()){
                            $condition = ' AND id='.kcGetDefaultClinicId().' ';
                        }else{
                            if($this->getLoginUserRole() == 'kiviCare_receptionist') {
                                $clinic =  (new KCReceptionistClinicMapping())->get_by([ 'receptionist_id' => get_current_user_id()]);
                                $clinic_id = isset($clinic[0]->clinic_id) ? $clinic[0]->clinic_id : kcGetDefaultClinicId() ;
                                $clinic_id = (int)$clinic_id;
                                $clinic_condition = " AND id={$clinic_id} ";
                            }else if($this->getLoginUserRole() == 'kiviCare_clinic_admin') {
                                $clinic = (new KCClinic())->get_by([ 'clinic_admin_id' => get_current_user_id()]);
                                $clinic_id = isset($clinic[0]->id) ? $clinic[0]->id : kcGetDefaultClinicId() ;
                                $clinic_id = (int)$clinic_id;
                                $clinic_condition = " AND id={$clinic_id} ";
                            }
                        }
                        $query = "SELECT `id`, `name` as `label` FROM {$table_name} WHERE `status` = '1' {$condition} {$clinic_condition}";
                        $results = $wpdb->get_results($query, OBJECT);
                        break;
                    case 'patient_clinic':
                        if($this->getLoginUserRole() == 'kiviCare_patient' && $this->isKcProActivated()){
                            $user_id = get_current_user_id();
                            $results = $wpdb->get_results("SELECT clinic.id , clinic.name AS label FROM  {$wpdb->prefix}kc_clinics AS clinic LEFT JOIN 
                                   {$wpdb->prefix}kc_patient_clinic_mappings AS pcmap ON  pcmap.clinic_id = clinic.id WHERE pcmap.patient_id={$user_id}",OBJECT);
                        }else{
                            $results = [];
                        }
                        break;
                    case "doctors":
                        $da = [];
                        $doctors = get_users([
                            'role' => $this->getDoctorRole(),
                            'user_status' => '0'
                        ]);
                        $doctorList = collect($doctors)->toArray();
                        foreach ($doctorList as $d) {
                            $da[] = [
                                'id'    => $d->data->ID,
                                'label' => $d->data->display_name
                            ];
                        }
                        $results = $da;
                        break;

                    case "default_clinic":
                        $table_name = $wpdb->prefix  . 'kc_' . 'clinics';
                        $default_clinic = get_option('setup_step_1');
                        $option_data = json_decode($default_clinic, true);
                        if(isset($option_data['id'][0])) {
                            $query = "SELECT * FROM {$table_name} WHERE `status`= '1' AND `id` = '{$option_data['id'][0]}' ";
                            $results = $wpdb->get_results($query, OBJECT);
                            // $results = $results[0];
                            if($results[0]->extra == null) {
                                $results[0]->extra->currency_prefix = '';
                                $results[0]->extra->currency_postfix = '';
                            }
                        } else {
                            $results = [];
                        }
                        break;

                    case "services_with_price":
                        $service_table = $wpdb->prefix . 'kc_' . 'services';
                        $service_doctor_mapping = $wpdb->prefix . 'kc_' . 'service_doctor_mapping';
                        $zoom_config_data = get_user_meta($request_data['doctorId'], 'zoom_config_data', true);
                        $zoom_config_data = json_decode($zoom_config_data);
                        if(isset($request_data['doctorId'])){
                            $request_data['doctorId'] = (int)$request_data['doctorId'];
                            if(!empty($zoom_config_data->enableTeleMed) && $zoom_config_data->enableTeleMed == 1){
                                $query = "SELECT {$service_table}.id ,{$service_doctor_mapping}.charges AS price,{$service_table}.name AS label FROM  {$service_table} 
                                JOIN {$service_doctor_mapping} ON  {$service_table}.id = {$service_doctor_mapping}.service_id 
                                WHERE {$service_table}.status = 1 AND {$service_doctor_mapping}.doctor_id =".$request_data['doctorId'];
                            }else{
                                $query = "SELECT {$service_table}.id ,{$service_doctor_mapping}.charges AS price,{$service_table}.name AS label FROM  {$service_table} 
                                JOIN {$service_doctor_mapping} ON  {$service_table}.id = {$service_doctor_mapping}.service_id 
                                WHERE {$service_table}.status = 1 AND {$service_doctor_mapping}.doctor_id =".$request_data['doctorId']." AND {$service_table}.type != 'system_service' ";
                            }

                        }else{
                            $query = "SELECT `id`, `price`, `name` as `label` FROM {$service_table} WHERE status = 1 ";
                        }
                        $results = $wpdb->get_results($query, OBJECT);
                        break;

                    case "prescriptions":
                        $table_name = $wpdb->prefix . 'kc_' . 'prescription';
                        $query = "SELECT `name` as `id`, `name` as `label` FROM {$table_name}";
                        $results = collect($wpdb->get_results($query, OBJECT))->unique('id')->toArray();
                        $results2 = collect($wpdb->get_results("SELECT `label` as id, label FROM {$wpdb->prefix}kc_static_data WHERE status = 1 AND `type`='prescription_medicine' GROUP BY `type`"))->unique('id')->toArray();
                        $results = collect(array_merge($results,$results2))->unique('label');
                        break;

                    case "email_template_type":
                        $query = "SELECT `id`, `value`, `label` FROM {$table_name} WHERE `status` = '1' AND `type` = 'email_template' ";
                        $results = $wpdb->get_results($query, ARRAY_A);
                        break;
                    case "email_template_key":
                        $results = ['{{user_name}}', '{{user_email}}', '{{user_contact}}'];
                        break;
                    case "get_users_by_clinic":
                        if(empty($request_data['clinic_id'])) {
                            $clinic_id = kcGetDefaultClinicId();
                        } else {
                            $clinic_id = (int)$request_data['clinic_id'] ;
                        }

                        $table_name = $wpdb->prefix . 'kc_' . 'doctor_clinic_mappings';
                        $query = "SELECT * FROM {$table_name} WHERE `clinic_id` = '{$clinic_id}' ";
                        $prefix_postfix =$wpdb->get_var('select extra from '.$wpdb->prefix.'kc_clinics where id='.$clinic_id);
                        if($prefix_postfix != null){
                            $prefix_postfix = json_decode( $prefix_postfix);
                            $data['prefix'] = (isset($prefix_postfix->currency_prefix) && $prefix_postfix->currency_prefix !== 'null') ? $prefix_postfix->currency_prefix : '';
                            $data['postfix'] = (isset($prefix_postfix->currency_postfix) && $prefix_postfix->currency_postfix !== 'null') ? $prefix_postfix->currency_postfix : '';
                        }
                        $clinic_data = $wpdb->get_results($query, OBJECT);
                        $results = [];
                        $doctor_ids = [];
                        foreach ($clinic_data as $clinic_map_data) {
                            if (isset($clinic_map_data->doctor_id)) {
                                if(isset($request_data['module_type']) && $request_data['module_type'] === 'appointment') {
                                    $doctor_session_data = collect($clinic_map_data)->toArray();
                                    if(in_array($clinic_map_data->doctor_id, $doctor_session_data)) {
                                        $doctor_ids[] = $clinic_map_data->doctor_id;
                                    }
                                } else {
                                    $doctor_ids[] = $clinic_map_data->doctor_id;
                                }
                            }
                        }
                        if (count($doctor_ids)) {
                            $users_table = $wpdb->base_prefix . 'users';
                            $new_query = "SELECT `ID` as `id`, `display_name` as `label`  FROM {$users_table} WHERE `ID` IN (" . implode(',', $doctor_ids) . ") AND `user_status` = '0'";
                            $results = $wpdb->get_results($new_query, OBJECT);
                        }
                        break;
                    case "clinic_doctors":
                        $table_name = $wpdb->prefix . 'kc_' . 'doctor_clinic_mappings';

                        if($active_domain === $this->kiviCareProOnName()){
                            if($this->getLoginUserRole() == 'kiviCare_receptionist') {
                                $receptionis_id = get_current_user_id();
                                $clinic =  (new KCReceptionistClinicMapping())->get_by([ 'receptionist_id' => $receptionis_id]);
                                $clinic_id = isset($clinic[0]->clinic_id) ? $clinic[0]->clinic_id : kcGetDefaultClinicId() ;
                            }else if($this->getLoginUserRole() == 'kiviCare_clinic_admin') {
                                $clinic = (new KCClinic())->get_by([ 'clinic_admin_id' => get_current_user_id()]);
                                $clinic_id = isset($clinic[0]->id) ? $clinic[0]->id : kcGetDefaultClinicId() ;
                            }else if($this->getLoginUserRole() == 'kiviCare_patient'){
                                $clinic = (new KCPatientClinicMapping())->get_by([ 'patient_id' => get_current_user_id()]);
                                $clinic_id = isset($clinic[0]->clinic_id) ? $clinic[0]->clinic_id : kcGetDefaultClinicId() ;
                            }
                            else{
                                $clinic_id = (!empty($request_data['clinic_id'])? (int)$request_data['clinic_id'] : 1 );
                            }
                            if(is_array($clinic_id)){
                                $clinic_id = !empty($clinic_id['id']) ? $clinic_id['id'] : 1;
                            }
                        }else{
                            $clinic_id = kcGetDefaultClinicId();
                        }
                        $clinic_id = (int)$clinic_id;
                        $prefix_postfix =$wpdb->get_var('select extra from '.$wpdb->prefix.'kc_clinics where id='.$clinic_id);
                        if($prefix_postfix != null){
                            $prefix_postfix = json_decode( $prefix_postfix);
                            $data['prefix'] = (isset($prefix_postfix->currency_prefix) && $prefix_postfix->currency_prefix !== 'null') ? $prefix_postfix->currency_prefix : '';
                            $data['postfix'] = (isset($prefix_postfix->currency_postfix) && $prefix_postfix->currency_postfix !== 'null') ? $prefix_postfix->currency_postfix : '';
                        }
                        $doctor_session_data = [];
                        if (isset($request_data['module_type']) && $request_data['module_type'] === 'appointment') {
                            $clinic_session_table = $wpdb->prefix. 'kc_' . 'clinic_sessions';
                            $doctor_sessions_query = "SELECT * FROM {$clinic_session_table} WHERE `clinic_id` = '{$clinic_id}' ";
                            $doctor_session_data = collect($wpdb->get_results($doctor_sessions_query, ARRAY_A))->pluck('doctor_id')->unique();
                        }
                        if (!current_user_can('administrator')) {
                            $query = "SELECT * FROM {$table_name} WHERE `clinic_id` = '{$clinic_id}' ";
                        }else{
                            if(!empty($request_data['clinic_id']) && !empty($request_data['module_type'])
                                && $request_data['module_type'] == 'appointment_filter'){
                                $query = "SELECT * FROM {$table_name} WHERE `clinic_id` = '{$clinic_id}' ";
                            }else{
                                $query = "SELECT * FROM {$table_name}";
                            }
                        }
                        $clinic_data = $wpdb->get_results($query, OBJECT);
                        $results = [];
                        $doctor_ids = [];

                        if (count($clinic_data)) {
                            foreach ($clinic_data as $clinic_map_data) {
                                if (isset($clinic_map_data->doctor_id)) {
                                    if(isset($request_data['module_type']) && $request_data['module_type'] === 'appointment') {
                                        $doctor_session_data = collect($doctor_session_data)->toArray();
                                        if(in_array($clinic_map_data->doctor_id, $doctor_session_data)) {
                                            $doctor_ids[] = $clinic_map_data->doctor_id;
                                        }
                                    } else {
                                        $doctor_ids[] = $clinic_map_data->doctor_id;
                                    }
                                }
                            }

                            if (count($doctor_ids)) {

                                $users_table = $wpdb->base_prefix . 'users';
                                $new_query = "SELECT `ID` as `id` , `display_name` as `label`  FROM {$users_table} WHERE `ID` IN (" . implode(',', $doctor_ids) . ") AND `user_status` = '0'";
                                $results = $wpdb->get_results($new_query, OBJECT);
                                if (count($results)) {
                                    foreach ($results as $result) {
                                        $user_data = get_user_meta($result->id, 'basic_data', true);
                                        if ($user_data) {
                                            $user_data = json_decode($user_data);
                                            $result->timeSlot = isset($user_data->time_slot) ? $user_data->time_slot : "";
                                            $specialties = collect($user_data->specialties)->pluck('label')->toArray();
                                            $result->label .= !empty($specialties) ? " (". implode( ',',$specialties).")" : '';
                                        }
                                        $zoom_config_data = get_user_meta($result->id, 'zoom_config_data', true);
                                        if ($zoom_config_data) {
                                            $zoom_config_data = json_decode($zoom_config_data);
                                            $enableTeleMed = false;
                                            if (isset($zoom_config_data->enableTeleMed) && (bool)$zoom_config_data->enableTeleMed) {
                                                if ($zoom_config_data->api_key !== "" && $zoom_config_data->api_secret !== "") {
                                                    $enableTeleMed = true;
                                                }
                                            }
                                            $result->enableTeleMed = $enableTeleMed;
                                        }
                                    }
                                }
                            }
                        }

                        break;

                    case "users":
                        $results = [];
                        $user_id = get_current_user_id();
                        $userObj = new WP_User($user_id);

                        $users = get_users([
                            'role' => $request_data['user_type'],
                            'user_status' => '0'
                        ]);

                        if($active_domain === $this->kiviCareProOnName()){
                            if($userObj->roles[0] !== 'kiviCare_patient'){
                                if($userObj->roles[0] == 'kiviCare_clinic_admin'){
                                    $clinic = (new KCClinic())->get_by([ 'clinic_admin_id' => get_current_user_id()]);
                                    $clinic_id = isset($clinic[0]->id) ? $clinic[0]->id : kcGetDefaultClinicId() ;
                                }else if($userObj->roles[0] == 'kiviCare_receptionist'){
                                    $clinic =  (new KCReceptionistClinicMapping())->get_by([ 'receptionist_id' => get_current_user_id()]);
                                    $clinic_id = isset($clinic[0]->clinic_id) ? $clinic[0]->clinic_id : kcGetDefaultClinicId() ;
                                }else if($userObj->roles[0] == 'kiviCare_doctor'){
                                    $clinic_id = collect($this->db->get_results("SELECT clinic_id FROM {$this->db->prefix}kc_doctor_clinic_mappings WHERE doctor_id=".get_current_user_id()))->pluck('clinic_id')->implode(',');
                                }
                                if(($userObj->roles[0] === 'administrator' || $userObj->roles[0] == 'kiviCare_doctor') && !empty($request_data['request_clinic_id'])){
                                    $clinic_id = $request_data['request_clinic_id'];
                                }

                                if(!empty($clinic_id)){
                                    $patient_clinic = $this->db->prefix.'kc_patient_clinic_mappings';
                                    if($userObj->roles[0] == 'kiviCare_doctor'){
                                        $result = collect($wpdb->get_results("select patient_id from ".$patient_clinic .' where clinic_id  IN ('.$clinic_id.')'))->unique('patient_id')->pluck('patient_id')->toArray();
                                    }else{
                                        $clinic_id = (int)$clinic_id;
                                        $result = collect($wpdb->get_results("select patient_id from ".$patient_clinic .' where clinic_id='.$clinic_id))->unique('patient_id')->pluck('patient_id')->toArray();
                                    }
                                    if(count($result) === 0){
                                        $users = [];
                                    }else{
                                        $users = collect($users)->whereIn('ID',$result)->toArray();
                                    }
                                }
                            }
                        }

                        if ($userObj->roles[0] == 'kiviCare_doctor') {
                            $result = kcDoctorPatientList();
                            if(count($result) === 0){
                                $users = [];
                            }else{
                                $users = collect($users)->whereIn('ID',$result)->toArray();
                            }
                        }
                        if (count($users)) {
                            $i = 0 ;
                            foreach ($users as $key => $user) {
                                $namePostFix = '';
                                if($request_data['user_type'] === $this->getPatientRole() && kcPatientUniqueIdEnable('status')){
                                    $uniqueID = get_user_meta( $user->ID, 'patient_unique_id',true);
                                    $namePostFix = '('.(!empty($uniqueID) ? $uniqueID : '-').')';
                                }
                                $results[$i]['id'] = $user->ID;
                                $results[$i]['label'] = $user->data->display_name.$namePostFix;
                                $user_data = get_user_meta($user->ID, 'basic_data', true);
                                if ($user_data) {
                                    $user_data = json_decode($user_data);
                                    $results[$i]['timeSlot'] = isset($user_data->time_slot) ? $user_data->time_slot : "";
                                }
                                $i++ ;
                            }
                        }

                        break;

                    default:
                        $results = [];
                }

                $data['status'] = true;
                $data['message'] = esc_html__('Datatype found', 'kc-lang');
                $data['data'] = $results;
            }
            echo json_encode($data);
        }

    }
}

