<?php

use App\baseClasses\KCBase;
use App\models\KCAppointment;
use App\models\KCAppointmentServiceMapping;
use App\models\KCClinicSession;
use App\models\KCCustomField;
use App\models\KCCustomFieldData;
use App\models\KCDoctorClinicMapping;
use App\models\KCPatientEncounter;
use App\models\KCService;

function kcUpdateFields($table_name,$new_fields){
    foreach ($new_fields as $key => $nf){
        $new_field = "ALTER TABLE `{$table_name}` ADD `{$key}` {$nf};";
        maybe_add_column($table_name,$key,$new_field);
    }
}

function kcValidateRequest($rules, $request, $message = [])
{
    $error_messages = [];
    $required_message = __(' field is required','kc-lang');
    $email_message = __(' has invalid email address','kc-lang');
    if (count($rules)) {
        foreach ($rules as $key => $rule) {
            if (strpos($rule, '|') !== false) {
                $ruleArray = explode('|', $rule);
                foreach ($ruleArray as $r) {
                    if ($r === 'required') {
                        if (!isset($request[$key])) {
                            $error_messages[] = isset($message[$key]) ? $message[$key] : str_replace('_', ' ', $key) . $required_message;
                        }
                    } elseif ($r === 'email') {
                        if (isset($request[$key])) {
                            if (!filter_var($request[$key], FILTER_VALIDATE_EMAIL)) {
                                $error_messages[] = isset($message[$key]) ? $message[$key] : str_replace('_', ' ', $key) . $email_message;
                            }
                        }
                    }
                }
            } else {
                if ($rule === 'required') {
                    if (!isset($request[$key])) {
                        $error_messages[] = isset($message[$key]) ? $message[$key] : str_replace('_', '', $key) . $required_message;
                    }
                } elseif (!empty($r) && $r === 'email') {
                    if (isset($request[$key])) {
                        if (!filter_var($request[$key], FILTER_VALIDATE_EMAIL)) {
                            $error_messages[] = isset($message[$key]) ? $message[$key] : str_replace('_', ' ', $key) . $email_message;
                        }
                    }
                }
            }

        }
    }

    return $error_messages;
}

function kcRecursiveSanitizeTextField($array)
{
    $filterParameters = [];
    foreach ($array as $key => $value) {

        if ($value === '') {
            $filterParameters[$key] = null;
        } else {
            if (is_array($value)) {
                $filterParameters[$key] = kcRecursiveSanitizeTextField($value);
            } else {
                if(is_object($value)){
                    $filterParameters[$key] = $value;
                }
                else if (preg_match("/<[^<]+>/", $value, $m) !== 0) {
                    $filterParameters[$key] = $value;
                }elseif (strpos(strtolower($key), 'email') !== false){
                    $filterParameters[$key] = sanitize_email($value);
                }
                elseif(strpos(strtolower($key), '_ajax_nonce') !== false){
                    $filterParameters[$key] = sanitize_key($value);
                }
                else {
                    $filterParameters[$key] = sanitize_text_field($value);
                }
            }
        }

    }

    return $filterParameters;
}

function kcRemoveBlankKeyFromArray($data)
{
    foreach($data as $key => $value) {
        if($key === null || $key === '' ) {
            unset($data[$key]);
        }
    }
    return  $data ;
}

function kcGenerateString($length_of_string = 10)
{
    // String of all alphanumeric character
    $str_result = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
    return substr(str_shuffle($str_result),0, $length_of_string);
}

function kcGenerateUsername($first_name)
{

    if (empty($first_name)) {
        return "";
    }
    $randomString = kcGenerateString(6);
    $first_name = str_replace(' ', '_', $first_name);
    return $first_name . '_' . $randomString;

}

function kcDebugLog($log, $module = 'Module -1') {

    $file_log = $log;
    if(gettype($file_log) !== 'string') {
        $file_log = json_encode($file_log) ;
    }
    $log_detail = array(
        'Log Module' => $module,
        'Variable Type' =>  gettype($log)
    );
    if(file_exists( KIVI_CARE_DIR . '/log/kivicare_log.txt')){
        error_log(
            "\n" . json_encode($log_detail) .
            "\n" . $file_log .
            "\n" . "----------------------------------------------",
            3,
            KIVI_CARE_DIR . '/log/kivicare_log.txt');
    }else{
        fopen(KIVI_CARE_DIR . '/log/kivicare_log.txt', 'w');

        error_log(
            "\n" . json_encode($log_detail) .
            "\n" . $file_log .
            "\n" . "----------------------------------------------",
            3,
            KIVI_CARE_DIR . '/log/kivicare_log.txt');
    }
}

function kcGetUserData($user_id) {

    $userObj = new WP_User($user_id);
    $user = $userObj->data;
    $user_data = get_user_meta($userObj->ID, 'basic_data', true);
    if ($user_data) {
        $user_data = json_decode($user_data);
        $user->basicData = $user_data;
    }

    unset($user->user_pass);
    return $user;
}

function kcGetSetupWizardOptions()
{
    return collect([
        [
            'icon' => "fa fa-info fa-lg",
            'name' => "getting_started",
            'title' => __("Welcome","kc-lang"),
            'subtitle' => "",
            'prevStep' => '',
            'routeName' => 'setup.step1',
            'nextStep' => 'setup.step3',
            'completed' => false
        ],
        [
            'icon' => "fa fa-clinic-medical fa-lg",
            'name' => "clinic",
            'title' => __("Clinic Detail","kc-lang"),
            'prevStep' => 'setup.step1',
            'routeName' => 'setup.step3',
            'nextStep' => 'setup.clinic.admin',
            'subtitle' => "",
            'completed' => false
        ],
        [
            'icon' => "fa fa-user fa-lg",
            'name' => "clinic_admin",
            'title' => __("Clinic Admin","kc-lang"),
            'prevStep' => 'setup.step3',
            'routeName' => 'setup.clinic.admin',
            'nextStep' => 'setup.step6',
            'subtitle' => "",
            'completed' => false
        ]
    ]);
}

function kcGetModules()
{
    $prefix = KIVI_CARE_PREFIX;
    $modules = get_option($prefix . 'modules');
    if ($modules) {
        return json_decode($modules);
    } else {
        return '';
    }
}

function kcGetStepConfig()
{
    $prefix = KIVI_CARE_PREFIX;
    $modules = get_option($prefix . 'setup_config');
    if ($modules) {
        return json_decode($modules);
    } else {
        return '';
    }
}

function kcCountryCurrencyList ($search = '') {
    return array(
        'AED' =>  'United Arab Emirates dirham' ,
        'AFN' =>  'Afghan afghani' ,
        'ALL' =>  'Albanian lek' ,
        'AMD' =>  'Armenian dram' ,
        'ANG' =>  'Netherlands Antillean guilder' ,
        'AOA' =>  'Angolan kwanza' ,
        'ARS' =>  'Argentine peso' ,
        'AUD' =>  'Australian dollar' ,
        'AWG' =>  'Aruban florin' ,
        'AZN' =>  'Azerbaijani manat' ,
        'BAM' =>  'Bosnia and Herzegovina convertible mark' ,
        'BBD' =>  'Barbadian dollar' ,
        'BDT' =>  'Bangladeshi taka' ,
        'BGN' =>  'Bulgarian lev' ,
        'BHD' =>  'Bahraini dinar' ,
        'BIF' =>  'Burundian franc' ,
        'BMD' =>  'Bermudian dollar' ,
        'BND' =>  'Brunei dollar' ,
        'BOB' =>  'Bolivian boliviano' ,
        'BRL' =>  'Brazilian real' ,
        'BSD' =>  'Bahamian dollar' ,
        'BTN' =>  'Bhutanese ngultrum' ,
        'BWP' =>  'Botswana pula' ,
        'BYR' =>  'Belarusian ruble (old)' ,
        'BYN' =>  'Belarusian ruble' ,
        'BZD' =>  'Belize dollar' ,
        'CAD' =>  'Canadian dollar' ,
        'CDF' =>  'Congolese franc' ,
        'CHF' =>  'Swiss franc' ,
        'CLP' =>  'Chilean peso' ,
        'CNY' =>  'Chinese yuan' ,
        'COP' =>  'Colombian peso' ,
        'CRC' =>  'Costa Rican col&oacute;n' ,
        'CUC' =>  'Cuban convertible peso' ,
        'CUP' =>  'Cuban peso' ,
        'CVE' =>  'Cape Verdean escudo' ,
        'CZK' =>  'Czech koruna' ,
        'DJF' =>  'Djiboutian franc' ,
        'DKK' =>  'Danish krone' ,
        'DOP' =>  'Dominican peso' ,
        'DZD' =>  'Algerian dinar' ,
        'EGP' =>  'Egyptian pound' ,
        'ERN' =>  'Eritrean nakfa' ,
        'ETB' =>  'Ethiopian birr' ,
        'EUR' =>  'Euro' ,
        'FJD' =>  'Fijian dollar' ,
        'FKP' =>  'Falkland Islands pound' ,
        'GBP' =>  'Pound sterling' ,
        'GEL' =>  'Georgian lari' ,
        'GGP' =>  'Guernsey pound' ,
        'GHS' =>  'Ghana cedi' ,
        'GIP' =>  'Gibraltar pound' ,
        'GMD' =>  'Gambian dalasi' ,
        'GNF' =>  'Guinean franc' ,
        'GTQ' =>  'Guatemalan quetzal' ,
        'GYD' =>  'Guyanese dollar' ,
        'HKD' =>  'Hong Kong dollar' ,
        'HNL' =>  'Honduran lempira' ,
        'HRK' =>  'Croatian kuna' ,
        'HTG' =>  'Haitian gourde' ,
        'HUF' =>  'Hungarian forint' ,
        'IDR' =>  'Indonesian rupiah' ,
        'ILS' =>  'Israeli new shekel' ,
        'IMP' =>  'Manx pound' ,
        'INR' =>  'Indian rupee' ,
        'IQD' =>  'Iraqi dinar' ,
        'IRR' =>  'Iranian rial' ,
        'IRT' =>  'Iranian toman' ,
        'ISK' =>  'Icelandic kr&oacute;na' ,
        'JEP' =>  'Jersey pound' ,
        'JMD' =>  'Jamaican dollar' ,
        'JOD' =>  'Jordanian dinar' ,
        'JPY' =>  'Japanese yen' ,
        'KES' =>  'Kenyan shilling' ,
        'KGS' =>  'Kyrgyzstani som' ,
        'KHR' =>  'Cambodian riel' ,
        'KMF' =>  'Comorian franc' ,
        'KPW' =>  'North Korean won' ,
        'KRW' =>  'South Korean won' ,
        'KWD' =>  'Kuwaiti dinar' ,
        'KYD' =>  'Cayman Islands dollar' ,
        'KZT' =>  'Kazakhstani tenge' ,
        'LAK' =>  'Lao kip' ,
        'LBP' =>  'Lebanese pound' ,
        'LKR' =>  'Sri Lankan rupee' ,
        'LRD' =>  'Liberian dollar' ,
        'LSL' =>  'Lesotho loti' ,
        'LYD' =>  'Libyan dinar' ,
        'MAD' =>  'Moroccan dirham' ,
        'MDL' =>  'Moldovan leu' ,
        'MGA' =>  'Malagasy ariary' ,
        'MKD' =>  'Macedonian denar' ,
        'MMK' =>  'Burmese kyat' ,
        'MNT' =>  'Mongolian t&ouml;gr&ouml;g' ,
        'MOP' =>  'Macanese pataca' ,
        'MRU' =>  'Mauritanian ouguiya' ,
        'MUR' =>  'Mauritian rupee' ,
        'MVR' =>  'Maldivian rufiyaa' ,
        'MWK' =>  'Malawian kwacha' ,
        'MXN' =>  'Mexican peso' ,
        'MYR' =>  'Malaysian ringgit' ,
        'MZN' =>  'Mozambican metical' ,
        'NAD' =>  'Namibian dollar' ,
        'NGN' =>  'Nigerian naira' ,
        'NIO' =>  'Nicaraguan c&oacute;rdoba' ,
        'NOK' =>  'Norwegian krone' ,
        'NPR' =>  'Nepalese rupee' ,
        'NZD' =>  'New Zealand dollar' ,
        'OMR' =>  'Omani rial' ,
        'PAB' =>  'Panamanian balboa' ,
        'PEN' =>  'Sol' ,
        'PGK' =>  'Papua New Guinean kina' ,
        'PHP' =>  'Philippine peso' ,
        'PKR' =>  'Pakistani rupee' ,
        'PLN' =>  'Polish z&#x142;oty' ,
        'PRB' =>  'Transnistrian ruble' ,
        'PYG' =>  'Paraguayan guaran&iacute;' ,
        'QAR' =>  'Qatari riyal' ,
        'RON' =>  'Romanian leu' ,
        'RSD' =>  'Serbian dinar' ,
        'RUB' =>  'Russian ruble' ,
        'RWF' =>  'Rwandan franc' ,
        'SAR' =>  'Saudi riyal' ,
        'SBD' =>  'Solomon Islands dollar' ,
        'SCR' =>  'Seychellois rupee' ,
        'SDG' =>  'Sudanese pound' ,
        'SEK' =>  'Swedish krona' ,
        'SGD' =>  'Singapore dollar' ,
        'SHP' =>  'Saint Helena pound' ,
        'SLL' =>  'Sierra Leonean leone' ,
        'SOS' =>  'Somali shilling' ,
        'SRD' =>  'Surinamese dollar' ,
        'SSP' =>  'South Sudanese pound' ,
        'STN' =>  'S&atilde;o Tom&eacute; and Pr&iacute;ncipe dobra' ,
        'SYP' =>  'Syrian pound' ,
        'SZL' =>  'Swazi lilangeni' ,
        'THB' =>  'Thai baht' ,
        'TJS' =>  'Tajikistani somoni' ,
        'TMT' =>  'Turkmenistan manat' ,
        'TND' =>  'Tunisian dinar' ,
        'TOP' =>  'Tongan pa&#x2bb;anga' ,
        'TRY' =>  'Turkish lira' ,
        'TTD' =>  'Trinidad and Tobago dollar' ,
        'TWD' =>  'New Taiwan dollar' ,
        'TZS' =>  'Tanzanian shilling' ,
        'UAH' =>  'Ukrainian hryvnia' ,
        'UGX' =>  'Ugandan shilling' ,
        'USD' =>  'United States (US) dollar' ,
        'UYU' =>  'Uruguayan peso' ,
        'UZS' => 'Uzbekistani som' ,
        'VEF' => 'Venezuelan bol&iacute;var' ,
        'VES' => 'Bol&iacute;var soberano' ,
        'VND' => 'Vietnamese &#x111;&#x1ed3;ng' ,
        'VUV' => 'Vanuatu vatu' ,
        'WST' => 'Samoan t&#x101;l&#x101;' ,
        'XAF' => 'Central African CFA franc' ,
        'XCD' => 'East Caribbean dollar' ,
        'XOF' => 'West African CFA franc' ,
        'XPF' => 'CFP franc' ,
        'YER' => 'Yemeni rial' ,
        'ZAR' => 'South African rand' ,
        'ZMW' => 'Zambian kwacha' ,
    ) ;
}

function kcCountryCurrencySymbolsList() {
    return array(
        'AED' => '&#x62f;.&#x625;',
        'AFN' => '&#x60b;',
        'ALL' => 'L',
        'AMD' => 'AMD',
        'ANG' => '&fnof;',
        'AOA' => 'Kz',
        'ARS' => '&#36;',
        'AUD' => '&#36;',
        'AWG' => 'Afl.',
        'AZN' => 'AZN',
        'BAM' => 'KM',
        'BBD' => '&#36;',
        'BDT' => '&#2547;&nbsp;',
        'BGN' => '&#1083;&#1074;.',
        'BHD' => '.&#x62f;.&#x628;',
        'BIF' => 'Fr',
        'BMD' => '&#36;',
        'BND' => '&#36;',
        'BOB' => 'Bs.',
        'BRL' => '&#82;&#36;',
        'BSD' => '&#36;',
        'BTC' => '&#3647;',
        'BTN' => 'Nu.',
        'BWP' => 'P',
        'BYR' => 'Br',
        'BYN' => 'Br',
        'BZD' => '&#36;',
        'CAD' => '&#36;',
        'CDF' => 'Fr',
        'CHF' => '&#67;&#72;&#70;',
        'CLP' => '&#36;',
        'CNY' => '&yen;',
        'COP' => '&#36;',
        'CRC' => '&#x20a1;',
        'CUC' => '&#36;',
        'CUP' => '&#36;',
        'CVE' => '&#36;',
        'CZK' => '&#75;&#269;',
        'DJF' => 'Fr',
        'DKK' => 'DKK',
        'DOP' => 'RD&#36;',
        'DZD' => '&#x62f;.&#x62c;',
        'EGP' => 'EGP',
        'ERN' => 'Nfk',
        'ETB' => 'Br',
        'EUR' => '&euro;',
        'FJD' => '&#36;',
        'FKP' => '&pound;',
        'GBP' => '&pound;',
        'GEL' => '&#x20be;',
        'GGP' => '&pound;',
        'GHS' => '&#x20b5;',
        'GIP' => '&pound;',
        'GMD' => 'D',
        'GNF' => 'Fr',
        'GTQ' => 'Q',
        'GYD' => '&#36;',
        'HKD' => '&#36;',
        'HNL' => 'L',
        'HRK' => 'kn',
        'HTG' => 'G',
        'HUF' => '&#70;&#116;',
        'IDR' => 'Rp',
        'ILS' => '&#8362;',
        'IMP' => '&pound;',
        'INR' => '&#8377;',
        'IQD' => '&#x639;.&#x62f;',
        'IRR' => '&#xfdfc;',
        'IRT' => '&#x062A;&#x0648;&#x0645;&#x0627;&#x0646;',
        'ISK' => 'kr.',
        'JEP' => '&pound;',
        'JMD' => '&#36;',
        'JOD' => '&#x62f;.&#x627;',
        'JPY' => '&yen;',
        'KES' => 'KSh',
        'KGS' => '&#x441;&#x43e;&#x43c;',
        'KHR' => '&#x17db;',
        'KMF' => 'Fr',
        'KPW' => '&#x20a9;',
        'KRW' => '&#8361;',
        'KWD' => '&#x62f;.&#x643;',
        'KYD' => '&#36;',
        'KZT' => '&#8376;',
        'LAK' => '&#8365;',
        'LBP' => '&#x644;.&#x644;',
        'LKR' => '&#xdbb;&#xdd4;',
        'LRD' => '&#36;',
        'LSL' => 'L',
        'LYD' => '&#x644;.&#x62f;',
        'MAD' => '&#x62f;.&#x645;.',
        'MDL' => 'MDL',
        'MGA' => 'Ar',
        'MKD' => '&#x434;&#x435;&#x43d;',
        'MMK' => 'Ks',
        'MNT' => '&#x20ae;',
        'MOP' => 'P',
        'MRU' => 'UM',
        'MUR' => '&#x20a8;',
        'MVR' => '.&#x783;',
        'MWK' => 'MK',
        'MXN' => '&#36;',
        'MYR' => '&#82;&#77;',
        'MZN' => 'MT',
        'NAD' => 'N&#36;',
        'NGN' => '&#8358;',
        'NIO' => 'C&#36;',
        'NOK' => '&#107;&#114;',
        'NPR' => '&#8360;',
        'NZD' => '&#36;',
        'OMR' => '&#x631;.&#x639;.',
        'PAB' => 'B/.',
        'PEN' => 'S/',
        'PGK' => 'K',
        'PHP' => '&#8369;',
        'PKR' => '&#8360;',
        'PLN' => '&#122;&#322;',
        'PRB' => '&#x440;.',
        'PYG' => '&#8370;',
        'QAR' => '&#x631;.&#x642;',
        'RMB' => '&yen;',
        'RON' => 'lei',
        'RSD' => '&#1088;&#1089;&#1076;',
        'RUB' => '&#8381;',
        'RWF' => 'Fr',
        'SAR' => '&#x631;.&#x633;',
        'SBD' => '&#36;',
        'SCR' => '&#x20a8;',
        'SDG' => '&#x62c;.&#x633;.',
        'SEK' => '&#107;&#114;',
        'SGD' => '&#36;',
        'SHP' => '&pound;',
        'SLL' => 'Le',
        'SOS' => 'Sh',
        'SRD' => '&#36;',
        'SSP' => '&pound;',
        'STN' => 'Db',
        'SYP' => '&#x644;.&#x633;',
        'SZL' => 'L',
        'THB' => '&#3647;',
        'TJS' => '&#x405;&#x41c;',
        'TMT' => 'm',
        'TND' => '&#x62f;.&#x62a;',
        'TOP' => 'T&#36;',
        'TRY' => '&#8378;',
        'TTD' => '&#36;',
        'TWD' => '&#78;&#84;&#36;',
        'TZS' => 'Sh',
        'UAH' => '&#8372;',
        'UGX' => 'UGX',
        'USD' => '&#36;',
        'UYU' => '&#36;',
        'UZS' => 'UZS',
        'VEF' => 'Bs F',
        'VES' => 'Bs.S',
        'VND' => '&#8363;',
        'VUV' => 'Vt',
        'WST' => 'T',
        'XAF' => 'CFA',
        'XCD' => '&#36;',
        'XOF' => 'CFA',
        'XPF' => 'Fr',
        'YER' => '&#xfdfc;',
        'ZAR' => '&#82;',
        'ZMW' => 'ZK',
    ) ;
}

function kcGetAllWeeks($year){

    $date = new DateTime;
    $date->setISODate($year, 53);

    $weeks = ($date->format("W") === "53" ? 53 : 52);
    $data = [];

    for($x=1; $x<=$weeks; $x++){
        $dto = new DateTime();
        $dates['week_start'] = $dto->setISODate($year, $x)->format('Y-m-d');
        $dates['week_end']   = $dto->modify('+6 days')->format('Y-m-d');
        if($x<10) {
            $x = '0'.$x;
        }
        $data[date('m', strtotime($dates['week_start']))][$x] =  $dates;
    }
    return $data;
}

function kcGetAllMonth() {
    $month    = [];
    $monthsArray = kcMonthsTranslate();
    for($i=1;$i<13;$i++) {
        $date = strtotime('2021-'.$i.'-01');
        $month[date('m',$date)] = !empty($monthsArray[date('F',$date)]) ? $monthsArray[date('F',$date)] : date('F',$date);
    }
    return $month;
}

function kcMonthsTranslate(){
    return array(
        'January' =>  esc_html__('January', 'kc-lang'),
        'February' =>  esc_html__('February', 'kc-lang'),
        'March' =>  esc_html__('March', 'kc-lang'),
        'April' =>  esc_html__('April', 'kc-lang'),
        'May' =>  esc_html__('May', 'kc-lang'),
        'June' =>  esc_html__('June', 'kc-lang'),
        'July' =>  esc_html__('July', 'kc-lang'),
        'August' =>  esc_html__('August', 'kc-lang'),
        'September' =>  esc_html__('September', 'kc-lang'),
        'October' =>  esc_html__('October', 'kc-lang'),
        'November' =>  esc_html__('November', 'kc-lang'),
        'December' =>  esc_html__('December', 'kc-lang')
    );
}

function kcMonthsWeeksArray($month){
    $year =date('Y');
    $totalDays = cal_days_in_month(CAL_GREGORIAN, $month, $year);
    $list = $weeks = [];
    for($d=1; $d<=$totalDays; $d++)
    {
        $time=mktime(12, 0, 0, $month, $d, $year);
        if (date('m', $time)==$month){
            $list[]=date('Y-m-d', $time);
        }
    }
    if(!empty($list) && count($list) > 0){
        $weeks = array_chunk($list,7);
    }
    return $weeks;
}

function kvSaveCustomFields($module_type, $module_id, $data)
{
    $module_id = !empty($module_id) ? (int)$module_id : '';
    $customFieldData = new KCCustomFieldData();
    $data = kcRemoveBlankKeyFromArray($data);
    foreach($data as $key => $value) {
        $field_id = str_replace("custom_field_","",$key);

        $fieldObj = $customFieldData->get_by(['module_type' => $module_type, 'module_id' => $module_id,'field_id'=>$field_id], '=', true);
        if(gettype($value) === 'array'){
            $value  = json_encode($value);
        }
        $temp = [
            'module_type' => $module_type,
            'module_id' => $module_id,
            'fields_data' => $value,
            'field_id'=>(int)$field_id
        ];
        if (empty($fieldObj)) {
            $customFieldData->insert($temp);
        } else {
            $customFieldData->update($temp, ['id' => (int)$fieldObj->id]);
        }
    }
}

function kcGetCustomFields($module_type, $module_id, $data_module_id = 0,$exclude_null_field_data = false)
{
    global  $wpdb;
    $module_id = (int)$module_id;
    $user_id = get_current_user_id();
    $userObj = new WP_User($user_id);
    $data = [];
    $id = '';
    $custom_field_table =  $wpdb->prefix.'kc_custom_fields';
    $custom_field_data_table =  $wpdb->prefix.'kc_custom_fields_data';
    $type = "'$module_type'";

    if($module_type === 'doctor_module'){
        $query = "SELECT p.*, u.fields_data " .
            "FROM {$custom_field_table} AS p " .
            "LEFT JOIN (SELECT * FROM {$custom_field_data_table} WHERE module_id=".$module_id." ) AS u ON p.id = u.field_id WHERE p.status !=0 AND p.module_type =" .$type;
    }
    if($module_type === 'patient_module'){
        if(current_user_can('administrator')){
            $id = "AND p.module_id =0";
        }if($userObj->roles[0] == 'kiviCare_doctor'){
            $id = "AND p.module_id IN($user_id,0)";
        }
        $query = "SELECT p.*, u.fields_data " .
        "FROM {$custom_field_table} AS p " .
        "LEFT JOIN (SELECT * FROM {$custom_field_data_table} WHERE module_id=".$module_id." ) AS u ON p.id = u.field_id WHERE p.status !=0 AND p.module_type =" .$type . $id;
    }
    if($module_type === 'appointment_module'){
        if(current_user_can('administrator')){
            $id = "AND p.module_id =0";
        }if(!empty($userObj->roles) && $userObj->roles[0] == 'kiviCare_doctor'){
            $id = "AND p.module_id IN($user_id,0)";
        }
        $nullCondition = '';
        if($exclude_null_field_data){
            $nullCondition = ' AND u.fields_data != "" ';
        }
        $query = "SELECT p.*, u.fields_data " .
            "FROM {$custom_field_table} AS p " .
            "LEFT JOIN (SELECT * FROM {$custom_field_data_table} WHERE module_id=".$module_id." ) AS u ON p.id = u.field_id WHERE p.module_type =" .$type . 
            " AND p.module_id IN($data_module_id,0) {$nullCondition}";
    }
    if($module_type === 'patient_encounter_module'){
        
        if($userObj->roles[0] == 'kiviCare_doctor'){
            $id = " AND p.module_id IN($user_id,0)";
        }
        $query = "SELECT p.*, u.fields_data " .
            "FROM {$custom_field_table} AS p " .
            "LEFT JOIN (SELECT * FROM {$custom_field_data_table} WHERE module_id=".$module_id." ) AS u ON p.id = u.field_id WHERE p.module_type =" .$type .$id;
    }
    $customData =  $wpdb->get_results($query);

    if(empty($customData)){
        return $data;
    }
    $fields= [];
    if ($customData !== []) {
        foreach ($customData as $value){
            $fields[] = array_merge(json_decode($value->fields,true), ['field_data'=> $value->fields_data],['id'=> $value->id]);
        }
        $data = $fields;
    }
    if ($data === [] || count($customData) === 0) {
        $customField = (new KCCustomField())->get_by([ 'module_type' => $module_type], '=', true);
        if ($customField !== []) {
            $fields = $customField;
            foreach ($fields as $key => $field) {
                $field_detail = json_decode($field->fields) ;
                if ($field_detail->type === "checkbox") {
                    $data[][$field_detail->name] = [];
                } else {
                    $data[][$field_detail->name] = "";
                }
            }
        }
    }

    if(is_array($data) && count($data) > 0){
        $data = array_map(function ($v){
            if(!empty($v['type']) && $v['type'] === 'checkbox'){
                $v['field_data'] = json_decode($v['field_data']);
            }
            return $v;
        },$data);
    }

    return $data;
}

function kcGetCustomFieldsList( $module_type, $doctor_id ){

    global $wpdb;
    $condition = '';
    if($module_type == 'appointment_module' ){
        $condition = ' AND module_id IN ( 0 ,'.(int)$doctor_id.')';
    }
    $query = "SELECT * FROM {$wpdb->prefix}kc_custom_fields WHERE status = 1 AND module_type = '".esc_sql($module_type)."' {$condition}";
    $custom_module = $wpdb->get_results($query);

    $fields = [];
    if (count($custom_module) > 0) {
        foreach ($custom_module as $key => $value) {
            $field_data = '';
            if (!empty($value->fields_data)) {
                $temp = json_decode($value->fields);
                if ($temp->type != null
                    && $temp->type === 'checkbox') {
                    $value->fields_data = $temp;
                }
                $field_data = $value->fields_data;
            }
            $fields[] = array_merge(json_decode($value->fields, true), ['field_data' => $field_data], ['id' => $value->id]);
        }
    }

    $divopen = true;
    if (!empty($fields)) {
        foreach ($fields as $keynew => $customField) {
            ?>
            <?php if ($keynew % 2 === 0) {
                $divClose = true;
                ?>
                <div class="kivi-row">
                <?php
            } ?>
            <div class="form-group kivi-col-6 mt-2">
                <label class="form-label"
                       for="<?php echo esc_html($customField['name'] . '_' . $customField['id']); ?>"
                       class="form-control-label mb-2">

                    <?php echo esc_html($customField['label']); ?>

                    <?php

                    if ($customField['isRequired'] === 1 || $customField['isRequired'] === '1') {
                        ?>
                        <span >*</span>
                        <?php
                    }
                    ?>
                </label>
                <?php

                if ($customField['type'] === 'text' || $customField['type'] === 'number' || $customField['type'] === 'calendar') {
                    ?>
                    <input
                            id="<?php echo esc_html($customField['name'] . '_' . $customField['id']); ?>"
                            placeholder="<?php echo esc_html($customField['placeholder']); ?>"
                            name="custom_field_<?php echo esc_html($customField['id']); ?>"
                            type="<?php echo esc_html($customField['type'] === 'calendar' ? 'date' : $customField['type']); ?>"
                            class="iq-kivicare-form-control <?php echo esc_html(in_array($customField['isRequired'] ,[1,'1']) ? 'kivicare-required' :''); ?>"
                    />
                    <?php
                }

                if ($customField['type'] === 'select') {
                    ?>
                    <select
                            id="<?php echo esc_html($customField['name'] . '_' . $customField['id']); ?>"
                            class="iq-kivicare-form-control text-capitalize <?php echo esc_html(in_array($customField['isRequired'] ,[1,'1']) ? 'kivicare-required' :''); ?>"
                            name="custom_field_<?php echo esc_html($customField['id']); ?>" >
                        <option value=""> <?php echo esc_html__('Select Option', 'kc-lang'); ?> </option>
                        <?php
                        foreach ($customField['options'] as $key => $option) {
                            ?>
                            <option value="<?php echo esc_html($option['text']); ?>"
                                    key="<?php echo esc_html($key); ?>">
                                <?php echo esc_html($option['text']); ?>
                            </option>
                            <?php
                        }
                        ?>
                    </select>
                    <?php
                }

                if ($customField['type'] === 'radio') {
                    ?>
                    <div class="d-flex flex-wrap">
                        <?php

                        foreach ($customField['options'] as $key => $option) {
                            ?>
                            <div class="custom-control custom-radio custom-control-inline">
                                <input
                                        type="radio"
                                        id="<?php echo esc_html($customField['name'] . '_' . $key); ?>"
                                        name="custom_field_<?php echo esc_html($customField['id']); ?>"
                                        value="<?php echo esc_html($option['text']); ?>"
                                        class="custom-control-input <?php echo esc_html(in_array($customField['isRequired'] ,[1,'1']) ? 'kivicare-required' :''); ?>"

                                >
                                <label class="custom-control-label"
                                       for="<?php echo esc_html($customField['name'] . '_' . $key); ?>">
                                    <?php echo esc_html($option['text']); ?>
                                </label>
                            </div>
                            <?php
                        }
                        ?>
                    </div>
                    <?php
                }

                if ($customField['type'] === 'checkbox') {
                    ?>
                    <div class="d-flex flex-wrap justify-content-space " style="gap:5px;">
                        <?php

                        foreach ($customField['options'] as $key => $option) {
                            ?>
                            <div class="custom-control custom-checkbox custom-control-inline d-flex align-items-center" style="gap:5px;">
                                <input
                                        type="checkbox"
                                        id="<?php echo esc_html($customField['name'] . '_' . $key); ?>"
                                        name="custom_field_<?php echo esc_html($customField['id']); ?>"
                                        value="<?php echo esc_html($option['id']); ?>"
                                        class="custom-control-input <?php echo esc_html(in_array($customField['isRequired'] ,[1,'1']) ? 'kivicare-required' :''); ?>"
                                >
                                <label class="custom-control-label"
                                       for="<?php echo esc_html($customField['name'] . '_' . $key); ?>">
                                    <?php echo esc_html($option['text']); ?>
                                </label>
                            </div>
                            <?php
                        }

                        ?>
                    </div>
                    <?php
                }

                if ($customField['type'] === 'textarea') {
                    ?>
                    <textarea
                            id="<?php echo esc_html($customField['name'] . '_' . $customField['id']); ?>"
                            name="custom_field_<?php echo esc_html($customField['id']); ?>"
                            placeholder="<?php echo esc_html($customField['placeholder']); ?>"
                            class="iq-kivicare-form-control <?php echo esc_html(in_array($customField['isRequired'] ,[1,'1']) ? 'kivicare-required' :''); ?>"
                    ></textarea>
                    <?php
                }

                ?>
            </div>
            <?php if ($keynew % 2 !== 0) {
                $divClose = false;
                ?>
                </div>
                <?php
            }
        }
        if($divClose){
            ?>
            </div>
            <?php
        }
    }

}

function kcCheckSetupStatus()
{
    // return false is setup is not complete
    $prefix = KIVI_CARE_PREFIX;
    $modules = get_option($prefix . 'modules');
    $total_steps = get_option('total_setup_steps');
    for ($i = 1; $i <= $total_steps; $i++) {
        $current_step_json = get_option('setup_step_' . $i);
        $current_step_array = json_decode($current_step_json);
        if ($modules['module_config']['name'] === 'receptionist' && $modules['module_config']['status'] === '1') {
            continue;
        }

        if ($current_step_array->status === false && $current_step_array->status === null || !$current_step_array->status === '') {
            return false;
        }
    }

    return true;
}

function kcGetAdminPermissions()
{

    $prefix = KIVI_CARE_PREFIX;

    return collect([

	    'read' => ['name' => 'read', 'status' => 1],
        'dashboard' => ['name' => $prefix . 'dashboard', 'status' => 1],
        'setting' => ['name' => $prefix . 'settings', 'status' => 1],
        'doctor_dashboard' => ['name' => $prefix . 'doctor_dashboard', 'status' => 1],
        'patient_dashboard' => ['name' => $prefix . 'doctor_dashboard', 'status' => 1],

        'doctor_list' => ['name' => $prefix . 'doctor_list', 'status' => 1],
        'doctor_add' => ['name' => $prefix . 'doctor_add', 'status' => 1],
        'doctor_edit' => ['name' => $prefix . 'doctor_edit', 'status' => 1],
        'doctor_view' => ['name' => $prefix . 'doctor_view', 'status' => 1],
        'doctor_delete' => ['name' => $prefix . 'doctor_delete', 'status' => 1],

        'receptionist_list' => ['name' => $prefix . 'receptionist_list', 'status' => 1],
        'receptionist_add' => ['name' => $prefix . 'receptionist_add', 'status' => 1],
        'receptionist_edit' => ['name' => $prefix . 'receptionist_edit', 'status' => 1],
        'receptionist_view' => ['name' => $prefix . 'receptionist_view', 'status' => 1],
        'receptionist_delete' => ['name' => $prefix . 'receptionist_delete', 'status' => 1],

        'patient_list' => ['name' => $prefix . 'patient_list', 'status' => 1],
        'patient_add' => ['name' => $prefix . 'patient_add', 'status' => 1],
        'patient_edit' => ['name' => $prefix . 'patient_edit', 'status' => 1],
        'patient_view' => ['name' => $prefix . 'patient_view', 'status' => 1],
        'patient_delete' => ['name' => $prefix . 'patient_delete', 'status' => 1],

        'clinic_list' => ['name' => $prefix . 'clinic_list', 'status' => 1],
        'clinic_add' => ['name' => $prefix . 'clinic_add', 'status' => 1],
        'clinic_edit' => ['name' => $prefix . 'clinic_edit', 'status' => 1],
        'clinic_view' => ['name' => $prefix . 'clinic_view', 'status' => 1],
        'clinic_delete' => ['name' => $prefix . 'clinic_delete', 'status' => 1],
        'clinic_profile' => ['name' => $prefix . 'clinic_profile', 'status' => 1],

        'appointment_list' => ['name' => $prefix . 'appointment_list', 'status' => 1],
        'appointment_add' => ['name' => $prefix . 'appointment_add', 'status' => 1],
        'appointment_edit' => ['name' => $prefix . 'appointment_edit', 'status' => 1],
        'appointment_view' => ['name' => $prefix . 'appointment_view', 'status' => 1],
        'appointment_delete' => ['name' => $prefix . 'appointment_delete', 'status' => 1],

        'service_list' => ['name' => $prefix . 'service_list', 'status' => 1],
        'service_add' => ['name' => $prefix . 'service_add', 'status' => 1],
        'service_edit' => ['name' => $prefix . 'service_edit', 'status' => 1],
        'service_view' => ['name' => $prefix . 'service_view', 'status' => 1],
        'service_delete' => ['name' => $prefix . 'service_delete', 'status' => 1],

        'static_data_list' => ['name' => $prefix . 'static_data_list', 'status' => 1],
        'static_data_add' => ['name' => $prefix . 'static_data_add', 'status' => 1],
        'static_data_edit' => ['name' => $prefix . 'static_data_edit', 'status' => 1],
        'static_data_view' => ['name' => $prefix . 'static_data_view', 'status' => 1],
        'static_data_delete' => ['name' => $prefix . 'static_data_delete', 'status' => 1],

        'patient_encounters'     => [ 'name' => $prefix . 'patient_encounters', 'status' => 1 ],
        'patient_encounter_list' => ['name' => $prefix . 'patient_encounter_list', 'status' => 1],
        'patient_encounter_add' => ['name' => $prefix . 'patient_encounter_add', 'status' => 1],
        'patient_encounter_edit' => ['name' => $prefix . 'patient_encounter_edit', 'status' => 1],
        'patient_encounter_view' => ['name' => $prefix . 'patient_encounter_view', 'status' => 1],
        'patient_encounter_delete' => ['name' => $prefix . 'patient_encounter_delete', 'status' => 1],

        'patient_appointment_status_change' => ['name' => $prefix . 'patient_appointment_status_change', 'status' => 1],

        'medical_records_list' => ['name' => $prefix . 'medical_records_list', 'status' => 1],
        'medical_records_add' => ['name' => $prefix . 'medical_records_add', 'status' => 1],
        'medical_records_edit' => ['name' => $prefix . 'medical_records_edit', 'status' => 1],
        'medical_records_view' => ['name' => $prefix . 'medical_records_view', 'status' => 1],
        'medical_records_delete' => ['name' => $prefix . 'medical_records_delete', 'status' => 1],

        'prescription_list' => ['name' => $prefix . 'prescription_list', 'status' => 1],
        'prescription_add' => ['name' => $prefix . 'prescription_add', 'status' => 1],
        'prescription_edit' => ['name' => $prefix . 'prescription_edit', 'status' => 1],
        'prescription_view' => ['name' => $prefix . 'prescription_view', 'status' => 1],
        'prescription_delete' => ['name' => $prefix . 'prescription_delete', 'status' => 1],

        'patient_bill_list' => ['name' => $prefix . 'patient_bill_list', 'status' => 1],
        'patient_bill_add' => ['name' => $prefix . 'patient_bill_add', 'status' => 1],
        'patient_bill_edit' => ['name' => $prefix . 'patient_bill_edit', 'status' => 1],
        'patient_bill_view' => ['name' => $prefix . 'patient_bill_view', 'status' => 1],
        'patient_bill_delete' => ['name' => $prefix . 'patient_bill_delete', 'status' => 1],

        'custom_field_list' => ['name' => $prefix . 'custom_field_list', 'status' => 1],
        'custom_field_add' => ['name' => $prefix . 'custom_field_add', 'status' => 1],
        'custom_field_edit' => ['name' => $prefix . 'custom_field_edit', 'status' => 1],
        'custom_field_view' => ['name' => $prefix . 'custom_field_view', 'status' => 1],
        'custom_field_delete' => ['name' => $prefix . 'custom_field_delete', 'status' => 1],

        'terms_condition' => ['name' => $prefix . 'terms_condition', 'status' => 1],
        'clinic_schedule' => ['name' => $prefix . 'clinic_schedule', 'status' => 1],
        'common_settings' => ['name' => $prefix . 'common_settings', 'status' => 1],
        'notification_setting' => ['name' => $prefix . 'notification_setting', 'status' => 1],
        'change_password'=>['name' => $prefix . 'change_password', 'status' => 1],

    ]);
}

function kcGetDoctorPermission() {

    $prefix = KIVI_CARE_PREFIX;

    return collect([

        'read' => ['name' => 'read', 'status' => 1],

        'dashboard' => ['name' => $prefix . 'dashboard', 'status' => 1],

        'settings' => ['name' => $prefix . 'settings', 'status' => 1],

        'doctor_dashboard' => ['name' => $prefix . 'doctor_dashboard', 'status' => 1],
        'doctor_profile' => ['name' => $prefix . 'doctor_profile', 'status' => 1],
        'change_password' => ['name' => $prefix . 'change_password', 'status' => 1],

        'appointment_list' => ['name' => $prefix . 'appointment_list', 'status' => 1],
        'appointment_add' => ['name' => $prefix . 'appointment_add', 'status' => 1],
        'appointment_edit' => ['name' => $prefix . 'appointment_edit', 'status' => 1],
        'appointment_view' => ['name' => $prefix . 'appointment_view', 'status' => 1],
        'appointment_delete' => ['name' => $prefix . 'appointment_delete', 'status' => 1],

        'static_data_list' => ['name' => $prefix . 'static_data_list', 'status' => 1],
        'static_data_add' => ['name' => $prefix . 'static_data_add', 'status' => 1],
        'static_data_edit' => ['name' => $prefix . 'static_data_edit', 'status' => 1],
        'static_data_view' => ['name' => $prefix . 'static_data_view', 'status' => 1],
        'static_data_delete' => ['name' => $prefix . 'static_data_delete', 'status' => 1],

        'doctor_session_add' => ['name' => $prefix . 'doctor_session_add', 'status' => 1],

        'clinic_schedule' => ['name' => $prefix . 'clinic_schedule', 'status' => 1],

        'service_list' => ['name' => $prefix . 'service_list', 'status' => 1],
        'service_add' => ['name' => $prefix . 'service_add', 'status' => 1],
        'service_edit' => ['name' => $prefix . 'service_edit', 'status' => 1],
        'service_view' => ['name' => $prefix . 'service_view', 'status' => 1],
        'service_delete' => ['name' => $prefix . 'service_delete', 'status' => 1],

        'custom_field_list' => ['name' => $prefix . 'custom_field_list', 'status' => 1],
        'custom_field_add' => ['name' => $prefix . 'custom_field_add', 'status' => 1],
        'custom_field_edit' => ['name' => $prefix . 'custom_field_edit', 'status' => 1],
        'custom_field_view' => ['name' => $prefix . 'custom_field_view', 'status' => 1],
        'custom_field_delete' => ['name' => $prefix . 'custom_field_delete', 'status' => 1],

        'patient_encounters'     => [ 'name' => $prefix . 'patient_encounters', 'status' => 1 ],
        'patient_encounter_list' => ['name' => $prefix . 'patient_encounter_list', 'status' => 1],
        'patient_encounter_add' => ['name' => $prefix . 'patient_encounter_add', 'status' => 1],
        'patient_encounter_edit' => ['name' => $prefix . 'patient_encounter_edit', 'status' => 1],
        'patient_encounter_view' => ['name' => $prefix . 'patient_encounter_view', 'status' => 1],
        'patient_encounter_delete' => ['name' => $prefix . 'patient_encounter_delete', 'status' => 1],

        'patient_appointment_status_change' => ['name' => $prefix . 'patient_appointment_status_change', 'status' => 1],

        'patient_list' => ['name' => $prefix . 'patient_list', 'status' => 1],
        'patient_add' => ['name' => $prefix . 'patient_add', 'status' => 1],
        'patient_edit' => ['name' => $prefix . 'patient_edit', 'status' => 1],
        'patient_view' => ['name' => $prefix . 'patient_view', 'status' => 1],
        'patient_delete' => ['name' => $prefix . 'patient_delete', 'status' => 1],
      
        'medical_records_list' => ['name' => $prefix . 'medical_records_list', 'status' => 1],
        'medical_records_add' => ['name' => $prefix . 'medical_records_add', 'status' => 1],
        'medical_records_edit' => ['name' => $prefix . 'medical_records_edit', 'status' => 1],
        'medical_records_view' => ['name' => $prefix . 'medical_records_view', 'status' => 1],
        'medical_records_delete' => ['name' => $prefix . 'medical_records_delete', 'status' => 1],

        'prescription_list' => ['name' => $prefix . 'prescription_list', 'status' => 1],
        'prescription_add' => ['name' => $prefix . 'prescription_add', 'status' => 1],
        'prescription_edit' => ['name' => $prefix . 'prescription_edit', 'status' => 1],
        'prescription_view' => ['name' => $prefix . 'prescription_view', 'status' => 1],
        'prescription_delete' => ['name' => $prefix . 'prescription_delete', 'status' => 1],

        'patient_bill_list' => ['name' => $prefix . 'patient_bill_list', 'status' => 1],
        'patient_bill_add' => ['name' => $prefix . 'patient_bill_add', 'status' => 1],
        'patient_bill_edit' => ['name' => $prefix . 'patient_bill_edit', 'status' => 1],
        'patient_bill_view' => ['name' => $prefix . 'patient_bill_view', 'status' => 1],
        'patient_bill_delete' => ['name' => $prefix . 'patient_bill_delete', 'status' => 1],

    ]);


}

function kcGetPatientPermissions()
{

    $prefix = KIVI_CARE_PREFIX;

    return collect([

        'read' => ['name' => 'read', 'status' => 1],
        'dashboard' => ['name' => $prefix . 'dashboard', 'status' => 1],
        'patient_dashboard' => ['name' => $prefix . 'doctor_dashboard', 'status' => 1],
        'patient_profile' => ['name' => $prefix . 'patient_profile', 'status' => 1],
        'change_password' => ['name' => $prefix . 'change_password', 'status' => 1],

        'service_list' => ['name' => $prefix . 'service_list', 'status' => 1],
        
        'appointment_list' => ['name' => $prefix . 'appointment_list', 'status' => 1],
        'appointment_add' => ['name' => $prefix . 'appointment_add', 'status' => 1],
        'appointment_edit' => ['name' => $prefix . 'appointment_edit', 'status' => 1],
        'appointment_view' => ['name' => $prefix . 'appointment_view', 'status' => 1],
        'appointment_delete' => ['name' => $prefix . 'appointment_delete', 'status' => 1],

		'patient_encounters'     => [ 'name' => $prefix . 'patient_encounters', 'status' => 1 ],
		'patient_encounter_list'   => [ 'name' => $prefix . 'patient_encounter_list', 'status' => 1 ],
		'patient_encounter_add'    => [ 'name' => $prefix . 'patient_encounter_add', 'status' => 1 ],
		'patient_encounter_edit'   => [ 'name' => $prefix . 'patient_encounter_edit', 'status' => 1 ],
		'patient_encounter_view'   => [ 'name' => $prefix . 'patient_encounter_view', 'status' => 1 ],
		'patient_encounter_delete' => [ 'name' => $prefix . 'patient_encounter_delete', 'status' => 1 ],

        'medical_records_list' => ['name' => $prefix . 'medical_records_list', 'status' => 1],
        'medical_records_view' => ['name' => $prefix . 'medical_records_view', 'status' => 1],

        'prescription_list' => ['name' => $prefix . 'prescription_list', 'status' => 1],
        'prescription_view' => ['name' => $prefix . 'prescription_view', 'status' => 1],

        'patient_bill_list' => ['name' => $prefix . 'patient_bill_list', 'status' => 1],
        'patient_bill_view' => ['name' => $prefix . 'patient_bill_view', 'status' => 1],

	] );

}

function kcGetReceptionistPermission()
{

    $prefix = KIVI_CARE_PREFIX;

    return collect([

        'read' => ['name' => 'read', 'status' => 1],

        'settings' => ['name' => $prefix . 'settings', 'status' => 1],

        'dashboard' => ['name' => $prefix . 'dashboard', 'status' => 1],
        'doctor_dashboard' => ['name' => $prefix . 'doctor_dashboard', 'status' => 1],
        'receptionist_profile' => ['name' => $prefix . 'receptionist_profile', 'status' => 1],
        'change_password' => ['name' => $prefix . 'change_password', 'status' => 1],

        'doctor_list' => ['name' => $prefix . 'doctor_list', 'status' => 1],
        'doctor_add' => ['name' => $prefix . 'doctor_add', 'status' => 1],
        'doctor_edit' => ['name' => $prefix . 'doctor_edit', 'status' => 1],
        'doctor_view' => ['name' => $prefix . 'doctor_view', 'status' => 1],
        'doctor_delete' => ['name' => $prefix . 'doctor_delete', 'status' => 1],

        'patient_list' => ['name' => $prefix . 'patient_list', 'status' => 1],
        'patient_add' => ['name' => $prefix . 'patient_add', 'status' => 0],
        'patient_edit' => ['name' => $prefix . 'patient_edit', 'status' => 1],
        'patient_view' => ['name' => $prefix . 'patient_view', 'status' => 1],
        'patient_delete' => ['name' => $prefix . 'patient_delete', 'status' => 1],

        'clinic_list' => ['name' => $prefix . 'clinic_list', 'status' => 1],
        'clinic_add' => ['name' => $prefix . 'clinic_add', 'status' => 1],
        'clinic_edit' => ['name' => $prefix . 'clinic_edit', 'status' => 1],
        'clinic_view' => ['name' => $prefix . 'clinic_view', 'status' => 1],
        'clinic_delete' => ['name' => $prefix . 'clinic_delete', 'status' => 1],
        'clinic_profile' => ['name' => $prefix . 'clinic_profile', 'status' => 1],

        'service_list' => ['name' => $prefix . 'service_list', 'status' => 1],
        'service_add' => ['name' => $prefix . 'service_add', 'status' => 1],
        'service_edit' => ['name' => $prefix . 'service_edit', 'status' => 1],
        'service_view' => ['name' => $prefix . 'service_view', 'status' => 1],
        'service_delete' => ['name' => $prefix . 'service_delete', 'status' => 1],

        'appointment_list' => ['name' => $prefix . 'appointment_list', 'status' => 1],
        'appointment_add' => ['name' => $prefix . 'appointment_add', 'status' => 1],
        'appointment_edit' => ['name' => $prefix . 'appointment_edit', 'status' => 1],
        'appointment_view' => ['name' => $prefix . 'appointment_view', 'status' => 1],
        'appointment_delete' => ['name' => $prefix . 'appointment_delete', 'status' => 1],

        'patient_encounters'     => [ 'name' => $prefix . 'patient_encounters', 'status' => 1 ],
        'patient_encounter_list' => ['name' => $prefix . 'patient_encounter_list', 'status' => 1],
        'patient_encounter_add' => ['name' => $prefix . 'patient_encounter_add', 'status' => 1],
        'patient_encounter_edit' => ['name' => $prefix . 'patient_encounter_edit', 'status' => 1],
        'patient_encounter_view' => ['name' => $prefix . 'patient_encounter_view', 'status' => 1],
        'patient_encounter_delete' => ['name' => $prefix . 'patient_encounter_delete', 'status' => 1],

        'patient_appointment_status_change' => ['name' => $prefix . 'patient_appointment_status_change', 'status' => 1],

        'medical_records_list' => ['name' => $prefix . 'medical_records_list', 'status' => 1],
        'medical_records_add' => ['name' => $prefix . 'medical_records_add', 'status' => 1],
        'medical_records_edit' => ['name' => $prefix . 'medical_records_edit', 'status' => 1],
        'medical_records_view' => ['name' => $prefix . 'medical_records_view', 'status' => 1],
        'medical_records_delete' => ['name' => $prefix . 'medical_records_delete', 'status' => 1],

        'prescription_list' => ['name' => $prefix . 'prescription_list', 'status' => 1],
        'prescription_add' => ['name' => $prefix . 'prescription_add', 'status' => 1],
        'prescription_edit' => ['name' => $prefix . 'prescription_edit', 'status' => 1],
        'prescription_view' => ['name' => $prefix . 'prescription_view', 'status' => 1],
        'prescription_delete' => ['name' => $prefix . 'prescription_delete', 'status' => 1],

        'patient_bill_list' => ['name' => $prefix . 'patient_bill_list', 'status' => 1],
        'patient_bill_add' => ['name' => $prefix . 'patient_bill_add', 'status' => 1],
        'patient_bill_edit' => ['name' => $prefix . 'patient_bill_edit', 'status' => 1],
        'patient_bill_view' => ['name' => $prefix . 'patient_bill_view', 'status' => 1],
        'patient_bill_delete' => ['name' => $prefix . 'patient_bill_delete', 'status' => 1],

        'clinic_schedule' => ['name' => $prefix . 'clinic_schedule', 'status' => 1],

        'static_data_list' => ['name' => $prefix . 'static_data_list', 'status' => 1],
        'static_data_add' => ['name' => $prefix . 'static_data_add', 'status' => 1],
        'static_data_edit' => ['name' => $prefix . 'static_data_edit', 'status' => 1],
        'static_data_view' => ['name' => $prefix . 'static_data_view', 'status' => 1],
        'static_data_delete' => ['name' => $prefix . 'static_data_delete', 'status' => 1],

    ]);
}

function kcCheckPermission($permission_name)
{

    $user_id = get_current_user_id();

    $userObj = (new WP_User($user_id));

    if (in_array(KIVI_CARE_PREFIX . "doctor", $userObj->roles)) {
        $permissions = kcGetDoctorPermission()->toArray();
    } elseif (in_array(KIVI_CARE_PREFIX . "clinic_admin", $userObj->roles)) {
        $permissions = kcGetAdminPermissions()->toArray();
    } elseif (in_array(KIVI_CARE_PREFIX . "patient", $userObj->roles)) {
        $permissions = kcGetPatientPermissions()->toArray();
    } elseif (in_array(KIVI_CARE_PREFIX . "receptionist", $userObj->roles)) {
        $permissions = kcGetReceptionistPermission()->toArray();
    } elseif (in_array("administrator", $userObj->roles)) {
        $permissions = kcGetAdminPermissions()->toArray();
    } else {
        $permissions = collect([])->toArray();
    }

    if (isset($permissions[$permission_name]['name'])) {
        if (current_user_can($permissions[$permission_name]['name'])) {
            return true;
        }
    }

    return false;
}

function kcGetPermission($permission_name)
{
    $permissions = kcGetAdminPermissions()->toArray();

    return $permissions[$permission_name]['name'];
}

function kcGetPluginVersion($pluginDomain){
    if (!function_exists('get_plugins')) {
        include_once ABSPATH . 'wp-admin/includes/plugin.php';
    }
    $plugins = get_plugins();
    foreach ($plugins as $key => $value) {
        if($value['TextDomain'] === $pluginDomain) {
            return $value['Version'];
        }
    }
    return '0' ;
}
function getKiviCareProVersion () {

    return kcGetPluginVersion('kiviCare-clinic-&-patient-management-system-pro');
}

function isKiviCareProActive () {
    return kcCheckPluginActive('kiviCare-clinic-&-patient-management-system-pro');
}

function isKiviCareTelemedActive () {
    return kcCheckPluginActive('kiviCare-telemed-addon');
}

function isKiviCareGoogleMeetActive () {
    return kcCheckPluginActive('kc-googlemeet');
}

function iskcWooCommerceActive () {
    return kcCheckPluginActive('woocommerce');
}

function kcCheckPluginActive($pluginTextDomain){

    if (!function_exists('get_plugins')) {
        include_once ABSPATH . 'wp-admin/includes/plugin.php';
    }

    $plugins = get_plugins();

    foreach ($plugins as $key => $value) {

        if($value['TextDomain'] === $pluginTextDomain) {
            if(is_plugin_active($key)) {
                return true;
            }
        }

    }
    return false ;
}

function kcCommonTemplate($type){
    $prefix = KIVI_CARE_PREFIX;
    $mail_template = $type === "sms" ? $prefix.'sms_tmp' : $prefix.'mail_tmp';
    $data = [
        [
            'post_name' => $prefix.'patient_register',
            'post_content' => '<p>Welcome to KiviCare ,</p><p>Your registration process with {{user_email}} is successfully completed, and your password is  {{user_password}} </p><p>Thank you.</p>',
            'post_title' => 'Patient Registration Template',
            'post_type' => $mail_template,
            'post_status' => 'publish',
        ],
        [
            'post_name' => $prefix.'receptionist_register',
            'post_content' => '<p>Welcome to KiviCare ,</p><p>Your registration process with {{user_email}} is successfully completed, and your password is  {{user_password}} </p><p>Thank you.</p>',
            'post_title' => 'Receptionist Registration Template',
            'post_type' => $mail_template,
            'post_status' => 'publish',
        ],
        [
            'post_name' => $prefix.'doctor_registration',
            'post_content' => '<p>Welcome to KiviCare ,</p><p>You are successfully registered with  </p><p> Your  email:  {{user_email}}  ,  username: {{user_name}} and password: {{user_password}}  </p><p>Thank you.</p>',
            'post_title' => 'Doctor Registration Template',
            'post_type' => $mail_template,
            'post_status' => 'publish',
        ],
        [
            'post_name' => $prefix.'doctor_book_appointment',
            'post_content' => '<p> New appointment </p><p> Your have new appointment on </p><p> Date: {{appointment_date}}  , Time : {{appointment_time}} ,Patient : {{patient_name}} </p><p> Thank you. </p>',
            'post_title' => 'Doctor Booked Appointment Template',
            'post_type' => $mail_template,
            'post_status' => 'publish',
        ],
        [
            'post_name' => $prefix.'resend_user_credential',
            'post_content' => '<p> Welcome to KiviCare ,</p><p> Your kivicare account user credential </p><p> Your  email:  {{user_email}}  ,  username: {{user_name}} and password: {{user_password}}  </p><p>Thank you.</p>',
            'post_title' => 'Resend user credentials',
            'post_type' => $mail_template,
            'post_status' => 'publish',
        ],
        [
            'post_name' => $prefix.'cancel_appointment',
            'post_content' => '<p> Welcome to KiviCare ,</p><p> Your appointment Booking is cancel. </p><p> Date: {{appointment_date}}  , Time : {{appointment_time}}   </p><p>Clinic: {{clinic_name}} Doctor: {{doctor_name}}</p><p>Thank you.</p>',
            'post_title' => 'Cancel appointment',
            'post_type' => $mail_template,
            'post_status' => 'publish',
        ],
        [
            'post_name' => $prefix.'zoom_link',
            'post_content' => '<p> Zoom video conference </p><p> Your have new appointment on </p><p> Date: {{appointment_date}}  , Time : {{appointment_time}} ,Doctor : {{doctor_name}} , Zoom Link : {{zoom_link}} </p><p> Thank you. </p>',
            'post_title' => 'Video Conference appointment Template',
            'post_type' => $mail_template,
            'post_status' => 'publish',
        ],
        [
            'post_name' => $prefix.'add_doctor_zoom_link',
            'post_content' => '<p> Zoom video conference </p><p> Your have new appointment on </p><p> Date: {{appointment_date}}  , Time : {{appointment_time}} ,Patient : {{patient_name}} , Zoom Link : {{add_doctor_zoom_link}} </p><p> Thank you. </p>',
            'post_title' => 'Doctor Zoom Video Conference appointment Template',
            'post_type' => $mail_template,
            'post_status' => 'publish',
        ],
        [
            'post_name' => $prefix.'clinic_admin_registration',
            'post_content' => '<p> Welcome to Clinic, </p><p> You are successfully registered as clinic admin </p><p> Your email:  {{user_email}}  ,  username: {{user_name}} and password: {{user_password}} </p> <p>Thank you.</p>',
            'post_title' => 'Clinic Admin Registration',
            'post_type' => $mail_template,
            'post_status' => 'publish',
        ],
        [
            'post_name' => $prefix.'clinic_book_appointment',
            'post_content' => '<p> New appointment </p><p> New appointment Book on {{current_date}} </p><p> For Date: {{appointment_date}}  , Time : {{appointment_time}} , Patient : {{patient_name}} , Doctor : {{doctor_name}}  </p><p> Thank you. </p>',
            'post_title' => 'Clinic Booked Appointment Template',
            'post_type' => $mail_template,
            'post_status' => 'publish'
        ],
        [
            'post_name' => $prefix.'book_appointment_reminder',
            'post_content' => '<p> Welcome to KiviCare ,</p><p> You Have appointment  on </p><p> {{appointment_date}}  , Time : {{appointment_time}}  </p><p> Thank you. </p>',
            'post_title' => 'Patient Appointment Reminder',
            'post_type' => $mail_template,
            'post_status' => 'publish',
        ],
        [
            'post_name' => $prefix.'user_verified',
            'post_content' => '<p> Your Account Has been Verified By admin On Date: {{current_date}} </p><p> Login Page: {{login_url}} </p><p> Thank you. </p>',
            'post_title' => 'User Verified By Admin',
            'post_type' => $mail_template,
            'post_status' => 'publish',
        ],
        [
            'post_name' => $prefix.'admin_new_user_register',
            'post_content' => '<p> New User Register On site {{site_url}} On Date: {{current_date}} </p><p> Name : {{user_name}} </p><p> Email :{{user_email}} </p><p> Contact No :{{user_contact}} </p><p> User Role :{{user_role}} </p><p> Thank you. </p>',
            'post_title' => 'New User Register On site',
            'post_type' => $mail_template,
            'post_status' => 'publish',
        ],
        [
            'post_name' => $prefix.'meet_link',
            'post_content' => '<p> Google Meet conference </p><p> Your have new appointment on </p><p> Date: {{appointment_date}}  , Time : {{appointment_time}} ,Doctor : {{doctor_name}} , Google Meet Link : {{meet_link}} </p><p> Event Link {{meet_event_link}} </p><p> Thank you. </p>',
            'post_title' => 'Google Meet Video Conference appointment Template',
            'post_type' => $mail_template,
            'post_status' => 'publish',
        ],
        [
            'post_name' => $prefix.'add_doctor_meet_link',
            'post_content' => '<p> Google Meet conference </p><p> Your have new appointment on </p><p> Date: {{appointment_date}}  , Time : {{appointment_time}} ,Patient : {{patient_name}} , Google Meet Link : {{meet_link}} </p><p> Event Link {{meet_event_link}} </p><p> Thank you. </p>',
            'post_title' => 'Doctor Google Meet Video Conference appointment Template',
            'post_type' => $mail_template,
            'post_status' => 'publish',
        ],
        [
            'post_name' => $prefix.'patient_clinic_check_in_check_out',
            'post_content' => '<p> Welcome to KiviCare ,</p><p> New Patient Check In to Clinic </p> <p> Patient: {{patient_name}} </p> <p> Patient Email: {{patient_email}}</p><p> Check In Date: {{current_date}}</p><p> Thank you. </p>',
            'post_title' => 'Patient Clinic In',
            'post_type' => $mail_template,
            'post_status' => 'publish',
        ],
        [
            'post_name' => $mail_template === 'kiviCare_sms_tmp' ? $prefix.'add_appointment' : $prefix.'book_appointment',
            'post_content' => '<p> Welcome to KiviCare ,</p><p> Your appointment has been booked  successfully on </p><p> {{appointment_date}}  , Time : {{appointment_time}}  </p><p> Thank you. </p>',
            'post_title' => 'Patient Appointment Booking Template',
            'post_type' => $mail_template,
            'post_status' => 'publish',
        ],
        [
            'post_name' => $prefix.'payment_pending',
            'post_content' => '<p> Appointment Payment ,</p><p> Your Appointment is cancelled due to pending payment </p><p>Thank you.</p>',
            'post_title' => 'Appointment Payment Pending Template',
            'post_type' => $mail_template,
            'post_status' => 'publish',
        ]
    ];

    if ($mail_template === 'kivicare_sms_tmp') {
        $temp = [
            [
                'post_name' => $prefix . 'encounter_close',
                'post_content' => '<p> Welcome to Clinic ,</p><p> Your appointment has been closed with your total amount {{total_amount}}  </p><p>Thank you.</p>',
                'post_title' => 'Encounter SMS Template',
                'post_type' => $mail_template,
                'post_status' => 'publish',
            ]
        ];

    }else{
        $temp = [
            [
                'post_name' => $prefix . 'patient_report',
                'post_content' => '<p> Welcome to KiviCare ,</p><p> Find your Report in attachment </p><p> Thank you. </p>',
                'post_title' => 'Patient Report',
                'post_type' => $mail_template,
                'post_status' => 'publish',
            ],
            [
                'post_name' => $prefix.'book_prescription',
                'post_content' => '<p> Welcome to KiviCare ,</p><p> You Have Medicine Prescription on </p><p> Clinic : {{clinic_name}}</p><p>Doctor : {{doctor_name}}</p><p>Prescription :{{prescription}} </p><p> Thank you. </p>',
                'post_title' => 'Patient Prescription Reminder',
                'post_type' => $mail_template,
                'post_status' => 'publish',
            ]
        ];
    }

    return array_merge($data, $temp);
}

function kcAddMailSmsPosts($default_template){
    global $wpdb;
    $postTable = $wpdb->prefix . 'posts';
    $query = "SELECT LOWER(CONCAT(post_type,post_name)) AS post_join_name FROM {$postTable}  where post_type IN ('kivicare_sms_tmp','kivicare_mail_tmp','kivicare_gcal_tmp','kivicare_gmeet_tmp')";
    $results = collect($wpdb->get_results($query))->pluck('post_join_name')->toArray();
    foreach ($default_template as $email_template) {
        $post_join_name = strtolower($email_template['post_type'].$email_template['post_name']);
        if (!in_array($post_join_name,$results)) {
            wp_insert_post($email_template);
        }
    }
}

function kcGetEmailSmsDynamicKeys(){
    $data = [
        'kivicare_book_prescription' => [
            '{{prescription}}',
            '{{clinic_name}}',
            '{{clinic_email}}',
            '{{clinic_contact_number}}',
            '{{clinic_address}}',
            '{{doctor_name}}',
            '{{doctor_email}}',
            '{{doctor_contact_number}}',
            '{{current_date}}',
            '{{current_date_time}}',
        ],
        'kivicare_payment_pending' => [
            '{{current_date}}',
            '{{current_date_time}}'
        ],
        'kivicare_meet_link' => [
            '{{appointment_date}}',
            '{{appointment_time}}',
            '{{patient_name}}',
            '{{meet_link}}',
            '{{meet_event_link}}',
            '{{clinic_name}}',
            '{{clinic_email}}',
            '{{clinic_contact_number}}',
            '{{clinic_address}}',
            '{{doctor_name}}',
            '{{doctor_email}}',
            '{{doctor_contact_number}}',
            '{{current_date}}',
            '{{current_date_time}}',
        ],
        'kivicare_add_doctor_meet_link' => [
            '{{appointment_date}}',
            '{{appointment_time}}',
            '{{meet_link}}',
            '{{meet_event_link}}',
            '{{clinic_name}}',
            '{{clinic_email}}',
            '{{clinic_contact_number}}',
            '{{clinic_address}}',
            '{{patient_name}}',
            '{{patient_email}}',
            '{{patient_contact_number}}',
            '{{current_date}}',
            '{{current_date_time}}',
        ],
        'kivicare_book_appointment' => [
            '{{appointment_date}}',
            '{{appointment_time}}',
            '{{service_name}}',
            '{{clinic_name}}',
            '{{clinic_email}}',
            '{{clinic_contact_number}}',
            '{{clinic_address}}',
            '{{doctor_name}}',
            '{{doctor_email}}',
            '{{doctor_contact_number}}',
            '{{current_date}}',
            '{{current_date_time}}',
        ],
        'kivicare_book_appointment_reminder' => [
            '{{appointment_date}}',
            '{{appointment_time}}',
            '{{clinic_name}}',
            '{{clinic_email}}',
            '{{clinic_contact_number}}',
            '{{clinic_address}}',
            '{{doctor_name}}',
            '{{doctor_email}}',
            '{{doctor_contact_number}}',
            '{{current_date}}',
            '{{current_date_time}}',
        ],
        'kivicare_patient_register' => [
            '{{user_email}}',
            '{{user_password}}',
            '{{login_url}}',
            '{{widgets_login_url}}',
            '{{current_date}}',
            '{{current_date_time}}',
            '{{appointment_page_url}}'
        ],
        'kivicare_receptionist_register' => [
            '{{user_email}}',
            '{{user_password}}',
            '{{login_url}}',
            '{{current_date}}',
            '{{current_date_time}}'
        ],
        'kivicare_doctor_registration' => [
            '{{user_email}}',
            '{{user_name}}',
            '{{user_password}}',
            '{{login_url}}',
            '{{current_date}}',
            '{{current_date_time}}'
        ],
        'kivicare_doctor_book_appointment' => [
            '{{appointment_date}}',
            '{{appointment_time}}',
            '{{service_name}}',
            '{{patient_name}}',
            '{{patient_email}}',
            '{{clinic_name}}',
            '{{clinic_email}}',
            '{{clinic_contact_number}}',
            '{{clinic_address}}',
            '{{current_date}}',
            '{{current_date_time}}',
        ],
        'kivicare_resend_user_credential' => [
            '{{user_email}}',
            '{{user_name}}',
            '{{user_password}}',
            '{{login_url}}',
            '{{current_date}}',
            '{{current_date_time}}'
        ],
        'kivicare_cancel_appointment' => [
            '{{appointment_date}}',
            '{{appointment_time}}',
            '{{clinic_name}}',
            '{{clinic_email}}',
            '{{clinic_contact_number}}',
            '{{clinic_address}}',
            '{{doctor_name}}',
            '{{doctor_email}}',
            '{{doctor_contact_number}}',
            '{{current_date}}',
            '{{current_date_time}}',
        ],
        'kivicare_zoom_link' => [
            '{{appointment_date}}',
            '{{appointment_time}}',
            '{{zoom_link}}',
            '{{clinic_name}}',
            '{{clinic_email}}',
            '{{clinic_contact_number}}',
            '{{clinic_address}}',
            '{{doctor_name}}',
            '{{doctor_email}}',
            '{{doctor_contact_number}}',
            '{{current_date}}',
            '{{current_date_time}}',
        ],
        'kivicare_add_doctor_zoom_link' => [
            '{{appointment_date}}',
            '{{appointment_time}}',
            '{{add_doctor_zoom_link}}',
            '{{patient_name}}',
            '{{patient_email}}',
            '{{patient_contact_number}}',
            '{{clinic_name}}',
            '{{clinic_email}}',
            '{{clinic_contact_number}}',
            '{{clinic_address}}',
            '{{current_date}}',
            '{{current_date_time}}',
        ],
        'kivicare_clinic_admin_registration' => [
            '{{user_email}}',
            '{{user_name}}',
            '{{user_password}}',
            '{{login_url}}',
            '{{current_date}}',
            '{{current_date_time}}'
        ],
        'kivicare_clinic_book_appointment' => [
            '{{appointment_date}}',
            '{{appointment_time}}',
            '{{service_name}}',
            '{{patient_name}}',
            '{{patient_email}}',
            '{{patient_contact_number}}',
            '{{doctor_name}}',
            '{{doctor_email}}',
            '{{doctor_contact_number}}',
            '{{current_date}}',
            '{{current_date_time}}',
        ],
        'kivicare_encounter_close' => [
            '{{total_amount}}',
            '{{current_date}}',
            '{{current_date_time}}'
        ],
        'kivicare_patient_report' =>[
            '{{current_date}}',
            '{{current_date_time}}',
        ],
        'kivicare_add_appointment' =>[
            '{{appointment_date}}',
            '{{appointment_time}}',
            '{{current_date}}',
            '{{current_date_time}}',
            '{{clinic_name}}',
            '{{clinic_email}}',
            '{{clinic_contact_number}}',
            '{{clinic_address}}',
            '{{doctor_name}}',
            '{{doctor_email}}',
            '{{doctor_contact_number}}',
        ]

    ];

    return $data;
}

function kcGetNotificationTemplateLists($template_type){
    $prefix = KIVI_CARE_PREFIX;
    $args['post_type'] = strtolower($prefix.$template_type.'_tmp');
    $args['nopaging'] = true;
    $args['post_status'] = 'any' ;
    $template_result = get_posts($args);
    $patient = [
        'kivicare_patient_register',
        'kivicare_book_appointment_reminder',
        'kivicare_book_appointment',
        'kivicare_add_appointment',
        'kivicare_cancel_appointment',
        'kivicare_encounter_close',
        'kivicare_zoom_link',
        'kivicare_meet_link',
        'kivicare_patient_clinic_check_in_check_out',
        'kivicare_patient_report',
        'kivicare_book_prescription',
        'kivicare_encounter_close'
    ];

    $doctor = [
        'kivicare_doctor_registration',
        'kivicare_doctor_book_appointment',
        'kivicare_add_doctor_zoom_link',
        'kivicare_add_doctor_meet_link',
    ];
    $clinic = [
        'kivicare_clinic_admin_registration',
        'kivicare_clinic_book_appointment'
    ];
    $receptionist=[
        'kivicare_receptionist_register'
    ];

    $common = [
        'kivicare_resend_user_credential',
        'kivicare_user_verified',
        'kivicare_admin_new_user_register'
    ];
    $template_result = collect($template_result)->unique('post_title')->sortBy('ID')->map(function ($value) use($patient,
        $doctor,$clinic,$receptionist,$common){
        if(in_array($value->post_name,$patient)){
            $value->user_type = 'patient';
        }
        if(in_array($value->post_name,$doctor)){
            $value->user_type = 'doctor';
        }
        if(in_array($value->post_name,$clinic)){
            $value->user_type = 'clinic';
        }
        if(in_array($value->post_name,$receptionist)){
            $value->user_type = 'receptionist';
        }
        if(in_array($value->post_name,$common)){
            $value->user_type = 'common';
        }
        if(empty($value->user_type)){
            $value->user_type = 'common';
        }
        return $value;
    });

    return  $template_result->groupBy('user_type')->sortKeys();
}
/**
 * // Data param required date
 *
 * @param $data
 *
 * @return bool
 */

function kcSendEmail($data)
{
    
    global $wpdb;
    $args['post_name'] = strtolower(KIVI_CARE_PREFIX.$data['email_template_type']);
    $args['post_type'] = strtolower(KIVI_CARE_PREFIX.'mail_tmp') ;

    $query = "SELECT * FROM {$wpdb->prefix}posts WHERE `post_name` = '" . $args['post_name'] . "' AND `post_type` = '".$args['post_type']."' AND post_status = 'publish' ";
    $check_exist_post = $wpdb->get_row($query, ARRAY_A);

    if (!empty($check_exist_post)) {

        $email_content = $check_exist_post['post_content'];

        $email_content = kcEmailContentKeyReplace($email_content, $data);
        $small_prefix = strtolower(KIVI_CARE_PREFIX);

        switch ($args['post_name']) {
            case $small_prefix.'doctor_registration':
                $email_title = esc_html__('Doctor Registration','kc-lang');
                break;
            case $small_prefix.'patient_registration':
                $email_title = esc_html__('Patient Registration','kc-lang');
                break;
            case $small_prefix.'receptionist_registration':
                $email_title = esc_html__('Receptionist Registration','kc-lang');
                break;
            case $small_prefix.'book_appointment':
                $email_title = esc_html__('Patient Appointment Booking','kc-lang');
                break;
            case $small_prefix.'doctor_book_appointment':
                $email_title = esc_html__('Doctor Appointment Booking','kc-lang');
                break;
            case $small_prefix.'clinic_book_appointment':
                $email_title = esc_html__('Clinic Appointment Booking','kc-lang');
                break;
            case $small_prefix.'zoom_link':
            case $small_prefix.'meet_link':
                $email_title = esc_html__('Patient Telemed Appointment Booking','kc-lang');
                break;
            case $small_prefix. 'clinic_admin_registration':
                $email_title = esc_html__('Clinic Admin Registration','kc-lang');
                break;
            case $small_prefix. 'payment_pending':
                $email_title = esc_html__('Patient Appointment Payment','kc-lang');
                break;
            case $small_prefix. 'add_doctor_zoom_link':
            case $small_prefix. 'add_doctor_meet_link':
                $email_title = esc_html__('Doctor Telemed Appointment Booking','kc-lang');
                break;
            case $small_prefix. 'book_appointment_reminder':
                $email_title = esc_html__('Patient Booked Appointment Reminder','kc-lang');
                break;
            case $small_prefix. 'book_prescription':
                $email_title = esc_html__('Patient Prescription','kc-lang');
                break;
            case $small_prefix. 'cancel_appointment':
                $email_title = esc_html__('Patient Appointment Cancel','kc-lang');
                break;
            case $small_prefix. 'patient_report':
                $email_title = esc_html__('Patient Report','kc-lang');
                break;
            case $small_prefix. 'patient_clinic_check_in_check_out':
                $email_title = esc_html__('Patient Check In','kc-lang');
                break;
            case $small_prefix. 'admin_new_user_register':
                $email_title = esc_html__('Admin New User Register','kc-lang');
                break;
            case $small_prefix. 'user_verified':
                $email_title = esc_html__('Account Verified','kc-lang');
                break;
            default:
                $email_title = esc_html__('Welcome To Clinic ','kc-lang');
        }

        if(isset($data['attachment']) && $data['attachment'] ){
             $email_status = wp_mail($data['user_email'], $email_title, $email_content ,'', isset($data['attachment_file']) ? $data['attachment_file'] : '');
        }else{
             $email_status = wp_mail($data['user_email'], $email_title, $email_content);
        }

        if ($email_status) {
            return true;
        } else {
            return false;
        }
    }
    else {
        return false ;
    }

}

/**
 * // Data param required content
 *
 * @param $content - email content for replace email template key
 *
 * @return string
 *
 */

function kcEmailContentKeyReplace($content, $data)
{
    $email_template_key = kcGetEmailTemplateKey();
    $email_content = $content;

    if (count($email_template_key) > 0) {
        foreach ($email_template_key as $item => $value) {
            switch ($value) {
                case '{{user_name}}':
                    if(isset($data['username'])) {
                        $email_content = str_replace($value, esc_html($data['username']), $email_content);
                    }
                    break;
                case '{{user_password}}':
                    if(isset($data['password'])) {
                        $email_content = str_replace($value, esc_html($data['password']), $email_content);
                    }
                    break;
                case '{{user_email}}':
                    if(isset($data['user_email'])) {
                        $email_content = str_replace($value, esc_html($data['user_email']), $email_content);
                    }
                    break;
                case '{{appointment_date}}':
                    if(isset($data['appointment_date'])) {
                        $email_content = str_replace($value, esc_html($data['appointment_date']), $email_content);
                    }
                    break;
                case '{{appointment_time}}':
                    if(isset($data['appointment_time'])) {
                        if(kcGetAppointmentTimeFormatOption() == 'on'){
                            $data['appointment_time']= date("H:i", strtotime($data['appointment_time']));
                        }else{
                            $data['appointment_time'] = date("g:i a", strtotime($data['appointment_time']));
                        }
                        $email_content = str_replace($value, esc_html($data['appointment_time']), $email_content);
                    }
                    break;
                case '{{patient_name}}':
                    if(isset($data['patient_name'])) {
                        $email_content = str_replace($value, esc_html($data['patient_name']), $email_content);
                    }                  
                    break;
                case '{{doctor_name}}':
                    if(isset($data['doctor_name'])) {
                        $email_content = str_replace($value, esc_html($data['doctor_name']), $email_content);
                    }                  
                    break;
                case '{{zoom_link}}':
                    if(isset($data['zoom_link'])) {
                        $email_content = str_replace($value, esc_url($data['zoom_link']), $email_content);
                    }
                    break;
                case '{{add_doctor_zoom_link}}':
                    if(isset($data['add_doctor_zoom_link'])) {
                        $email_content = str_replace($value, esc_url($data['add_doctor_zoom_link']), $email_content);
                    }
                    break;
                case '{{service_name}}':
                    if(isset($data['service_name'])) {
                        $email_content = str_replace($value, esc_html($data['service_name']), $email_content);
                    }
                    break;
                case '{{current_date}}':
                    $current_date = current_time(get_option('date_format'));
                    $email_content = str_replace($value, esc_html($current_date), $email_content);
                    break;
                case '{{current_date_time}}':
                    $current_date_time = current_time(get_option('date_format') ." ".get_option('time_format'));
                    $email_content = str_replace($value, esc_html($current_date_time), $email_content);
                    break;
                case '{{total_amount}}':
                    if(isset($data['total_amount'])){
                        $email_content = str_replace($value, esc_html($data['total_amount']), $email_content);
                    }
                    break;
                case '{{clinic_name}}':
                    if(isset($data['clinic_name'])){
                        $email_content = str_replace($value, esc_html($data['clinic_name']), $email_content);
                    }
                    break;
                case '{{prescription}}':
                    if(isset($data['prescription'])){
                        $email_content = str_replace($value, wp_kses_post($data['prescription']), $email_content);
                    }
                    break;
                case '{{meet_link}}':
                    if(isset($data['meet_link'])){
                        $email_content = str_replace($value, esc_url($data['meet_link']), $email_content);
                    }
                    break;
                case '{{meet_event_link}}':
                    if(isset($data['meet_event_link'])){
                        $email_content = str_replace($value, esc_url($data['meet_event_link']), $email_content);
                    }
                    break;
                case '{{widgets_login_url}}':
                    $email_content = str_replace($value, esc_url(kcGetDashboardPageUrl()), $email_content);
                    break;
                case '{{login_url}}':
                    $email_content = str_replace($value, esc_url(wp_login_url()), $email_content);
                    break;
                case '{{appointment_page_url}}':
                    $email_content = str_replace($value,  esc_url(kcGetAppointmentPageUrl()), $email_content);
                    break;
                case '{{doctor_email}}':
                    if(isset($data['doctor_email'])){
                        $email_content = str_replace($value, esc_html($data['doctor_email']), $email_content);
                    }
                    break;
                case '{{doctor_contact_number}}':
                    if(isset($data['doctor_contact_number'])){
                        $email_content = str_replace($value, esc_html($data['doctor_contact_number']), $email_content);
                    }
                    break;
                case '{{clinic_email}}':
                    if(isset($data['clinic_email'])){
                        $email_content = str_replace($value, esc_html($data['clinic_email']), $email_content);
                    }
                    break;
                case '{{clinic_contact_number}}':
                    if(isset($data['clinic_contact_number'])){
                        $email_content = str_replace($value, esc_html($data['clinic_contact_number']), $email_content);
                    }
                    break;
                case '{{clinic_address}}':
                    if(isset($data['clinic_address'])){
                        $email_content = str_replace($value, esc_html($data['clinic_address']), $email_content);
                    }
                    break;
                case '{{patient_email}}':
                    if(isset($data['patient_email'])){
                        $email_content = str_replace($value, esc_html($data['patient_email']), $email_content);
                    }
                    break;
                case '{{user_contact}}':
                    if(isset($data['user_contact'])){
                        $email_content = str_replace($value, esc_html($data['user_contact']), $email_content);
                    }
                    break;
                case '{{site_url}}':
                    $email_content = str_replace($value, esc_url(get_site_url()), $email_content);
                    break;
                case '{{user_role}}':
                    if(isset($data['user_role'])){
                        $email_content = str_replace($value, esc_html($data['user_role']), $email_content);
                    }
                    break;
                case '{{appointment_desc}}':
                    if(isset($data['description'])){
                        $email_content = str_replace($value, esc_html($data['description']), $email_content);
                    }
                    break;
            }
        }
    }
    return $email_content;

}

function kcGetEmailTemplateKey()
{
    return [
        '{{user_name}}',
        '{{user_password}}',
        '{{user_email}}',
        '{{user_contact}}',
        '{{user_role}}',
        '{{appointment_date}}',
        '{{appointment_time}}',
        '{{patient_name}}',
        '{{doctor_name}}',
        '{{zoom_link}}',
        '{{add_doctor_zoom_link}}',
        '{{service_name}}',
        '{{current_date}}',
        '{{total_amount}}',
        '{{prescription}}',
        '{{clinic_name}}',
        '{{meet_link}}',
        '{{meet_event_link}}',
        '{{login_url}}',
        '{{widgets_login_url}}',
        '{{current_date_time}}',
        '{{appointment_page_url}}',
        '{{doctor_email}}',
        '{{clinic_email}}',
        '{{doctor_contact_number}}',
        '{{clinic_contact_number}}',
        '{{clinic_address}}',
        '{{patient_email}}',
        '{{site_url}}',
        '{{appointment_desc}}'
    ];
}

function kcCheckUserEmailAlreadyUsed($request_data)
{
    $status = true;
    $message = '';
    $email_exists = email_exists($request_data['user_email']);
    //check if email already used
    if (!empty($email_exists)) {
        //if editing user and email is used by editing user only
        if (!empty($request_data['ID']) && (int)$request_data['ID'] == $email_exists) {
            $email_exists = false;
        }
        if ($email_exists) {
            $status = false;
            $message = esc_html__('There already exists an User registered with this email address,please use other email ID', 'kc-lang');

        }
    }
    return [
        'status' => $status,
        'message' => $message
    ];
}

function kcSendAppointmentSmsOnPro($appointment_id){
    $sms1 =$sms2=$sms3= '';
    $appointment_id = (int)$appointment_id;
    if(kcCheckSmsOptionEnable()){
        $sms1 = apply_filters('kcpro_send_sms', [
            'type' => 'clinic_book_appointment',
            'appointment_id' => $appointment_id,
        ]);

        $sms2 = apply_filters('kcpro_send_sms', [
            'type' => 'doctor_book_appointment',
            'appointment_id' => $appointment_id,
        ]);

        $sms3 = apply_filters('kcpro_send_sms', [
            'type' => 'add_appointment',
            'appointment_id' => $appointment_id,
        ]);
    }

    return [ 'clinic_book_appointment' =>$sms1,
        'doctor_book_appointment' => $sms2,
        'add_appointment' => $sms3
    ];
}

function kcSendAppointmentZoomSms($appointment_id){
    $sms =$sms1= '';
    $appointment_id= (int)$appointment_id;
    if(kcCheckSmsOptionEnable()){
        $sms = apply_filters('kcpro_send_sms', [
            'type' => 'zoom_link',
            'appointment_id' => $appointment_id,
        ]);

        $sms1 = apply_filters('kcpro_send_sms', [
            'type' => 'add_doctor_zoom_link',
            'appointment_id' => $appointment_id,
        ]);
    }
    return [
        'zoom_link'  => $sms,
        'add_doctor_zoom_link' =>$sms1
    ];
}

function kcSendAppointmentMeetSms($appointment_id){
    $sms =$sms1= '';
    $appointment_id= (int)$appointment_id;
    if(kcCheckSmsOptionEnable()){
        $sms = apply_filters('kcpro_send_sms', [
            'type' => 'meet_link',
            'appointment_id' => $appointment_id,
        ]);

        $sms1 = apply_filters('kcpro_send_sms', [
            'type' => 'add_doctor_meet_link',
            'appointment_id' => $appointment_id,
        ]);
    }
    return [
        'zoom_link'  => $sms,
        'add_doctor_zoom_link' =>$sms1
    ];
}

function kcTelemedSms($filterData){

    $status = false;
    if(isKiviCareTelemedActive()){
        if(!empty($filterData) && !empty( $filterData['appointment_id']) ){
            $status =  kcAppointmentZoomDoctorEmail( $filterData['appointment_id']);
            $status2 =  kcAppointmentZoomPatientEmail( $filterData['appointment_id']);
            $smsResponse = kcSendAppointmentZoomSms($filterData['appointment_id']);
            if($status && $status2){
                $status = true;
            }else{
                $status = false;
            }
        }
    }
    return [
        'status' => $status,
        'message' => esc_html__('Meetings has email send', 'kct-lang')
    ];
}

function kcGetDefaultClinicId()
{
    $option = get_option('setup_step_1');
    if ($option) {
        $option = json_decode($option);
        return (int)$option->id[0];
    } else {
        return 0;
    }
}

function kcGetDefaultClinic() {
    global $wpdb;
    $clinic_table_name = $wpdb->prefix . 'kc_clinics';
    $clinic_id = kcGetDefaultClinicId();
    $clinic_query = "SELECT * FROM {$clinic_table_name}  WHERE `id` = {$clinic_id} ";
    return $wpdb->get_results($clinic_query, ARRAY_A);
}

/**
 * @since 2.3.0
 * @return array
 */

function kcClinicList () {
    global $wpdb;
    $clinic_table = $wpdb->prefix . 'kc_clinics';
    $clinic_query = "SELECT * FROM {$clinic_table}" ;
    $clinic_list = $wpdb->get_results($clinic_query) ;
    if (!empty($clinic_list)) {
        $clinic_list = collect($clinic_list);
        return $clinic_list;
    } else {
        return [];
    }
}

/**
 * @param int $id
 * @since 2.3.0
 * @return object,bool
 *
 */

function kcClinicDetail ($id) {
    global $wpdb;
    if(!empty($id)) {
        $id = (int)$id;
        $clinic_table = $wpdb->prefix . 'kc_clinics';
        $clinic_query = "SELECT * FROM {$clinic_table} WHERE id = ".$id ;
        $clinic_detail = $wpdb->get_row($clinic_query);
        if (!empty($clinic_detail)) {
            return $clinic_detail ;
        } else {
            return false;
        }
    } else {
        return false;
    }
}

function kcGetClinicSessions($clinic_id)
{

    $clinic_id = (int)$clinic_id;
    $clinic_sessions = collect((new KCClinicSession())->get_by([ 'clinic_id' => $clinic_id]));
    $doctors = collect((new KCDoctorClinicMapping())->get_by([ 'clinic_id' => $clinic_id]))->map(function ($mapping) {
        $doctor = WP_User::get_data_by('ID', $mapping->doctor_id);
        $mapping->doctor_id = [
            'id' => (int)$doctor->ID,
            'label' => $doctor->display_name,
        ];

        $user_data = get_user_meta($doctor->ID, 'basic_data', true);
        $user_data = json_decode($user_data);

        $mapping->doctor_id['timeSlot'] = isset($user_data->time_slot) ? $user_data->time_slot : "";

        return $mapping;
    })->pluck('doctor_id')->toArray();

    $sessions = collect([]);
    if (count($clinic_sessions)) {
        foreach ($clinic_sessions as $session) {
            if ($session->parent_id === null || $session->parent_id === "") {
                $days = [];
                $session_doctors = [];
                $sec_start_time = "";
                $sec_end_time = ""  ;

                $all_clinic_sessions  = collect($clinic_sessions);

                $child_session = $all_clinic_sessions->where('parent_id', $session->id);

                $child_session->all();

                if(count($child_session) > 0) {
                    foreach ( $clinic_sessions as $child_session ) {
                        if ( $child_session->parent_id !== null && $session->id === $child_session->parent_id ) {
                            array_push( $days, substr($child_session->day, 0, 3) );
                            array_push( $session_doctors, $child_session->doctor_id );

                            if ( $session->start_time !== $child_session->start_time || $session->start_time == $session->end_time ) {
                                $sec_start_time = $child_session->start_time;
                                $sec_end_time   = $child_session->end_time;
                            }
                        }
                    }
                } else {

                    array_push($session_doctors, $session->doctor_id);
                    array_push($days, substr($session->day, 0, 3));

                }


                if($session->start_time == $session->end_time){
                    $start_time = ['',''];
                }else{

                    $start_time = explode(":",date('H:i',strtotime($session->start_time)));
                }

                if($session->start_time == $session->end_time){

                    $end_time = ['',''];

                }else{

                    $end_time = explode(":",date('H:i',strtotime($session->end_time)));
                }


                $session_doctors = array_unique($session_doctors);

                if (count($session_doctors) === 0 && count($days) === 0) {
                    $session_doctors[] = $session->doctor_id;
                    $days[] = substr($session->day, 0, 3);
                } else {
                    $sec_start_time = $sec_start_time !== "" ? explode(":", date('H:i', strtotime($sec_start_time))) : "";
                    $sec_end_time = $sec_end_time !== "" ? explode(":", date('H:i', strtotime($sec_end_time))) : "";
                }

                $new_doctors = [];

                foreach ($session_doctors as $doctor_id) {
                    foreach ($doctors as $doctor) {
                        if ((int)$doctor['id'] === (int)$doctor_id) {
                            $new_doctors = $doctor;
                        }
                    }
                }

                $new_session = [
                    'id' => $session->id,
                    'clinic_id' => $session->clinic_id,
                    'doctor_id' => $session->doctor_id,
                    'days' => array_values(array_unique($days)),
                    'doctors' => $new_doctors,
                    'time_slot' => $session->time_slot,
                    's_one_start_time' => [
                        "HH" => $start_time[0],
                        "mm" => $start_time[1],
                    ],
                    's_one_end_time' => [
                        "HH" => $end_time[0],
                        "mm" => $end_time[1],
                    ],
                    's_two_start_time' => [
                        "HH" => isset($sec_start_time[0]) ? $sec_start_time[0] : "",
                        "mm" => isset($sec_start_time[1]) ? $sec_start_time[1] : "",
                    ],
                    's_two_end_time' => [
                        "HH" => isset($sec_end_time[0]) ? $sec_end_time[0] : "",
                        "mm" => isset($sec_end_time[1]) ? $sec_end_time[1] : "",
                    ]
                ];

                $sessions->push($new_session);

            }
        }
    }

    return $sessions;
}

function kcGetAllClinicHaveSession(){

    global $wpdb;
    $status = false;
    $message = esc_html__('All Clinic Have Doctor Session','kc-lang');
    $doctor_session_table = $wpdb->prefix.'kc_clinic_sessions';
    $clinic_table = $wpdb->prefix.'kc_clinics';
    $msg2 = __('Clinic ','kc-lang');
    if(isKiviCareProActive()){
        $clinic_id = collect($wpdb->get_results('select * from '.$clinic_table))->pluck('id')->toArray();
        $clinic_session_id = collect($wpdb->get_results('select * from '.$doctor_session_table))->unique('clinic_id')->pluck('clinic_id')->toArray();
        $result = array_diff($clinic_id,$clinic_session_id);
        if(!empty($result) && count($result) > 0){
            $clinic_name =  collect($wpdb->get_results('select name from '.$clinic_table.' where id in ('.implode(',',$result).')'))->pluck('name')->toArray();
            if(!empty($clinic_name) && count($clinic_name) > 0) {
                $status = true;
                $clinic_name1 = implode(',', $clinic_name);

                /* translators: clinic name*/
                $message = $msg2.$clinic_name1.__( ' do not have a doctor session', 'kc-lang');
            }
        }
    }else{
        $clinic_session_id = $wpdb->get_var("select count(*) from {$doctor_session_table} where clinic_id=".kcGetDefaultClinicId());
        if(!empty($clinic_session_id) && $clinic_session_id < 1){
            $status = true;
            $data = $wpdb->get_var("select name from {$clinic_table} where id=".kcGetDefaultClinicId());

            /* translators: doctor name*/
            $message = $msg2.$data.__( ' do not have a doctor session', 'kc-lang');
        }
    }
    return [
        'status'  => $status,
        'message' => $message,
    ];
}

function kcGetClinicCurrenyPrefixAndPostfix(){
    global $wpdb;
    $clinic_prefix = '';
    $clinic_postfix = '';
    $prefix_postfix = $wpdb->get_var("SELECT extra FROM {$wpdb->prefix}kc_clinics");
    if (!empty($prefix_postfix)) {
        $prefix_postfix = json_decode($prefix_postfix);
        $clinic_prefix = (!empty($prefix_postfix->currency_prefix) && $prefix_postfix->currency_prefix !== 'null') ? $prefix_postfix->currency_prefix : '';
        $clinic_postfix = (!empty($prefix_postfix->currency_postfix) && $prefix_postfix->currency_postfix !== 'null') ? $prefix_postfix->currency_postfix : '';
    }
    return [
        'prefix' => $clinic_prefix,
        'postfix' => $clinic_postfix
    ];
}

function kcGetClinicIdOfClinicAdmin(){
    $current_user_id= get_current_user_id();
    global $wpdb;
    $clinic_id = $wpdb->get_var("SELECT id FROM {$wpdb->prefix}kc_clinics WHERE clinic_admin_id={$current_user_id}");
    if(isKiviCareProActive()){
        return !empty($clinic_id) ? (int)$clinic_id : 0;
    }else{
        return kcGetDefaultClinicId();
    }
}

function kcGetClinicIdOfReceptionist(){
    $current_user_id= get_current_user_id();
    global $wpdb;
    $clinic_id = $wpdb->get_var("SELECT clinic_id FROM {$wpdb->prefix}kc_receptionist_clinic_mappings WHERE receptionist_id={$current_user_id}");
    if(isKiviCareProActive()){
        return !empty($clinic_id) ? (int)$clinic_id : 0;
    }else{
        return kcGetDefaultClinicId();
    }

}

function kcDoctorPatientList(){
    global  $wpdb;
    $appointments = collect((new KCAppointment)->get_by(['doctor_id' => get_current_user_id()]))->pluck('patient_id')->unique()->toArray();
    $get_doctor_patient = collect($wpdb->get_results("SELECT *  FROM {$wpdb->base_prefix}usermeta WHERE `meta_value` = " . get_current_user_id() . " AND
			                `meta_key` LIKE 'patient_added_by'"))->pluck('user_id')->unique()->toArray();
    $all_user = array_merge($get_doctor_patient, $appointments);
    $encounters = collect((new KCPatientEncounter())->get_by(['doctor_id' => get_current_user_id()]))->pluck('patient_id')->unique()->toArray();
    $all_user = array_unique(array_merge($all_user, $encounters));
    return $all_user;
}

function kcGetUserValueByKey($userType,$user_id,$key){
    $data = '';
    switch ($userType){
        case 'doctor':
        case 'patient':
        case "clinic_admin":
        case 'receptionist':
        case 'common':
            $doctor_detail = !empty($user_id) ? json_decode(get_user_meta($user_id,'basic_data',true)) : '';
            $data =  !empty($doctor_detail->{$key}) ? $doctor_detail->{$key} : '';
            break;

    }

    return $data;
}

function generatePatientUniqueIdRegister(){
    global $wpdb;
    $patient_unique_id = kcPatientUniqueIdEnable('value');
    $patient_unique_id_exist = $wpdb->get_results("SELECT  meta_value FROM  " . $wpdb->base_prefix . "usermeta WHERE  meta_key = 'patient_unique_id'  AND  meta_value ='" . $patient_unique_id . "' LIMIT 1");
    if ($patient_unique_id_exist === NULL) {
        generatePatientUniqueIdRegister();
    } else {
        return $patient_unique_id;
    }
}

function kcGetServiceCharges ($service) {
    global $wpdb;
    $service_doctor_mapping_table = $wpdb->prefix . 'kc_service_doctor_mapping';
    $service_id =  (int)$service['service_id'];
    $doctor_id =  (int)$service['doctor_id'];
    $service_query = "SELECT * FROM  {$service_doctor_mapping_table}  WHERE service_id = {$service_id}  AND doctor_id = {$doctor_id} " ;
    $service_charges = $wpdb->get_results($service_query, 'OBJECT');
    if(count($service_charges)) {
        return $service_charges[0];
    }
    return [];
}

function kcGetServiceById ($service_id) {
    global $wpdb;
    $service_id = (int)$service_id;
    $service = $wpdb->prefix . 'kc_services';
    $service_query = "SELECT * FROM  {$service} WHERE id = {$service_id} " ;
    $service = $wpdb->get_results($service_query, 'OBJECT');
    if(count($service)) {
        return $service[0];
    }
    return [];
}

function getServiceId($data) {

    global $wpdb;
    $service = $wpdb->prefix . 'kc_services';
    if($data['type'] == 'Telemed' || $data['type'] == 'telemed') {
        $condition  = " AND type = 'system_service' AND name = '{$data['type']}' " ;
    } else {
        $condition  = " AND name = '{$data['type']}' " ;
    }

    $service_query = "SELECT * FROM {$service} WHERE 0 = 0 " . $condition ;
    $service_id = $wpdb->get_results($service_query, 'OBJECT');
    if($service_id) {
        return $service_id ;
    } else {
        $data->id = 0 ;
    }

}

function getDoctorTelemedServiceCharges($doctor_id) {
    global $wpdb;
    $doctor_id = (int)$doctor_id;
    $table_name = $wpdb->prefix . 'kc_service_doctor_mapping';
    $telemed_service_id = $wpdb->get_var("SELECT id FROM {$wpdb->prefix}kc_services WHERE name='Telemed' OR name='telemed'");
    if($telemed_service_id != ''){
        $query = "SELECT charges FROM  {$table_name} charges WHERE doctor_id = {$doctor_id} AND service_id={$telemed_service_id} ";
        $result = $wpdb->get_var($query);
        if($result != '') {
            return $result;
        } else {
            return '0' ;
        }
    }else{
        return '0' ;
    }
}

function kcCheckDoctorTelemedType($appointment_id) {
    global $wpdb;
    $appointment_id = (int)$appointment_id;
    $doctor_telemed_type = '';
    if(isKiviCareTelemedActive() && isKiviCareGoogleMeetActive()){
        $doctor_id = $wpdb->get_var("SELECT doctor_id FROM {$wpdb->prefix}kc_appointments WHERE id={$appointment_id}");
        $doctor_telemed_type = get_user_meta($doctor_id,'telemed_type',true);
    }elseif (isKiviCareGoogleMeetActive()) {
        $doctor_telemed_type = 'googlemeet';
    }elseif(isKiviCareTelemedActive()){
        $doctor_telemed_type = 'zoom';
    }
    return $doctor_telemed_type == 'googlemeet' ? 'googlemeet' : 'zoom';
}

function kcDoctorTelemedServiceEnable($doctor_id){
    $doctor_telemed_enable = false;
    if(isKiviCareTelemedActive()){
        $zoom_config_data = get_user_meta($doctor_id, 'zoom_config_data', true);
        if(!empty($zoom_config_data)){
            $zoom_config_data = json_decode($zoom_config_data);
            $doctor_telemed_enable = !empty($zoom_config_data->enableTeleMed) && strval($zoom_config_data->enableTeleMed) == 'true';
        }
    }
    //check doctor googlemeet enable
    if(!$doctor_telemed_enable && isKiviCareGoogleMeetActive()){
        $googleMeet = get_option(KIVI_CARE_PREFIX . 'google_meet_setting',true);
        if(gettype($googleMeet !== 'boolean') && !empty($googleMeet['enableCal']) && in_array((string)$googleMeet['enableCal'] ,['1','true','Yes']) ){
            $doctor_telemed_enable = get_user_meta($doctor_id, KIVI_CARE_PREFIX.'google_meet_connect',true) == 'on';
        }
    }

    return $doctor_telemed_enable;
}

function kcServiceListFromRequestData ($request_data) {
    $serviceList =[];
    $serviceName = '';
    if(is_array($request_data['visit_type'])){
        foreach ($request_data['visit_type'] as $key => $value){
            $serviceList[] = sanitize_text_field($value['name']);
        }
    }
    if(is_array($serviceList) && count($serviceList) > 0 ){
        $serviceName = implode(",",$serviceList);
    }
    return $serviceName;
}

function kcGetTelemedServiceId(){

    // Add telemed service if not in service list
    $telemed_service_id = getServiceId(['type' => 'Telemed']);

    if (isset($telemed_service_id[0])) {

        $telemed_Service = $telemed_service_id[0]->id;

    } else {

        $service_data = new KCService;
        $services = [
            'type' => 'system_service',
            'name' => 'telemed',
            'price' => 0,
            'status' => 1,
            'created_at' => current_time('Y-m-d H:i:s')
        ];
        $telemed_Service = $service_data->insert($services);
    }

    return (int)$telemed_Service;
}

function kivicareGetProductIdOfService($id){
    global $wpdb;
    $product_id = '';
    $appointments_service_table =  $wpdb->prefix . 'kc_service_doctor_mapping';
    $data = $wpdb->get_var('select extra from '.$appointments_service_table.' where id='.$id);
    if($data != null){
        $data = json_decode($data);
        $product_id = $data->product_id;
    }
    return $product_id;
}

/**
 * // Data param required date, clinic_id, doctor_id
 *
 * @param $data
 *
 * @param string $new_time_slot
 * @param $only_available_slots
 * @return array
 */

function kvGetTimeSlots($data, $new_time_slot = "", $only_available_slots = false)
{
    global $wpdb;
    $slots = [];

    $clinic_session_table = $wpdb->prefix . 'kc_clinic_sessions';

    if (!isset($data['date']) || !isset($data['doctor_id']) || !isset($data['clinic_id'])) {
        return $slots;
    }

    $appointment_day = strtolower(date('l', strtotime($data['date'])));

    $day_short = substr($appointment_day, 0, 3) ;

    $query = "SELECT * FROM {$clinic_session_table}  WHERE `doctor_id` = ".(int)$data['doctor_id']." AND `clinic_id` = ".(int)$data['clinic_id']."  AND ( `day` = '{$day_short}' OR `day` = '{$appointment_day}') ";
    $clinic_session = collect($wpdb->get_results($query, OBJECT))->sortBy('start_time');

    if (count($clinic_session)) {

        $appointmentModel = new KCAppointment();
        $slot_date = $data['date'];

        $appointment_table = $wpdb->prefix . 'kc_appointments' ;

        $appointment_query = "SELECT * FROM " . $appointment_table . " WHERE appointment_start_date = '" . date("Y-m-d", strtotime($data["date"])) . "' AND status != 0 "  ;

        $appointments = $wpdb->get_results($appointment_query) ;

        $table_name = $wpdb->prefix . 'kc_clinic_schedule';
        $query = "SELECT * FROM $table_name WHERE `start_date` <= '$slot_date' AND `end_date` >= '$slot_date'  AND `status` = 1";
        $results = collect($wpdb->get_results($query, OBJECT))->sortBy('start_time');

        $leaves = $results->filter(function ($result) use ($data) {

            if ($result->module_type === "clinic") {
                if ((int) $result->module_id === (int) $data['clinic_id']) {
                    return true;
                }
            } elseif ($result->module_type === "doctor") {
                if ((int)$result->module_id === (int) $data['doctor_id']) {
                    return true;
                }
            } else {
                return false;
            }

            return false;
        });

        if (count($leaves)) {
            return $slots;
        }

        foreach ($clinic_session as $key => $session) {

            $newTimeSlot = "";
            $time_slot = $session->time_slot ;
            $start_time = new DateTime($session->start_time);
            $time_diff = $start_time->diff(new DateTime($session->end_time));

            if ($time_diff->h !== 0) {
                $time_diff_min = round(($time_diff->h * 60) / $time_slot);
            } else {
                $time_diff_min = round($time_diff->i / $time_slot);
            }

            for ($i = 0; $i <= $time_diff_min; $i++) {

                if ($i === 0) {
                    $newTimeSlot = date('H:i', strtotime($session->start_time));
                } else {
                    $newTimeSlot = date('H:i', strtotime('+' . $time_slot . ' minutes', strtotime($newTimeSlot)));
                }

                if (strtotime($newTimeSlot) < strtotime($session->end_time)) {

                    $temp = [
                        'time' => date('h:i A', strtotime($newTimeSlot)),
                        'available' => true
                    ];

                    $isAvailable = array_filter($appointments, function ($appointment) use ($newTimeSlot, $data) {
                        if ($appointment->appointment_start_time === date('H:i:s', strtotime($newTimeSlot))
                            && (int) $appointment->id !==  (int) $data['appointment_id']
                            && (int) $appointment->clinic_id === (int) $data['clinic_id']
                            && (int) $appointment->doctor_id === (int) $data['doctor_id']) {
                            return true;
                        } else {
                            return false;
                        }
                    });

                    if (count($isAvailable)) {
                        (bool) $temp['available'] = false;
                    }

                    $currentDateTime = current_time('Y-m-d H:i:s');
                    $newDateTime = date('Y-m-d', strtotime($data['date'])) . ' ' . $newTimeSlot . ':00';

                    if (strtotime($newDateTime) < strtotime($currentDateTime)) {
                        (bool) $temp['available'] = false;
                    }

                    // following condition is for get only available slots
                    if($only_available_slots !== false) {
                        if($temp['available'] !== false) {
                            $slots[$key][] = $temp;
                        }
                    } else {
                        $slots[$key][] = $temp;
                    }
                }
            }
        }
    }
    return array_values($slots);
}

function kcCancelAppointments ($data) {

	$start_date = $data['start_date'];
	$end_date = $data['end_date'];
	global $wpdb;

    $app_table_name = $wpdb->prefix . 'kc_appointments';

	$appointment_condition  = " `appointment_start_date` >= '$start_date' AND `appointment_start_date` <= '$end_date' " ;

    $query = "UPDATE {$app_table_name} SET `status` = 0  WHERE  {$appointment_condition} AND `status` = 1 " ;

    $select_recepients_query = "SELECT CONCAT(\"'\", GROUP_CONCAT(DISTINCT patient_id SEPARATOR \",'\" ), \"'\") AS patient_id FROM {$app_table_name} WHERE {$appointment_condition}" ;

    $data_condition = '';

	if (isset($data['doctor_id'])) {
        $data['doctor_id'] = (int)$data['doctor_id'];
        $data_condition = " AND doctor_id={$data['doctor_id']}";
        $query = $query . " AND doctor_id = " . $data['doctor_id'];
        $select_recepients_query =  $select_recepients_query . " AND doctor_id = " . $data['doctor_id'];
	}

	if (isset($data['clinic_id'])) {
        $data['clinic_id'] = (int)$data['clinic_id'];
        $data_condition = " AND clinic_id={$data['clinic_id']}";
        $query = $query . " AND clinic_id = " . $data['clinic_id'];
        $select_recepients_query =  $select_recepients_query . " AND clinic_id = " . $data['clinic_id'];
	}


    //send email to all cancel appointment
    $data_query = "select * from {$app_table_name} where {$appointment_condition} AND status = 1 {$data_condition}";
    $appointment_data = $wpdb->get_results($data_query);
    if($appointment_data != null){
        foreach ($appointment_data as $res){
            $email_data = kcCommonNotificationData($res,[],'','cancel_appointment');
           //send cancel email
            kcSendEmail($email_data);
            if(kcCheckSmsOptionEnable() || kcCheckWhatsappOptionEnable()){
                apply_filters('kcpro_send_sms', [
                    'type' => 'cancel_appointment',
                    'appointment_id' => $res->id,
                ]);
            }
            //cancel zoom meeting
            if(isKiviCareTelemedActive()){
                apply_filters('kct_delete_appointment_meeting', ['id'=>$res->id]);
            }
            //remove google calendar event
            if(kcCheckGoogleCalendarEnable()){
                apply_filters('kcpro_remove_appointment_event', ['appoinment_id'=>$res->id]);
            }

            //remove google meet event
            if(isKiviCareGoogleMeetActive()){
               apply_filters('kcgm_remove_appointment_event',['appoinment_id' => $res->id]);
            }
        }
    }

    $receptionist = $wpdb->query($select_recepients_query);

    $wpdb->query($query);

}


function kcAppointmentRestrictionData(){
    $data = get_option(KIVI_CARE_PREFIX . 'restrict_appointment',true);
    if(gettype($data) != 'boolean'){
        $temp = [
            'pre_book' => isset($data['pre']) && !empty($data['pre']) ? $data['pre'] : 0,
            'post_book' => isset($data['post']) && !empty($data['post']) ? $data['post'] : 365,
        ];
    }else{
        $temp = [
            'pre_book' =>  0,
            'post_book' => 365,
        ];
    }

    return $temp;
}

function kcAppointmentPatientEmail($appointmentid,$service_name){
    return kcCommonEmailFunction($appointmentid,$service_name,'patient');
}

function kcAppointmentDoctorEmail($appointmentid,$service_name){
    $status = kcCommonEmailFunction($appointmentid,$service_name,'clinic');
    $status1 = kcCommonEmailFunction($appointmentid,$service_name,'doctor');
    $data = false;
    if($status && $status1){
        $data = true;
    }
    return $data ;
}

function kcAppointmentZoomDoctorEmail($appointmentid){
    return kcCommonEmailFunction($appointmentid,'Telemed','zoom_doctor');
}

function kcAppointmentZoomPatientEmail($appointmentid){
    return kcCommonEmailFunction($appointmentid,'Telemed','zoom_patient');
}

function kcCommonEmailFunction($appointmentid,$service_name,$type){
    global $wpdb;
    $service_name = !empty($service_name) ? $service_name : '';
    $appointmentid = (int)$appointmentid;
    if(!empty($appointmentid) ){
        $appointment_data = $wpdb->get_row("SELECT * FROM {$wpdb->prefix}kc_appointments WHERE id={$appointmentid}");
        $zoom_data = [];
        if (isKiviCareTelemedActive()) {
            $zoom_data = $wpdb->get_row("SELECT * FROM {$wpdb->prefix}kc_appointment_zoom_mappings WHERE appointment_id={$appointmentid}");
        }
        if(isKiviCareGoogleMeetActive() && kcCheckDoctorTelemedType($appointmentid) == 'googlemeet'){
            $googlemeet_data =  $wpdb->get_row("SELECT * FROM {$wpdb->prefix}kc_appointment_google_meet_mappings WHERE appointment_id=".$appointmentid);
            if($googlemeet_data != ''){
                $zoom_data = new stdClass();
                $zoom_data->join_url = $googlemeet_data->url;
                $zoom_data->start_url = $googlemeet_data->url;
                $zoom_data->event_url = $googlemeet_data->event_url;
                $zoom_data->url = $googlemeet_data->url;
            }
        }
        if(!empty($appointment_data)){

            $commonData = kcCommonNotificationData($appointment_data,$zoom_data,$service_name,$type);

            return kcSendEmail($commonData);

        }
        return  false;
    }
    return  false;
}

function kcAppointmentCancelMail($appointmentData){

    $Status=$appointmentData->status;
    $returnValue = false;
    $date = date($appointmentData->appointment_start_date);
    $time = $appointmentData->appointment_start_time;
    $appointment_time=("$date"." "."$time");
    $current_date = current_time("Y-m-d H:i:s");
    if($Status != 0){
        if($current_date < $appointment_time){
            $returnValue = kcCommonEmailFunction($appointmentData->id,'kivicare','cancel_appointment');
            if(kcCheckSmsOptionEnable() || kcCheckWhatsappOptionEnable()){
                $sms_status = apply_filters('kcpro_send_sms', [
                    'type' => 'cancel_appointment',
                    'appointment_id' => $appointmentData->id,
                ]);
            }
        }
    }

    return $returnValue;
}

function kivicareCommonSendEmailIfOnlyLitePluginActive($request_data,$appointment_id){
    $patient_email_status = $doctor_email_status = false;
    if(!isKiviCareProActive() &&  !isKiviCareTelemedActive() && !isKiviCareGoogleMeetActive()){
        foreach ($request_data['visit_type'] as $key => $value) {
            $service = sanitize_text_field(strtolower($value['name']));
            if ($service != 'telemed') {
                $patient_email_status = kcAppointmentPatientEmail($appointment_id,kcServiceListFromRequestData($request_data));
            }
        }
        $doctor_email_status = kcAppointmentDoctorEmail($appointment_id,kcServiceListFromRequestData($request_data));
    }

    return ['patient_email_status' => $patient_email_status,
        'doctor_email_status' => $doctor_email_status
    ];
}

function kcProAllNotification($appointment_id,$request_data,$telemed_service_include){
    $google_event_status = $telemed = false;
    $doctor_email_status = kcAppointmentDoctorEmail($appointment_id,kcServiceListFromRequestData($request_data));
    $patient_email_status = kcAppointmentPatientEmail($appointment_id,kcServiceListFromRequestData($request_data));
    $smsResponse = kcSendAppointmentSmsOnPro($appointment_id);
    if(kcCheckGoogleCalendarEnable()){
        $google_event_status = apply_filters('kcpro_save_appointment_event', [
            'appoinment_id'=>$appointment_id,
        ]);
    }
    if($telemed_service_include) {
        if(kcCheckDoctorTelemedType($appointment_id) == 'googlemeet'){
            $telemed = apply_filters('kcgm_save_appointment_event_link_send',['appoinment_id' => $appointment_id]);
        }else{
            $telemed = apply_filters('kct_send_zoom_link', ['appointment_id' => $appointment_id ] );;
        }
    }
    return ['doctor_email_status' => $doctor_email_status,
        'patient_email_status' => $patient_email_status,
        'smsResponse' => $smsResponse,
        'google_event_status' => $google_event_status,
        'telemed' => $telemed
    ];
}

function kcCommonNotificationData($appointment_data,$zoom_data,$service_name,$type){
     global $wpdb;
    $clinic_table = $wpdb->prefix . 'kc_clinics';
    $clinic_data = $wpdb->get_row('select * from '.$clinic_table.' where id='.$appointment_data->clinic_id);
    $patient_id = $appointment_data->patient_id;
    $patient_details = get_user_by( 'ID', $patient_id);
    $doctor_id = $appointment_data->doctor_id;
    $doctor_details = get_user_by( 'ID', $doctor_id);

    $commonData = [
        'appointment_date' => $appointment_data->appointment_start_date,
        'appointment_time' => $appointment_data->appointment_start_time,
        'service_name'     => $service_name,
        'current_date'     => current_time('Y-m-d'),
        'patient_name'     => $patient_details->display_name,
        'patient_email'    =>$patient_details->user_email,
        'doctor_name'      => $doctor_details->display_name,
        'zoom_link'       => isset($zoom_data->join_url) ? $zoom_data->join_url : '',
        'add_doctor_zoom_link' => isset($zoom_data->start_url) ? $zoom_data->start_url : '',
        'clinic_name' => isset($clinic_data->name) ? $clinic_data->name : '',
        'meet_link'   => isset($zoom_data->url) ? $zoom_data->url : '',
        'meet_event_link' => isset($zoom_data->event_url) ? $zoom_data->event_url : '',
        'clinic_email' => isset($clinic_data->email) ? $clinic_data->email:'',
        'clinic_contact_number' => isset($clinic_data->telephone_no) ? $clinic_data->telephone_no:'',
        'doctor_email' => isset($doctor_details->user_email) ? $doctor_details->user_email:'',
        'doctor_contact_number' => kcGetUserValueByKey('doctor',$doctor_id,'mobile_number'),
        'clinic_address' =>  (!empty($clinic_data->address) ? $clinic_data->address : '') .','.(!empty($clinic_data->city) ? $clinic_data->city : '').','.(!empty($clinic_data->country) ? $clinic_data->country : ''),
        'patient_contact_number' =>  kcGetUserValueByKey('patient',$patient_id,'mobile_number'),
        'description' => $appointment_data->description
    ];

    switch ($type){
        case 'doctor':
            $commonData['user_email'] = $doctor_details->user_email;
            $commonData['email_template_type'] ='doctor_book_appointment';
            break;
        case 'patient':
            $commonData['user_email'] = $patient_details->user_email;
            $commonData['email_template_type'] = 'book_appointment';
            break;
        case 'zoom_patient':
            if ($zoom_data != null) {
                $commonData['user_email'] = $patient_details->user_email;
                $commonData['email_template_type'] = 'zoom_link';
            }
            break;
        case 'zoom_doctor':
            if($zoom_data != null){
                $commonData['user_email'] = $doctor_details->user_email;
                $commonData['email_template_type'] =  'add_doctor_zoom_link';
            }
            break;
        case 'cancel_appointment':
            $commonData['user_email'] = $patient_details->user_email;
            $commonData['email_template_type'] = 'cancel_appointment';
            break;
        case 'clinic' :
            $commonData['user_email'] = isset($clinic_data->email) ? $clinic_data->email : 'demo@gmail.com' ;
            $commonData['email_template_type'] = 'clinic_book_appointment';
            break;
        case 'appointment_reminder':
            $commonData['id'] = $patient_id;
            $commonData['user_email'] = $patient_details->user_email;
            $commonData['email_template_type'] = 'book_appointment_reminder';
            break;
        case 'meet_doctor':
            /**
			 * zoom condition
			 */
            if($zoom_data != null){
                $commonData['user_email'] = $doctor_details->user_email;
                $commonData['email_template_type'] =  'add_doctor_meet_link';
            }
            break;
        case 'meet_patient':
            if ($zoom_data != null) {
                $commonData['user_email'] = $patient_details->user_email;
                $commonData['email_template_type'] = 'meet_link';
            }
            break;
    }

    return $commonData;
}

function kcCommonNotificationUserData($id,$password){
    $kcbase = new KCBase();
    $data = [];
    $result = get_userdata($id);
    $data['id'] = $id;
    $data['username'] = $result->user_login;
    $data['user_email'] = $result->user_email;
    $data['password'] = $password;
    if(in_array( $kcbase->getDoctorRole(),$result->roles)) {

        $data['doctor_name'] = $result->display_name;
        $data['email_template_type'] = 'doctor_registration';

    }elseif (in_array( $kcbase->getPatientRole(),$result->roles)){

        $data['patient_name'] = $result->display_name;
        $data['email_template_type'] = 'patient_register';

    }elseif (in_array( $kcbase->getClinicAdminRole(),$result->roles)){

        $data['clinic_admin_name'] = $result->display_name;
        $data['email_template_type'] = 'clinic_admin_registration';

    }elseif (in_array( $kcbase->getReceptionistRole(),$result->roles)){

        $data['Receptionist_name'] = $result->display_name;
        $data['email_template_type'] = 'receptionist_register';
    }

    return $data;
}

function kcCheckSmsOptionEnable(){
    $status = false;
    $get_sms_config  = get_option('sms_config_data', true );
    if(!empty($get_sms_config)){
        $get_sms_config = json_decode($get_sms_config);
        if(isKiviCareProActive() && !empty($get_sms_config->enableSMS)
            && in_array((string)$get_sms_config->enableSMS,[ '1','true'])){
            $status = true;
        }
    }
    return $status;
}

function kcCheckWhatsappOptionEnable(){
    $status = false;
    $get_sms_config  = get_option('whatsapp_config_data', true );
    if(!empty($get_sms_config)){
        $get_sms_config = json_decode($get_sms_config);
        if( isKiviCareProActive() && getKiviCareProVersion() >= '1.2.0' &&
            !empty($get_sms_config->enableWhatsApp) &&
            in_array((string)$get_sms_config->enableWhatsApp,['1','true'])){
            $status = true;
        }
    }
    return $status;
}

function kcCheckGoogleCalendarEnable()
{
    $status = false;
    $get_googlecal_data = get_option(KIVI_CARE_PREFIX . 'google_cal_setting',true);
    if (isKiviCareProActive() && gettype($get_googlecal_data) != 'boolean' ) {
        $status = in_array((string)$get_googlecal_data['enableCal'],['1','true']);
    }
    return $status;
}

function kcAppointmentMultiFileUploadEnable(){
    $data = get_option(KIVI_CARE_PREFIX . 'multifile_appointment',true);
    if(gettype($data) != 'boolean' &&  $data === 'on'){
        return true;
    }else{
        return false;
    }
}

function kcGetAppointmentTimeFormatOption(){
    $data = get_option(KIVI_CARE_PREFIX . 'appointment_time_format',true);
    return gettype($data) != 'boolean' ? $data : 'off';
}

function kcGetiUnderstand() {
    $status = get_option(KIVI_CARE_PREFIX.'i_understnad_loco_translate');
    return !(in_array((string)$status,['1','true']));
}

function kcGetTimeZoneOption(){

    $wp_timezone_string = wp_timezone_string();

    /* translators: timezone */
    $message = __( 'Current Timezone: ', 'kc-lang').esc_html( $wp_timezone_string).__('Your appointment slots work based on your current time zone.', 'kc-lang') ;
    $status = get_option(KIVI_CARE_PREFIX.'timezone_understand', true);
    return [
        'status' => true,
        'data' => in_array((string)$status,['1','true']),
        'message' => $message,
    ];

}

function kcPluginLoader(){
    $get_site_logo = get_option(KIVI_CARE_PREFIX.'site_loader');
    return isset($get_site_logo) && $get_site_logo!= null
    && $get_site_logo!= '' ? wp_get_attachment_url($get_site_logo) :
        KIVI_CARE_DIR_URI.'assets/images/kivi-Loader-400.gif';
}

function kcPatientUniqueIdEnable($type){
    $get_unique_id =  get_option(KIVI_CARE_PREFIX . 'patient_id_setting' );
    $status = false;
    $patient_uid = '';
    if(!empty($get_unique_id)){
       $status = !empty($get_unique_id['enable']) && in_array((string)$get_unique_id['enable'],['1','true']);
        $randomValue = kcGenerateString(6);
        if(!empty($get_unique_id['only_number']) && in_array((string)$get_unique_id['only_number'],['1','true']) && $get_unique_id['only_number'] !== 'false' ){
            $randomValue = sprintf("%06d", mt_rand(1, 999999));
        }
        if(!empty($get_unique_id['prefix_value'])) {
            $patient_uid .= $get_unique_id['prefix_value'].$randomValue;
        } else {
            $patient_uid .= $randomValue;
        }

        if(!empty($get_unique_id['postfix_value'])) {
            $patient_uid .= $get_unique_id['postfix_value'];
        }
    }

    if($type === 'status'){
        return $status;
    }

    return $patient_uid;
}

function kcToCheckUserIsNew(){

    $data = get_option(KIVI_CARE_PREFIX . 'new_user',true);
    if(gettype($data) != 'boolean' &&  in_array((string)$data,['1','true'])){
        return true;
    }else{
        return false;
    }
}

function kcWordpressLogostatusAndImage($type){

    if($type == 'status'){
        $status = get_option(KIVI_CARE_PREFIX.'wordpress_logo_status',true);
        return gettype($status) != 'boolean' && $status == 'on';
    }
    if($type == 'image'){
        $logoImage = get_option(KIVI_CARE_PREFIX . 'wordpress_logo',true);
        return gettype($logoImage) != 'boolean' && kcWordpressLogostatusAndImage('status')? wp_get_attachment_url($logoImage) : KIVI_CARE_DIR_URI.'assets/images/wp-logo.png';
    }

}

function doctorWeeklyAvailability($data) {
    if(!empty($data)) {
        global $wpdb;
        $table = $wpdb->prefix . "kc_" . "clinic_sessions";
        $doctor_id = (int)$data['doctor_id'];
        $clinic_condition = '' ; 
        if(!empty($data['clinic_id'])) {
            $clinic_condition = ' AND clinic_id = '.(int)$data['clinic_id'] ;
        }
        $result = $wpdb->get_results('select day, start_time, end_time  from '.$table.' where 0 = 0 and doctor_id = '.$doctor_id . $clinic_condition , ARRAY_A );
        if(!empty($result)) {
            $result = collect($result);
            $data = $result->groupBy('day')->toArray();
            return $data;
        } else {
            return [] ;
        }
    } else {
        return [] ;
    }    
}

function kcNotInPreviewmode()
{
    if (isset($_REQUEST['elementor-preview'])) {
        return false;
    }

    if (isset($_REQUEST['ver'])) {
        return false;
    }

    if (isset($_REQUEST['action']) && $_REQUEST['action'] == 'elementor') {
        return false;
    }

    $url_params = isset($_SERVER['HTTP_REFERER']) ? wp_parse_url(sanitize_url(wp_unslash($_SERVER['HTTP_REFERER'])), PHP_URL_QUERY) : wp_parse_url(sanitize_url(wp_unslash($_SERVER['REQUEST_URI'])), PHP_URL_QUERY);

    parse_str($url_params, $params);
    if (!empty($params['action']) && $params['action'] == 'elementor') {
        return false;
    }

    if (!empty($params['preview']) && $params['preview'] == 'true') {
        return false;
    }

    if (!empty($params['elementor-preview'])) {
        return false;
    }

    return true;
}

function kcClinicForElementor($type){
    $clinic = kcClinicList();
    $clinic = collect($clinic)->pluck('name','id')->toArray();
    if($type === 'all'){
        if(!empty($clinic) &&  count($clinic) > 0){
            return isKiviCareProActive() ? $clinic : [kcGetDefaultClinicId() => $clinic[1]];
        }else{
            return [ 'default' => __('No clinic Found','kc-lang')];
        }
    }else{
        if(!empty($clinic) && count($clinic) > 0){
            $keys = array_keys($clinic);
            return count($keys) > 0 ? $keys[0] : '';
        }else{
            return 'default';
        }
    }
}

function kcElementorAllCommonController($this_ele,$type){
    // book_button
    $this_ele->add_control(
        'iq_kivicare_'.$type.'_button_height',
        [
            'label' => esc_html__('Button Height', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::NUMBER,
            'selectors' => [
                '{{WRAPPER}} .appointment_button' => 'height: {{VALUE}}px;',
            ]
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_button_width',
        [
            'label' => esc_html__('Button Width', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::NUMBER,
            'selectors' => [
                '{{WRAPPER}} .appointment_button' => 'width: {{VALUE}}%;',
            ]
        ]
    );

    $this_ele->add_control(
        'iq_card_kivicare_book_appointment_border_radius',
        [
            'label' => esc_html__('Button Radius', 'kc-lang'),
            'size_units' => ['px', '%', 'em'],
            'type' => \Elementor\Controls_Manager::DIMENSIONS,
            'selectors' => [
                '{{WRAPPER}} .appointment_button' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};overflow:hidden;',
            ],
        ]
    );

    $this_ele->add_control(
        'iq_card_kivicare_book_appointment_margin',
        [
            'label' => esc_html__('Margin', 'kc-lang'),
            'size_units' => ['px', '%', 'em'],
            'type' => \Elementor\Controls_Manager::DIMENSIONS,
            'selectors' => [
                '{{WRAPPER}} .appointment_button' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
            ],
        ]
    );

    $this_ele->add_group_control(
        \Elementor\Group_Control_Typography::get_type(),
        [
            'name' => 'iq_kivicare_'.$type.'_button_font_typography',
            'label' => esc_html__('Font Typography', 'kc-lang'),
            'scheme' => \Elementor\Core\Schemes\Typography::TYPOGRAPHY_1,
            'selector' => '{{WRAPPER}} .appointment_button',
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_button_hover_notice',
        [
            'label' => esc_html__('For hover in Button keep background type classic of button', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::HEADING,
        ]
    );

    /**
     *  Button hover
     */
    $this_ele->start_controls_tabs( 'iq_kivicare_'.$type.'_button_style' );

    $this_ele->start_controls_tab(
        'iq_kivicare_'.$type.'_button_normal',
        [
            'label' => __( 'Normal', 'kc-lang' ),
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_button_font_color',
        [
            'label' => esc_html__('Font Color', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::COLOR,
            'selectors' => [
                '{{WRAPPER}} .appointment_button' => 'color: {{VALUE}};',
            ]
        ]
    );

    $this_ele->add_group_control(
        \Elementor\Group_Control_Background::get_type(),
        [
            'name' => 'iq_kivicare_'.$type.'_book_appointment',
            'label' => esc_html__('Button Background', 'kc-lang'),
            'types' => ['classic', 'gradient'],
            'selector' => '{{WRAPPER}} .appointment_button',
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_form_button_hover_border_size',
        [
            'label' => esc_html__('Button Border', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::NUMBER,
            'selectors' => [
                '{{WRAPPER}} .appointment_button' => 'border: {{VALUE}}px;',
            ]
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_form_button_hover_border_style',
        [
            'label' => esc_html__('Button Border style', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::SELECT,
            'options' => [
                'solid' => esc_html__('solid', 'kc-lang'),
                'dashed' => esc_html__('dashed', 'kc-lang'),
                'dotted' => esc_html__('dotted', 'kc-lang'),
                'double' => esc_html__('double', 'kc-lang'),
                'groove' => esc_html__('groove', 'kc-lang'),
                'ridge' => esc_html__('ridge', 'kc-lang'),
                'inset' => esc_html__('inset', 'kc-lang'),
                'outset' => esc_html__('outset', 'kc-lang'),
                'none' => esc_html__('none', 'kc-lang'),
                'hidden' => esc_html__('hidden', 'kc-lang'),
            ],
            'selectors' => [
                '{{WRAPPER}} .appointment_button' => 'border-style: {{VALUE}};',
            ]
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_form_button_border_color',
        [
            'label' => __( 'Border Color', 'kc-lang' ),
            'type' => \Elementor\Controls_Manager::COLOR,
            'selectors' => [
                '{{WRAPPER}} .appointment_button' => ' border-color: {{VALUE}};',
            ]
        ]
    );

    $this_ele->end_controls_tab();

    $this_ele->start_controls_tab(
        'iq_kivicare_'.$type.'_form_button_hover',
        [
            'label' => __( 'Hover', 'kc-lang' ),
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_button_font_color_hover',
        [
            'label' => esc_html__('Font Color', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::COLOR,
            'selectors' => [
                '{{WRAPPER}} .appointment_button:hover' => 'color: {{VALUE}};',
            ]
        ]
    );

    $this_ele->add_group_control(
        \Elementor\Group_Control_Background::get_type(),
        [
            'name' => 'iq_kivicare_'.$type.'_book_appointment_hover_button',
            'label' => esc_html__('Button Background', 'kc-lang'),
            'types' => ['classic', 'gradient'],
            'selector' => '{{WRAPPER}} .appointment_button:hover',
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_form_button_hover_border_size_hover',
        [
            'label' => esc_html__('Button Border', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::NUMBER,
            'selectors' => [
                '{{WRAPPER}} .appointment_button:hover' => 'border: {{VALUE}}px;',
            ]
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_form_button_hover_border_style_hover',
        [
            'label' => esc_html__('Button Border style', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::SELECT,
            'options' => [
                'solid' => esc_html__('solid', 'kc-lang'),
                'dashed' => esc_html__('dashed', 'kc-lang'),
                'dotted' => esc_html__('dotted', 'kc-lang'),
                'double' => esc_html__('double', 'kc-lang'),
                'groove' => esc_html__('groove', 'kc-lang'),
                'ridge' => esc_html__('ridge', 'kc-lang'),
                'inset' => esc_html__('inset', 'kc-lang'),
                'outset' => esc_html__('outset', 'kc-lang'),
                'none' => esc_html__('none', 'kc-lang'),
                'hidden' => esc_html__('hidden', 'kc-lang'),
            ],
            'selectors' => [
                '{{WRAPPER}} .appointment_button:hover' => 'border-style: {{VALUE}};',
            ]
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_form_button_hover_border_color',
        [
            'label' => __( 'Border Color', 'kc-lang' ),
            'type' => \Elementor\Controls_Manager::COLOR,
            'selectors' => [
                '{{WRAPPER}} .appointment_button:hover, {{WRAPPER}} .appointment_button:focus' => ' border-color: {{VALUE}};',
            ]
        ]
    );
    $this_ele->end_controls_tab();

    $this_ele->end_controls_tabs();

    $this_ele->end_controls_section();

    $this_ele->start_controls_section('iq_kivicare_button',
        [
            'label' => esc_html__('Pagination Button style', 'kc-lang'),
            'tab' => \Elementor\Controls_Manager::TAB_STYLE,
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_normal_button_height',
        [
            'label' => esc_html__('Button Height', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::NUMBER,
            'selectors' => [
                '{{WRAPPER}} .iq_kivicare_next_previous' => 'height: {{VALUE}}px;',
            ]
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_normal_button_width',
        [
            'label' => esc_html__('Button Width', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::NUMBER,
            'selectors' => [
                '{{WRAPPER}} .iq_kivicare_next_previous' => 'width: {{VALUE}}px;',
            ]
        ]
    );

    $this_ele->add_group_control(
        \Elementor\Group_Control_Typography::get_type(),
        [
            'name' => 'iq_kivicare_'.$type.'_normal_button_font_typography',
            'label' => esc_html__('Font Typography', 'kc-lang'),
            'scheme' => \Elementor\Core\Schemes\Typography::TYPOGRAPHY_1,
            'selector' => '{{WRAPPER}} .iq_kivicare_next_previous',
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_normal_button_hover_notice',
        [
            'label' => esc_html__('For hover in Button keep background type classic of button', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::HEADING,
        ]
    );

    /**
     *  Button hover
     */
    $this_ele->start_controls_tabs( 'iq_kivicare_'.$type.'_normal_button_style' );

    $this_ele->start_controls_tab(
        'iq_kivicare_'.$type.'_normal_button',
        [
            'label' => __( 'Normal', 'kc-lang' ),
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_normal_button_font_color',
        [
            'label' => esc_html__('Font Color', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::COLOR,
            'selectors' => [
                '{{WRAPPER}} .iq_kivicare_next_previous' => 'color: {{VALUE}};',
            ]
        ]
    );

    $this_ele->add_group_control(
        \Elementor\Group_Control_Background::get_type(),
        [
            'name' => 'iq_kivicare_'.$type.'_button',
            'label' => esc_html__('Button Background', 'kc-lang'),
            'types' => ['classic', 'gradient'],
            'selector' => '{{WRAPPER}} .iq_kivicare_next_previous',
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_form_normal_button_hover_border_size',
        [
            'label' => esc_html__('Button Border', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::NUMBER,
            'selectors' => [
                '{{WRAPPER}} .iq_kivicare_next_previous' => 'border: {{VALUE}}px;',
            ]
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_form_normal_button_hover_border_style',
        [
            'label' => esc_html__('Button Border style', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::SELECT,
            'options' => [
                'solid' => esc_html__('solid', 'kc-lang'),
                'dashed' => esc_html__('dashed', 'kc-lang'),
                'dotted' => esc_html__('dotted', 'kc-lang'),
                'double' => esc_html__('double', 'kc-lang'),
                'groove' => esc_html__('groove', 'kc-lang'),
                'ridge' => esc_html__('ridge', 'kc-lang'),
                'inset' => esc_html__('inset', 'kc-lang'),
                'outset' => esc_html__('outset', 'kc-lang'),
                'none' => esc_html__('none', 'kc-lang'),
                'hidden' => esc_html__('hidden', 'kc-lang'),
            ],
            'selectors' => [
                '{{WRAPPER}} .iq_kivicare_next_previous' => 'border-style: {{VALUE}};',
            ]
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_form_normal_button_border_color',
        [
            'label' => __( 'Border Color', 'kc-lang' ),
            'type' => \Elementor\Controls_Manager::COLOR,
            'selectors' => [
                '{{WRAPPER}} .appointment_button' => ' border-color: {{VALUE}};',
            ]
        ]
    );

    $this_ele->end_controls_tab();

    $this_ele->start_controls_tab(
        'iq_kivicare_'.$type.'_form_normal_button_hover',
        [
            'label' => __( 'Hover', 'kc-lang' ),
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_normal_button_font_color_hover',
        [
            'label' => esc_html__('Font Color', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::COLOR,
            'selectors' => [
                '{{WRAPPER}} .iq_kivicare_next_previous:hover' => 'color: {{VALUE}};',
            ]
        ]
    );

    $this_ele->add_group_control(
        \Elementor\Group_Control_Background::get_type(),
        [
            'name' => 'iq_kivicare_'.$type.'_normal_button_hover_button',
            'label' => esc_html__('Button Background', 'kc-lang'),
            'types' => ['classic', 'gradient'],
            'selector' => '{{WRAPPER}} .iq_kivicare_next_previous:hover',
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_form_normal_button_hover_border_size_hover',
        [
            'label' => esc_html__('Button Border', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::NUMBER,
            'selectors' => [
                '{{WRAPPER}} .iq_kivicare_next_previous:hover' => 'border: {{VALUE}}px;',
            ]
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_form_normal_button_hover_border_style_hover',
        [
            'label' => esc_html__('Button Border style', 'kc-lang'),
            'type' => \Elementor\Controls_Manager::SELECT,
            'options' => [
                'solid' => esc_html__('solid', 'kc-lang'),
                'dashed' => esc_html__('dashed', 'kc-lang'),
                'dotted' => esc_html__('dotted', 'kc-lang'),
                'double' => esc_html__('double', 'kc-lang'),
                'groove' => esc_html__('groove', 'kc-lang'),
                'ridge' => esc_html__('ridge', 'kc-lang'),
                'inset' => esc_html__('inset', 'kc-lang'),
                'outset' => esc_html__('outset', 'kc-lang'),
                'none' => esc_html__('none', 'kc-lang'),
                'hidden' => esc_html__('hidden', 'kc-lang'),
            ],
            'selectors' => [
                '{{WRAPPER}} .iq_kivicare_next_previous:hover' => 'border-style: {{VALUE}};',
            ]
        ]
    );

    $this_ele->add_control(
        'iq_kivicare_'.$type.'_form_normal_button_hover_border_color',
        [
            'label' => __( 'Border Color', 'kc-lang' ),
            'type' => \Elementor\Controls_Manager::COLOR,
            'selectors' => [
                '{{WRAPPER}} .iq_kivicare_next_previous:hover, {{WRAPPER}} .iq_kivicare_next_previous:focus' => ' border-color: {{VALUE}};',
            ]
        ]
    );

    $this_ele->add_control(
        'iq_card_kivicare_normal_button_border_radius',
        [
            'label' => esc_html__('Button Radius', 'kc-lang'),
            'size_units' => ['px', '%', 'em'],
            'type' => \Elementor\Controls_Manager::DIMENSIONS,
            'selectors' => [
                '{{WRAPPER}} .iq_kivicare_next_previous' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};overflow:hidden;',
            ],
        ]
    );
    $this_ele->end_controls_tab();
    $this_ele->end_controls_tabs();
    $this_ele->add_control(
        'iq_card_kivicare_pagination_border_radius',
        [
            'label' => esc_html__('Button Radius', 'kc-lang'),
            'size_units' => ['px', '%', 'em'],
            'type' => \Elementor\Controls_Manager::DIMENSIONS,
            'selectors' => [
                '{{WRAPPER}} .iq_kivicare_next_previous' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};overflow:hidden;',
            ],
        ]
    );
    $this_ele->add_control(
        'iq_kivicare_'.$type.'_pagination-margin',
        [
            'label' => esc_html__('Margin', 'kc-lang'),
            'size_units' => ['px', '%', 'em'],
            'type' => \Elementor\Controls_Manager::DIMENSIONS,
            'selectors' => [
                '{{WRAPPER}} .kivi-pagination' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
            ],
        ]
    );
    $this_ele->add_control(
        'iq_kivicare_'.$type.'_pagination-padding',
        [
            'label' => esc_html__('Padding', 'kc-lang'),
            'size_units' => ['px', '%', 'em'],
            'type' => \Elementor\Controls_Manager::DIMENSIONS,
            'default'   => [
                'top' => '10',
                'right' => '0',
                'bottom' => '0',
                'left' => '0',
                'unit' => 'px',
                'isLinked' => false,
            ],
            'selectors' => [
                '{{WRAPPER}} .kivi-pagination' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
            ],
        ]
    );
}

function kcAddCronJob($type, $callback){
    add_filter( 'cron_schedules',  function ( $schedules ) {
        $schedules['every_set_minutes'] = array(
            'interval'  => 300,
            'display'   => __( 'Every 5 Minutes', 'kc-lang' )
        );
        return $schedules;
    });
    // Schedule an action if it's not already scheduled
    if ( ! wp_next_scheduled( $type ) ) {
        wp_schedule_event( time(), 'every_set_minutes', $type );
    }

    // Hook into that action that'll fire in set minutes
    add_action( $type, $callback);
}

function patientAppointmentReminder(){
    $reminder_setting = get_option(KIVI_CARE_PREFIX . 'email_appointment_reminder', true);
    global $wpdb;
    $whatsappOptionEnable = kcCheckWhatsappOptionEnable();
    $smsOptionEnable = kcCheckSmsOptionEnable();
    $smsSendFunctionExists = function_exists('kcProSendSms');
    $whatsappSendFunctionExists = function_exists('kcProWhatappSms');
    $successStatus = ['sent','queued','delivered'];

    if (gettype($reminder_setting) != 'boolean') {
        if (isset($reminder_setting['status']) && isset($reminder_setting['sms_status']) && isset($reminder_setting['whatapp_status']) && isset($reminder_setting['time'])) {
            $date1 = date('Y-m-d H:i:s', strtotime(current_time("Y-m-d H:i:s")) + ((int) $reminder_setting['time']  - 1 ) * 3600);
            $date2 = date('Y-m-d H:i:s', strtotime(current_time("Y-m-d H:i:s")) + ((int) $reminder_setting['time']) * 3600);
            $query = "SELECT * FROM {$wpdb->prefix}kc_appointments WHERE CAST(CONCAT(appointment_start_date, ' ', appointment_start_time) AS DATETIME ) BETWEEN '{$date1}' AND '{$date2}' AND status=1";
            $appointment_data = collect($wpdb->get_results($query))->toArray();
            if (!empty($appointment_data) && is_array($appointment_data) && count($appointment_data) > 0) {
                $msg_reminder_table = $wpdb->prefix . "kc_appointment_reminder_mapping";
                foreach ($appointment_data as $appoint) {
                    $data = kcCommonNotificationData($appoint, [], '', 'appointment_reminder');
                    $appointment_reminder_data_id = $wpdb->get_var("select id from {$msg_reminder_table} where msg_send_date=CURDATE() AND appointment_id=" . (int)$appoint->id);
                    if (empty($appointment_reminder_data_id)) {
                        $wpdb->insert($msg_reminder_table, [
                            'appointment_id' => $appoint->id,
                            'msg_send_date' => current_time('Y-m-d'),
                            'email_status' => 0,
                            'sms_status' =>  0,
                            'whatsapp_status' =>  0,
                        ]);
                        $appointment_reminder_data_id = $wpdb->insert_id;
                    }

                    $appointment_reminder_table_data = $wpdb->get_row("SELECT * FROM {$msg_reminder_table} WHERE id={$appointment_reminder_data_id}");

                    $temp = [
                        'appointment_id' => $appoint->id,
                    ];

                    if ($appointment_reminder_table_data->email_status != 1
                        && $reminder_setting['status'] == 'on') {
                        $temp['email_status'] = kcSendEmail($data) ? 1 : 0;
                    }
                    if ($smsOptionEnable && $smsSendFunctionExists &&
                        $appointment_reminder_table_data->sms_status != 1
                        && $reminder_setting['sms_status'] == 'on') {
                        $smsdata = kcProSendSms('book_appointment_reminder', $data);
                        $temp['sms_status'] = 1;
                        if (isset($smsdata['status']['sms']->status)
                            && in_array(trim($smsdata['status']['sms']->status),$successStatus)) {
//                            $temp['sms_status'] = 1;
                        }
                    }
                    if ($whatsappOptionEnable && $whatsappSendFunctionExists
                        && $appointment_reminder_table_data->whatsapp_status != 1
                        && $reminder_setting['whatapp_status'] == 'on') {
                        $whatsappData = kcProWhatappSms('book_appointment_reminder', $data);
                        $temp['whatsapp_status'] = 1;
                        if (isset($whatsappData['status']['whatsapp']->status)
                            && in_array(trim($whatsappData['status']['whatsapp']->status),$successStatus)) {
//                            $temp['whatsapp_status'] = 1;
                        }
                    }

                    $wpdb->update($msg_reminder_table, $temp, ['id' => $appointment_reminder_table_data->id]);
                }
            }
        }
    }
}

function kcGetAppointmentPageUrl(){
    global $wpdb;
    $data =  $wpdb->get_var("SELECT ID FROM {$wpdb->posts} WHERE post_status='publish' and post_content LIKE '%[kivicareBookAppointment%'", ARRAY_N);
    $appointmentPageUrl = '';
    if(!empty($data)){
        $appointmentPageUrl = get_permalink($data);
    }
    return $appointmentPageUrl;
}

function kcGetDashboardPageUrl(){
    global $wpdb;
    $data =  $wpdb->get_row("SELECT ID FROM {$wpdb->posts} WHERE post_status='publish' and post_content LIKE '%[patientDashboard%'", ARRAY_N);
    $appointmentPageUrl = '';
    if($data != null ){
        $appointmentPageUrl = get_permalink(isset($data[0]) ? $data[0] : 0  );
    }
    return $appointmentPageUrl;
}

function kcAppointmentIsWoocommerceOrder($appointment_id){
    global $wpdb;
    $appointment_id = (int)$appointment_id;
    $postTable = $wpdb->prefix . 'posts';
    $postMetaTable = $wpdb->prefix . 'postmeta';
    $query = "select {$postTable}.ID from {$postTable} 
              left join {$postMetaTable} on {$postMetaTable}.post_id={$postTable}.ID
              where post_type='shop_order' and meta_key='kivicare_appointment_id' and meta_value={$appointment_id}";
    return $wpdb->get_var($query);
}

function kcWoocommerceRedirect($appointment_id, $data){
    $kcbase = new KCBase();
    $appointment_id = (int)$appointment_id;
    $message = '';
    $res_data ='';
    $status = false ;
    if ($kcbase->getLoginUserRole() === $kcbase->getPatientRole() && iskcWooCommerceActive()
        && get_option( KIVI_CARE_PREFIX . 'woocommerce_payment') == 'on' ) {
        $status = true ;
        $data['doctor_id']['id'] = (int)$data['doctor_id']['id'];
        $temp = [
            'appointment_id' => $appointment_id,
            'doctor_id' => $data['doctor_id']['id']
        ];

        if(!empty($data['widgetType'])){
            $temp['widgetType'] = $data['widgetType'];
        }
        if (isKiviCareProActive()) {
            $res_data = apply_filters('kcpro_woocommerce_add_to_cart', $temp);
            $message = esc_html__('appointment successfully booked, Please check your email ', 'kc-lang');
        }else if (isKiviCareTelemedActive()) {
            $res_data = apply_filters('kct_woocommerce_add_to_cart', $temp);
            $message = esc_html__('appointment successfully booked, Please check your email for zoom meeting link.', 'kc-lang');
        }else if(isKiviCareGoogleMeetActive()){
            $res_data = apply_filters('kcgm_woocommerce_add_to_cart', $temp);
            $message = esc_html__('appointment successfully booked, Please check your email ', 'kc-lang');
        }

    }

    return  [
        'status' => $status,
        'message' => $message,
        'woocommerce_cart_data' => $res_data
    ];

}

// woocommerce hookup and filter
function kivicareWooocommerceAddToCart ($filterData) {

    global $wpdb;

    $status = ['status' => 0];
    $filterData['appointment_id'] = (int)$filterData['appointment_id'];
    $filterData['doctor_id'] = (int)$filterData['doctor_id'];
    $condition = ['id' => $filterData['appointment_id']];
    $wpdb->update($wpdb->prefix.'kc_appointments', $status, $condition);
    $kiviWooProductId = kivicareWoocommerceProduct($filterData);

    if ( defined( 'REST_REQUEST' ) && REST_REQUEST ) {

        global $woocommerce;
        $filterData['patient_id'] = (int)$filterData['patient_id'];
        $patient_data = kcGetUserData($filterData['patient_id']);

        $address = array(
            'first_name' => $patient_data->first_name,
            'last_name'  => $patient_data->last_name,
            'company'    => '',
            'email'      => $patient_data->user_email,
            'phone'      => isset($patient_data->basicData) && isset($patient_data->basicData->mobile_number) ? $patient_data->basicData->mobile_number : null ,
            'address_1'  => isset($patient_data->basicData) && isset($patient_data->basicData->address) ? $patient_data->basicData->address : null ,
            'address_2'  => isset($patient_data->basicData) && isset($patient_data->basicData->address) ? $patient_data->basicData->address : null ,
            'city'       => isset($patient_data->basicData) && isset($patient_data->basicData->city) ? $patient_data->basicData->city : null ,
            'state'      => "",
            'postcode'   => isset($patient_data->basicData) && isset($patient_data->basicData->postal_code) ? $patient_data->basicData->postal_code : null ,
            'country'    => isset($patient_data->basicData) && isset($patient_data->basicData->country) ? $patient_data->basicData->country : null ,
        );

        // Now we create the order
        $order = wc_create_order();

        foreach ($kiviWooProductId as $key => $value ){
            $objProduct = wc_get_product($value);
            $order->add_product( $objProduct );
        }

        // This is an existing SIMPLE product
        $order->set_address( $address, 'billing' );
        $order->set_customer_id( $filterData['patient_id'] );
        $order->calculate_totals();
        $order->update_status("Completed", 'Imported order', TRUE);

        update_post_meta( $order->get_id(), 'kivicare_appointment_id', $filterData['appointment_id'] );
        update_post_meta( $order->get_id(), 'kivicare_doctor_id', $filterData['doctor_id'] );
        // wc_add_order_item_meta($kiviWooProductId, 'kivicare_appointment_id', $filterData['appointment_id'] );
        // wc_add_order_item_meta($kiviWooProductId, 'doctor_id', $filterData['doctor_id'] );

        return [
            'status'  => true,
            'woocommerce_redirect' => $order->get_checkout_payment_url()
        ];

    }else {

        KivicareSetWoocommerCustomerCookie();
        WC()->cart->empty_cart();
        $temp = ['kivicare_appointment_id' => $filterData['appointment_id'], 'doctor_id' => $filterData['doctor_id']];
        if(!empty($filterData['widgetType'])){
            $temp['widgetType'] = $filterData['widgetType'];
        }
        foreach ($kiviWooProductId as $key => $value ){
            WC()->cart->add_to_cart( $value, 1, '', '', $temp );
        }
    }

    return [
        'status'  => true,
        'woocommerce_redirect' => wc_get_checkout_url()
    ];

}

function kivicareWoocommerceProduct($filterData) {

    global $wpdb;
    $appointments_service_table = $wpdb->prefix . 'kc_service_doctor_mapping';

    $id =[];

    $appointment_id = (int)$filterData['appointment_id'];
    $filterData['doctor_id'] = (int)$filterData['doctor_id'];
    $appointment_services = (new KCAppointmentServiceMapping())->get_by([
        'appointment_id' => $appointment_id,
    ]);

    foreach ($appointment_services as $key => $value) {
        $service_name = kcGetServiceById($value->service_id);

        $service_charges = kcGetServiceCharges([
            'service_id' => $value->service_id,
            'doctor_id' =>  $filterData['doctor_id']
        ]);
        $product_attachment = !empty($service_charges->image) ? $service_charges->image : '' ;
        $appointments_service_table = $wpdb->prefix . 'kc_service_doctor_mapping';
        $data = $wpdb->get_var('select extra from '.$appointments_service_table.' where id='.(int)$service_charges->id);
        $data = json_decode($data);
        $kiviWooProductId = $data->product_id != null ? (int)$data->product_id : 0 ;

        if( !get_post_status( $kiviWooProductId )  && 'publish' !== get_post_status( $kiviWooProductId ) &&  'product' != get_post_type($kiviWooProductId)) {

            $kiviWooProductId = wp_insert_post([
                'post_title'    => $service_name->name,
                'post_type'     => 'product',
                'post_status'   => 'publish'
            ]);

            $wpdb->update($appointments_service_table,['extra' => json_encode(["product_id" =>$kiviWooProductId])],['id' =>$service_charges->id ]);


            wp_set_object_terms( $kiviWooProductId, 'simple', 'product_type' );

            update_post_meta( $kiviWooProductId, '_visibility', 'hidden' );
            update_post_meta( $kiviWooProductId, '_thumbnail_id', $product_attachment );
            update_post_meta( $kiviWooProductId, '_stock_status', 'instock');
            update_post_meta( $kiviWooProductId, 'total_sales', '0' );
            update_post_meta( $kiviWooProductId, '_downloadable', 'yes' );
            update_post_meta( $kiviWooProductId, '_virtual', 'yes' );
            update_post_meta( $kiviWooProductId, '_regular_price', '' );
            update_post_meta( $kiviWooProductId, '_sale_price', $service_charges->charges );
            update_post_meta( $kiviWooProductId, '_purchase_note', '' );
            update_post_meta( $kiviWooProductId, '_featured', 'no' );
            update_post_meta( $kiviWooProductId, '_weight', '' );
            update_post_meta( $kiviWooProductId, '_length', '' );
            update_post_meta( $kiviWooProductId, '_width', '' );
            update_post_meta( $kiviWooProductId, '_height', '' );
            update_post_meta( $kiviWooProductId, '_sku', '' );
            update_post_meta( $kiviWooProductId, '_product_attributes', [] );
            update_post_meta( $kiviWooProductId, '_sale_price_dates_from', '' );
            update_post_meta( $kiviWooProductId, '_sale_price_dates_to', '' );
            update_post_meta( $kiviWooProductId, '_price', $service_charges->charges );
            update_post_meta( $kiviWooProductId, '_sold_individually', 'yes' );
            update_post_meta( $kiviWooProductId, '_manage_stock', 'no' );
            update_post_meta( $kiviWooProductId, '_backorders', 'no' );
            wc_update_product_stock($kiviWooProductId, 0, 'set');
            update_post_meta( $kiviWooProductId, '_stock', '' );
            update_post_meta($kiviWooProductId,'kivicare_service_id',$service_charges->id);
            update_post_meta($kiviWooProductId,'kivicare_doctor_id',$filterData['doctor_id']);

        } else {
            update_post_meta( $kiviWooProductId, '_downloadable', 'yes' );
            update_post_meta( $kiviWooProductId, '_virtual', 'yes' );
        }
    }

    foreach ($appointment_services as $key => $value) {
        $service_charges = kcGetServiceCharges([
            'service_id' => $value->service_id,
            'doctor_id' =>  $filterData['doctor_id']
        ]);
        $data = $wpdb->get_var('select extra from '.$appointments_service_table.' where id='.(int)$service_charges->id);
        $data = json_decode($data);
        $kiviWooProductId = $data->product_id ;
        if($kiviWooProductId != null){
            $id[] = $kiviWooProductId;
        }
    }

    return $id;
}

// set cookie for woocommerce payment
function KivicareSetWoocommerCustomerCookie() {

    if ( WC()->session && WC()->session instanceof \WC_Session_Handler && WC()->session->get_session_cookie() === false )
    {
        WC()->session->set_customer_session_cookie( true );
    }
    return true;
}

// woocommerce cart items
function kivicareGetCartItemsFromSession($item,$values,$key){
    if (array_key_exists( 'kivicare_appointment_id', $values ) )
    {
        $item['kivicare_appointment_id'] = $values['kivicare_appointment_id'];
    }
    if(array_key_exists( 'doctor_id', $values )){

        $item['doctor_id'] = $values['doctor_id'];
    }
    if(array_key_exists( 'widgetType', $values )){

        $item['widgetType'] = $values['widgetType'];
    }

    return $item;
}

// woocommerce kivicare appointment status change based on woocommerce order status.
function kivicareWooOrderStatusChangeCustom($order_id,$old_status,$new_status){
    global $wpdb;
    if(!empty($order_id) && get_post_status ( $order_id ) )
    {
        $appointment_id = get_post_meta($order_id,'kivicare_appointment_id',true);
        if(!empty($appointment_id)){
            $status=['status'=>0];
            if( !empty($new_status) && $new_status == 'completed' ){
                $status=['status'=>1];
            }
            $condition=['id'=>$appointment_id];
            $wpdb->update($wpdb->prefix . 'kc_appointments',$status,$condition);
        }
    }
}

function kivicareWoocommercePaymentComplete($order_id)
{
    global $wpdb;

    if(!empty($order_id) && get_post_status ( $order_id )) {
        $appointment_id = get_post_meta((int)$order_id, 'kivicare_appointment_id', true);
        if(empty($appointment_id)){
            return;
        }
        $service_category_table=$wpdb->prefix .'kc_services';
        $appointment_mapping_table = $wpdb->prefix .'kc_appointment_service_mapping';
        $get_service_query = "SELECT {$appointment_mapping_table}.*,{$service_category_table}.name
                                   FROM {$appointment_mapping_table} left join {$service_category_table} 
                                   on {$service_category_table}.id= {$appointment_mapping_table}.service_id 
                                   WHERE {$appointment_mapping_table}.appointment_id = " . $appointment_id;
        $appointment_service = collect($wpdb->get_results($get_service_query))->pluck('name')->toArray();
        if(!empty($appointment_service)){
            //hook on appointment payment complete
            do_action('kc_appointment_payment_complete',$appointment_id);
            $appointmentHaveTelemedService = in_array('telemed',$appointment_service) || in_array('Telemed',$appointment_service);
            $serviceName = implode(',',$appointmentHaveTelemedService);
            kcAppointmentDoctorEmail($appointment_id,$serviceName);
            kcAppointmentPatientEmail($appointment_id,$serviceName);
            kcSendAppointmentSmsOnPro($appointment_id);
            if(kcCheckGoogleCalendarEnable()){
                apply_filters('kcpro_save_appointment_event', [
                    'appoinment_id' => $appointment_id,
                ]);
            }
            if($appointmentHaveTelemedService){
                if (isKiviCareTelemedActive() && kcCheckDoctorTelemedType($appointment_id) != 'googlemeet') {
                    apply_filters('kct_send_zoom_link', ['appointment_id' => $appointment_id]);
                } else if(isKiviCareGoogleMeetActive()) {
                    apply_filters('kcgm_save_appointment_event_link_send', ['appoinment_id' => $appointment_id]);
                }
            }
        }
    }
}

function kivicareWoocommerceOrderDataAfterOrderDetails($order)
{
    $orderId = $order->get_id();
    $doctorId = get_post_meta($orderId, 'kivicare_doctor_id', true);
    $appointmentId = get_post_meta($orderId, 'kivicare_appointment_id', true);
    if ($appointmentId != null && $doctorId != null) {
        global $wpdb;
        $doctorName = get_user_by('id', $doctorId);
        $appointment_services = (new KCAppointmentServiceMapping())->get_by([
            'appointment_id' => $appointmentId,
        ]);

        $serviceName = [];
        foreach ($appointment_services as $key => $value) {
            $service_name = kcGetServiceById($value->service_id);
            $serviceName[] = $service_name->name;
        }

        $appointmentTable = $wpdb->prefix . 'kc_appointments';

        $appointmentData = $wpdb->get_row('select * from ' . $appointmentTable . ' where id=' . $appointmentId);

        $patientName = get_user_by('id', $appointmentData->patient_id);

        ?>
        <p class="form-field form-field-wide wc-order-status kivicare-orderinfo">
            <label for="kivicare_doctor">
                <strong>
                    <?php
                         /* translators: doctor name*/
                         echo esc_html__( 'Doctor Name : ', 'kc-lang').esc_html(  $doctorName->display_name ) ;
                    ?>
                </strong>
            </label>
            <label for="kivicare_doctor">
                <strong>
                    <?php
                        /* translators: patient name*/
                        echo esc_html__( 'Patient Name  : ', 'kc-lang').esc_html(  $patientName->display_name ) ;
                    ?>
                </strong>
            </label>
            <label for="kivicare_service_name">
                <strong>
                    <?php
                        $serviceNameWoo = implode(", ", $serviceName);
                        /* translators: patient name*/
                        echo esc_html__( 'Service Name  : ', 'kc-lang').esc_html(  $serviceNameWoo) ;
                    ?>
                </strong>
            </label>
            <label for="kivicare_appointment_date">
                <strong>
                    <?php
                        /* translators: Appointment*/
                        echo esc_html__( 'Appointment Date : ', 'kc-lang').esc_html(  $appointmentData->appointment_start_date) ;
                    ?>
                </strong>
            </label>
            <label for="kivicare_appointment_time">
                <strong>
                    <?php
                        /* translators: Appointment*/
                         echo esc_html__( 'Appointment Time : ', 'kc-lang').esc_html(  $appointmentData->appointment_start_time) ;
                    ?>
                </strong>
            </label>
            <label for="kivicare_appointment_print">
                <button class="button button-primary kivicare-order-print-appointment">
                    <strong>
                        <?php
                        /* translators: Appointment*/
                        echo esc_html__( 'Print Appointment Detail', 'kc-lang');
                        ?>
                    </strong>
                </button>
                <button class="button button-primary kivicare-order-print-appointment-disabled" style="display:none; cursor: not-allowed;pointer-events: none;">
                    <strong>
                        <?php
                        /* translators: Appointment*/
                        echo esc_html__( 'Loading.....', 'kc-lang');
                        ?>
                    </strong>
                </button>
                <script>
                    jQuery(document).on('click','.kivicare-order-print-appointment', function(e){
                       e.preventDefault();
                        jQuery('.kivicare-order-print-appointment').hide();
                        jQuery('.kivicare-order-print-appointment-disabled').show();
                        jQuery.ajax({
                            url: '<?php echo esc_js(admin_url('admin-ajax.php'));?>',
                            type: "get",
                            dataType: "json",
                            data: {
                                action: "ajax_get",
                                id:'<?php echo esc_js($appointmentId); ?>',
                                _ajax_nonce:'<?php echo esc_js(wp_create_nonce('ajax_get'));?>',
                                route_name:'get_appointment_print'
                            },
                            success: function (response) {
                                jQuery('.kivicare-order-print-appointment').show();
                                jQuery('.kivicare-order-print-appointment-disabled').hide();
                                if (response.status !== undefined && response.status === true) {
                                    var w = window.screen.availWidth ;
                                    var h = window.screen.availHeight ;
                                    var WindowObject = window.open('', "PrintWindow", "width="+w+",height="+h+",top=200,left=200");
                                    WindowObject.document.writeln(response.data);
                                    setTimeout(() => {
                                        WindowObject.document.close();
                                        WindowObject.focus();
                                        WindowObject.print();
                                        WindowObject.close();
                                    },500)
                                }
                            },
                            error: function () {
                                jQuery('.kivicare-order-print-appointment').show();
                                jQuery('.kivicare-order-print-appointment-disabled').hide();
                                console.log('fail');
                            }
                        });
                    })
                </script>
            </label>
        </p>
        <?php
    }
}

function kivicareServiceDeleteOnProductDelete($product_id){
    if('product' === get_post_type($product_id)){
        $serviceId = get_post_meta($product_id,'kivicare_service_id',true);
        if( $serviceId != null){
            global $wpdb;
            $appointments_service_table = $wpdb->prefix . 'kc_service_doctor_mapping';
            $wpdb->update($appointments_service_table,['extra' => ''],['id' =>$serviceId]);
        }
    }
}

function kivicareServiceUpdateOnProductUpdated( $product_id ) {
    global $wpdb;
    $appointments_service_table = $wpdb->prefix . 'kc_service_doctor_mapping';
    $service_table = $wpdb->prefix . 'kc_services';
    $product = wc_get_product( $product_id );
    $serviceId = get_post_meta($product_id,'kivicare_service_id',true);
    $doctorId = get_post_meta($product_id,'kivicare_doctor_id',true);
    if($doctorId != null && $serviceId != null){
        $id = $wpdb->get_var('select ser.id from '.$appointments_service_table.' as map join '.$service_table .' as ser on ser.id = map.service_id where map.id='.(int)$serviceId);
        if($id != null ){
            $service_data = $wpdb->get_results('select id from '.$appointments_service_table.' where service_id='.(int)$id );
            if($service_data != null && count($service_data) > 0 ){
                foreach ($service_data as $s){
                    $product_mapping_id = kivicareGetProductIdOfService($s->id);
                    if($product_mapping_id != null &&  get_post_status( $product_mapping_id )){
                        $my_post = array(
                            'ID'           => $product_mapping_id,
                            'post_title'   => get_the_title($product_id),
                        );
                        wp_update_post( $my_post );
                    }
                }
            }
            $wpdb->update($service_table,['name' =>get_the_title($product_id)],['id' => $id]);
        }
        $wpdb->update($appointments_service_table,['charges' => (int)$product->get_price()],['id' => (int)$serviceId,'doctor_id' =>(int)$doctorId]);
    }
}

/***
    * woocommerce kivicare appointment save for oreder-kivicare appointment mapping
 */
function kivicareSaveToPostMeta( $order_id ) {
    if(WC()->session->get('kivicare_appointment_id') !== 0 && WC()->session->get('doctor_id') !== 0 ){
        foreach( WC()->cart->get_cart() as $cart_item ){
            $kivicare_appointment_id= $cart_item['kivicare_appointment_id'];
            $kivicare_doctor_id=$cart_item['doctor_id'];
            $kivicare_widget_type = $cart_item['widgetType'];
        }
        update_post_meta( $order_id, 'kivicare_doctor_id', $kivicare_doctor_id );
        update_post_meta( $order_id, 'kivicare_appointment_id',$kivicare_appointment_id );
        if(!empty($kivicare_widget_type)){
            update_post_meta( $order_id, 'kivicare_widget_type',$kivicare_widget_type );
        }
    }
}

function kivicareServiceDetailOnWooProductTabs($tabs){
    global $post;
    $id = $post->ID;
    if('product' === get_post_type($id)){
        $serviceId = get_post_meta($id,'kivicare_service_id',true);
        $doctorId = get_post_meta($id,'kivicare_doctor_id',true);
        if($doctorId != null && $serviceId != null){
            $tabs['kivicare'] = array(
                'label'		=> __( 'kivicare', 'kiviCare-telemed-addon' ),
                'target'	=> 'kivicare_options',
                'class'		=> array( 'kivicare_product_icon'),
                'priority' => 10,
            );
        }
    }

    return $tabs;
}

function kivicareServiceWooProductTabContent()
{

    global $post;
    $id = $post->ID;
    $doctorId = get_post_meta($id, 'kivicare_doctor_id', true);
    if($doctorId != null){
        $doctorName = get_user_by('id', $doctorId);
        ?>
        <div id='kivicare_options' class='panel woocommerce_options_panel'><?php

        ?>
        <div class='options_group'><?php

            woocommerce_wp_text_input(array(
                'id' => '_valid_for_days',
                'label' => __('Doctor Name:', 'kc-lang'),
                'type' => 'text',
                'value' => $doctorName->display_name,
                'custom_attributes' => array(
                    'readonly' => 'readonly'
                ),
            ));

            ?></div>

        </div><?php
    }

}

function kivicareCheckoutRedirectWidgetPayment($order_id){
    $order = wc_get_order( $order_id );
    $appointment_id = get_post_meta($order_id, 'kivicare_appointment_id', true);
    $kivicare_widget_type = get_post_meta($order_id, 'kivicare_widget_type', true);
    if(!empty($appointment_id) && !empty($kivicare_widget_type)){
        if ( ! $order->has_status( 'failed' ) ) {
            global $wpdb;
            if($kivicare_widget_type === 'phpWidget'){
                $appointmentPageId =  $wpdb->get_var("SELECT ID FROM {$wpdb->posts} WHERE post_status='publish' and post_content LIKE '%[kivicareBookAppointment%'");
            }elseif ($kivicare_widget_type === 'popupPhpWidget'){
                $appointmentPageId =  $wpdb->get_var("SELECT ID FROM {$wpdb->posts} WHERE post_status='publish' and post_content LIKE '%[kivicareBookAppointmentButton%'");
            }
            if(!empty($appointmentPageId)){
                $pageUrl = get_permalink($appointmentPageId );
                $query = wp_parse_url($pageUrl, PHP_URL_QUERY);
                if ($query) {
                    $pageUrl .= '&confirm_page='.(int)$appointment_id;
                } else {
                    $pageUrl .= '?confirm_page='.(int)$appointment_id ;
                }
                wp_safe_redirect( $pageUrl );
                exit;
            }
        }
    }
}

function kcWoocommerceFillCheckoutFields( $fields ) {
    if(WC()->session->get('kivicare_appointment_id') !== 0 && WC()->session->get('doctor_id') !== 0 ){
        foreach( WC()->cart->get_cart() as $cart_item ){
            $kivicare_appointment_id= $cart_item['kivicare_appointment_id'];
        }
        if(!empty($kivicare_appointment_id)){
            global $wpdb;
            $kivicare_appointment_id= (int)$kivicare_appointment_id;
            $patient_id = $wpdb->get_var("SELECT patient_id FROM {$wpdb->prefix}kc_appointments WHERE id={$kivicare_appointment_id}");
            if(!empty($patient_id)){
                $first_name =  	get_user_meta( $patient_id, 'first_name', true );
                if(!empty($first_name)){
                    $fields['billing']['billing_first_name']['default'] = $first_name;
                    $fields['shipping']['shipping_first_name']['default'] = $first_name;
                }
                $last_name  = get_user_meta( $patient_id, 'last_name', true );
                if(!empty($last_name)){
                    $fields['billing']['billing_last_name']['default'] = $last_name;
                    $fields['shipping']['shipping_last_name']['default'] = $last_name;
                }

                $basic_data  = get_user_meta( $patient_id, 'basic_data', true );
                if(!empty($basic_data)){
                    $basic_data = json_decode($basic_data);
                    if(!empty($basic_data->postal_code)){

                        $fields['billing']['billing_postcode']['default'] = $basic_data->postal_code;
                        $fields['shipping']['shipping_postcode']['default'] = $basic_data->postal_code;
                    }
                    if(!empty($basic_data->city)){
                        $fields['billing']['billing_city']['default'] = $basic_data->city;
                        $fields['shipping']['shipping_city']['default'] = $basic_data->city;
                    }
                    if(!empty($basic_data->mobile_number)){
                        $fields['billing']['billing_phone']['default'] = $basic_data->mobile_number;
                        $fields['shipping']['billing_phone']['default'] = $basic_data->mobile_number;
                    }

                    if(!empty($basic_data->address)){
                        $fields['billing']['billing_address_1']['default'] = $basic_data->address;
                        $fields['shipping']['shipping_address_1']['default'] = $basic_data->address;
                    }

                    if(!empty($basic_data->country)){
                        $fields['billing']['billing_country']['default'] = 'IN';
                        $fields['shipping']['shipping_address_1']['default'] = $basic_data->address;
                    }
                }
            }
        }
    }

    return $fields;
}

function kcWoocommercePaymentGatewayEnable(){
    $response = 'off';
    if(!iskcWooCommerceActive()){
        update_option(KIVI_CARE_PREFIX.'woocommerce_payment',$response);
    }else{
        if(isKiviCareTelemedActive()){
            $response = apply_filters('kct_get_woocommerce_module_status', []);
        }elseif (isKiviCareGoogleMeetActive()){
            $response = apply_filters('kcgm_get_woocommerce_module_status', []);
        }
        elseif(isKiviCareProActive()){
            $response = apply_filters('kcpro_get_woocommerce_module_status', []);
        }
    }

    return $response;
}

function kcLocalPaymentGatewayEnable(){
    $data = get_option(KIVI_CARE_PREFIX.'local_payment_status',true);
    return gettype($data) !== 'boolean' && $data == 'on' ? 'on' : 'off';
}

function kcGetSingleWidgetSetting( $key_name ){
        
    $status = get_option(KIVI_CARE_PREFIX.'widgetSetting', false);
    $widgetSetting = json_decode( $status );
    $data = false;
    if ( $status ) {

        switch ( $key_name ) {
            case "clinicContactDetails":
                $data = !empty($widgetSetting->clinicContactDetails) ? $widgetSetting->clinicContactDetails : [];
              break;
            case "showDoctorImage":
                $data = !empty($widgetSetting->showDoctorImage) ? $widgetSetting->showDoctorImage : 0;
                break;
            case "doctorContactDetails":
                $data = !empty($widgetSetting->doctorContactDetails) ? $widgetSetting->doctorContactDetails : [];
              break;
            case "showClinicImage":
                $data = !empty($widgetSetting->showClinicImage) && in_array($widgetSetting->showClinicImage,['true',true,1,'1']);
              break;
            case "showClinicAddress":
                $data = !empty($widgetSetting->showClinicAddress) && in_array($widgetSetting->showClinicAddress,['true',true,1,'1']);
              break;
            case "showDoctorExperience":
                $data = !empty($widgetSetting->showDoctorExperience) && in_array($widgetSetting->showDoctorExperience,['true',true,1,'1']);
              break;
            case 'showServiceImage':
                $data = !empty($widgetSetting->showServiceImage) && in_array($widgetSetting->showServiceImage, ['true', true, 1, '1']);
              break;
            case 'showServicetype':
                $data = !empty($widgetSetting->showServicetype) && in_array($widgetSetting->showServicetype, ['true', true, 1, '1']);
              break;
            case 'showServicePrice':
                $data = !empty($widgetSetting->showServicePrice) && in_array($widgetSetting->showServicePrice, ['true', true, 1, '1']);
                break;
            case 'widget_print':
                $data = !empty($widgetSetting->widget_print) && in_array($widgetSetting->widget_print, ['true', true, 1, '1']);
                break;
            case 'afterWoocommerceRedirect':
                $data = !empty($widgetSetting->afterWoocommerceRedirect) && in_array($widgetSetting->afterWoocommerceRedirect, ['true', true, 1, '1']);
                break;
            default:
                $data = false;
        }
    }

    return $data;
}

function kcDefaultAppointmentWidgetOrder(){
    return [
        [ "name" => __("Choose a Clinic","kc-lang"), "fixed" => false, "att_name" => 'clinic'],
        [ "name" => __("Choose Your Doctor","kc-lang"),  "fixed" => false, "att_name" => 'doctor'],
        [ "name" => __("Services from Category","kc-lang"), "fixed" => false, "att_name" => 'category'],
        [ "name" => __("Select Date and Time","kc-lang"),  "fixed" => true, "att_name" => 'date-time'],
        [ "name" => __("Appointment Extra Data ","kc-lang"),  "fixed" => true, "att_name" => 'file-uploads-custom'],
        [ "name" => __("User Detail Information","kc-lang"),  "fixed" => true, "att_name" => 'detail-info'],
        [ "name" => __("Confirmation","kc-lang"),  "fixed" => true, "att_name" => 'confirm']
    ];
}

function kcCheckExtraTabConditionInAppointmentWidget($returnType='all'){
    global $wpdb;
    $query = "SELECT count(*)  FROM {$wpdb->prefix}kc_custom_fields WHERE module_type = 'appointment_module'";
    $custom_module = $wpdb->get_var($query);
    $status = false;
    $option = get_option(KIVI_CARE_PREFIX.'appointment_description_config_data',true);
    $enableAppointmentDescription = gettype($option) == 'boolean' ? 'on' : $option;

    $custom_module_status = $custom_module > 0 && isKiviCareProActive();

    if($returnType == 'description'){
        return $enableAppointmentDescription === 'on';
    }
    if(kcAppointmentMultiFileUploadEnable() || $custom_module_status || $enableAppointmentDescription === 'on'){
        $status = true;
    }
    return $status;
}

function kcAppointmentWidgetLoader(){
    $logoImage = get_option(KIVI_CARE_PREFIX . 'widget_loader',true);
    return  gettype($logoImage) != 'boolean' ? wp_get_attachment_url($logoImage) : KIVI_CARE_DIR_URI.'assets/images/circles-menu-1.gif';
}

function isLoaderCustomUrl(){
    $logoImageUrl = get_option(KIVI_CARE_PREFIX . 'widget_loader',true);
    return gettype($logoImageUrl) != 'boolean';
}

function kcAppendLanguageInHead(){
    if(isKiviCareProActive()){
        $prefix = KIVI_CARE_PREFIX;
        $langType = get_option(KIVI_CARE_PREFIX.'locoTranslateState');
        if($langType !== false && in_array((string)$langType ,['1','true'])){
            $var = require KIVI_CARE_DIR.'resources/assets/lang/temp.php';
        }else{
            $upload_dir = wp_upload_dir();
            $dir_name = $prefix .'lang';
            $user_dirname = $upload_dir['baseurl'] . '/' . $dir_name;
            $var = json_decode(file_get_contents($user_dirname.'/temp.json'));
        }
    }else{
        $var = require KIVI_CARE_DIR.'resources/assets/lang/temp.php';
    }
    ?>

    <script>
        window.__kivicarelang = <?php echo json_encode($var);?>
    </script>
    <?php
}

function kcGoogleCaptchaData($type){

    $data = get_option(KIVI_CARE_PREFIX.'google_recaptcha',true);

    if($type === 'status'){
        return !empty($data['status']) ? $data['status'] : 'off';
    }

    if($type === 'site_key'){
        return !empty($data['site_key']) ? $data['site_key'] : '';
    }

    if($type === 'secret_key'){
        return !empty($data['secret_key']) ? $data['secret_key'] : '';
    }
}

function kcGetLogoutRedirectSetting($type){
    $data = get_option( KIVI_CARE_PREFIX.'logout_redirect');
    $logout_url = wp_login_url();
    if(empty($data['clinic_admin'])){
        $data = [
            "clinic_admin" => $logout_url,
            "patient" => $logout_url,
            "receptionist" => $logout_url,
            "doctor" => $logout_url
        ];
    }

    switch ($type){
        case 'all':
            return $data;
            break;
        case 'clinic':
        case 'doctor':
        case 'receptionist':
        case 'patient':
            return $data[$type];
            break;
    }
}

function kcPaypalSettingData($type='all'){
    $status = get_option(KIVI_CARE_PREFIX.'paypalConfig', true);
    if(gettype($status) !== 'boolean'){
        $status = json_decode( $status );
        $status->mode = !empty($status->mode) ? $status->mode : [];
        $status->client_id = !empty($status->client_id) ? $status->client_id : '';
        $status->client_secret = !empty($status->client_secret) ? $status->client_secret : '';
        $status->currency = !empty($status->currency) ? $status->currency : 'USD';
        $status->enablePaypal = !empty($status->enablePaypal) && in_array($status->enablePaypal,['true',true,1,'1']) ? true : false;
    }else{
        $status = false;
    }
    if($type == 'all'){
        return $status;
    }

    return !empty($status->{$type}) ? $status->{$type} : false;
}

function kcAllPaymentMethodList(){
    $temp = [
       'paymentOffline' => __("Pay Later","kc-lang")
    ];
    $woocommerce = kcWoocommercePaymentGatewayEnable();
    if($woocommerce === 'on'){
        $temp['paymentWoocommerce'] =  __("Woocommerce","kc-lang");
    }
    if(kcPaypalSettingData('enablePaypal')){
        $temp['paymentPaypal'] =  __("Paypal","kc-lang");
    }
    if(kcLocalPaymentGatewayEnable() == 'off'){
        unset($temp['paymentOffline']);
    }
    return $temp;
}

function kcGetDateFormat(){
    $data = get_option(KIVI_CARE_PREFIX.'date_format',true);
    return gettype($data) !== 'boolean' ? $data : 'D-MM-YYYY';
}

function kcPrescriptionHtml($data,$id,$type="encounter"){
    if(!is_user_logged_in()){
        return;
    }
    $kcBase = new KCBase();
    $current_user_role = $kcBase->getLoginUserRole();
    $themeColor = gettype(get_option(KIVI_CARE_PREFIX . 'theme_color',true)) != 'boolean' ? get_option(KIVI_CARE_PREFIX . 'theme_color') : '#4874dc';
    ob_start();
    ?>
    <div class="container-kivicare ">
        <link href="<?php echo esc_js(KIVI_CARE_DIR_URI.'assets/css/printPage.css');?>" rel="stylesheet" type="text/css" />
        <div class="head-content" style="border-bottom: 8px solid <?php echo esc_html($themeColor); ?>">
            <?php do_action('kivicare_print_before_clinic_data',$data,$id)?>
            <div class="hospital-content">
                <div style="width:33%">
                    <img src="<?php echo esc_url($data->clinic_logo); ?>" class="logo">
                </div>
                <div class="hospital-name">
                    <p  style="color:<?php echo esc_html($themeColor); ?>; font-size:30px" >
                        <?php echo esc_html($data->name); ?>
                    </p>
                </div>
                <div class=" mb-3 nameheading" style="width:33%; display: flex;justify-content: flex-end;">
                    <?php echo esc_html__('Date:', 'kc-lang') . esc_html($data->date); ?>
                </div>
            </div>
            <?php do_action('kivicare_print_after_clinic_data',$data,$id);?>
            <div class="doc-header ">
                <div class="doctor-details patient-contact-1">
                    <div class="doc-name d-flex align-items-center">
                        <p class="name">
                            <?php echo esc_html__('Dr.', 'kc-lang') . esc_html($data->doctor_name); ?>
                        </p>
                        <span class="doc-degree">
                            <?php echo !empty($data->qualifications) ? esc_html__('Qualifications:', 'kc-lang') . esc_html($data->qualifications) : ''; ?>
                        </span>
                    </div>
                    <div class="doc-address">
                        <p class="address d-flex align-items-center">
                            <span class="nameheading">
                                <?php echo esc_html__('Address: ', 'kc-lang'); ?>
                            </span>
                            <span>
                                <?php echo esc_html($data->clinic_address); ?>
                            </span>
                        </p>
                    </div>
                </div>
                <div class="doc-contact patient-contact-1">
                    <p class="contact d-flex align-items-center justify-content-end">
                        <span class="nameheading">
                            <?php echo esc_html__('Contact No: ', 'kc-lang') ; ?>
                        </span>
                        <span>
                            <?php echo  esc_html($data->telephone_no); ?>
                        </span>
                    </p>
                    <p class="contact d-flex align-items-center justify-content-end">
                        <span class="nameheading">
                            <?php echo esc_html__('Email: ', 'kc-lang') ; ?>
                        </span>
                        <span>
                            <?php echo  esc_html($data->email); ?>
                        </span>
                    </p>
                </div>
                <?php do_action('kivicare_print_after_doctor_data',$data,$id);?>
            </div>
        </div>
        <div class="body-content">
            <div id="patient-detail-table">
                <table >
                    <tr>
                        <th>
                            <?php echo esc_html__('Patient Name:', 'kc-lang'); ?>
                            <span>
                        <?php echo esc_html($data->patient_name); ?>
                    </span>
                        </th>
                        <th>
                            <?php echo esc_html__('Age:', 'kc-lang'); ?>
                            <span>
                        <?php echo !empty($data->patient_age) ? esc_html($data->patient_age) : ''; ?>
                    </span>
                        </th>
                        <th>
                            <?php echo esc_html__('Gender:', 'kc-lang'); ?>
                            <span>
                            <?php echo esc_html($data->patient_gender); ?>
                        </span>
                        </th>
                    </tr>
                    <tr>
                        <th>
                            <?php echo esc_html__('Email:', 'kc-lang'); ?>
                            <span>
                        <?php echo esc_html($data->patient_email); ?>
                    </span>
                        </th>
                        <th>
                            <?php echo esc_html__('Contact No:', 'kc-lang'); ?>
                            <span>
                        <?php echo esc_html($data->contact_no); ?>
                    </span>
                        </th>
                        <th>
                        <span>

                        </span>
                        </th>
                    </tr>
                </table>
            </div>
            <?php do_action('kivicare_print_after_patient_data',$data,$id,$current_user_role);
            if($type === 'encounter'){
                kcEncounterPrintTableContent($themeColor, $data, $id,$current_user_role);
            }
            if($type === 'appointment'){
                kcAppointmentPrintContent($themeColor, $data, $id,$current_user_role);
            }
            if($type == 'bill_detail'){
                kcEncounterBillDetailsPrintContent($themeColor, $data, $id,$current_user_role);
            }
            ?>
        </div>
        <div class="footer-print mb-3">
            <div class="border-top" style="border:1px solid <?php echo esc_html($themeColor); ?>">
            </div>
            <?php if($type === 'encounter'){ ?>
                <div class="d-flex justify-content-end" >
                    <p style="margin: 0;" class="nameheading">
                        <?php echo esc_html__('Doctor Signature:', 'kc-lang') ?>
                    </p>
                    <div class="doc-line">
                        <?php
                        $signature = get_user_meta($data->doctor_id,'doctor_signature',true);
                        if(!empty($signature)){
                            ?>
                            <img src="<?php echo esc_html($signature); ?>">
                            <?php
                        }
                        ?>
                    </div>
                </div>
            <?php  } do_action('kivicare_print_after_doctor_signature',$data,$id);?>
        </div>
    </div>
    <?php
    $htmlContent = ob_get_clean();
    $htmlContent = apply_filters('kivicare_print_html_content',$htmlContent,$data,$id);
    return $htmlContent;
}

function kcEncounterBillDetailsPrintContent($themeColor, $data, $id,$current_user_role =''){
    $clinicCurrencyDetails= kcGetClinicCurrenyPrefixAndPostfix();
    $prefix =  (!empty($clinicCurrencyDetails['prefix']) ? $clinicCurrencyDetails['prefix'] :'');
    $postfix = (!empty($clinicCurrencyDetails['postfix']) ? $clinicCurrencyDetails['postfix'] :'');
    $thStyle = "border:1px solid {$themeColor}";
    if(!empty($data->billItems)){
        ?>
        <table>
            <caption style="margin-bottom:20px; text-align:center;caption-side: unset;font-size: 20px;font-weight: bold;">
                <?php echo esc_html__('Services', 'kc-lang'); ?>
            </caption>
            <tr>
                <th class="" style="<?php echo esc_html($thStyle); ?>">
                    <?php echo esc_html__('SR NO', 'kc-lang'); ?>
                </th>
                <th class="" style="<?php echo esc_html($thStyle); ?>">
                    <?php echo esc_html__('ITEM NAME', 'kc-lang'); ?>
                </th>
                <th class="" style="<?php echo esc_html($thStyle); ?>">
                    <?php echo esc_html__('PRICE', 'kc-lang'); ?>
                </th>
                <th class="" style="<?php echo esc_html($thStyle); ?>">
                    <?php echo esc_html__('QUANTITY', 'kc-lang'); ?>
                </th>
                <th class="" style="<?php echo esc_html($thStyle); ?>">
                    <?php echo esc_html__('TOTAL', 'kc-lang'); ?>
                </th>
            </tr>
            <?php foreach ($data->billItems as $key => $pre) {
                ?>
                <tr>
                    <td class="" style="<?php echo esc_html($thStyle); ?>">
                        <?php echo esc_html($key + 1); ?>
                    </td>
                    <td class=" width-33" style="<?php echo esc_html($thStyle); ?>">
                        <?php echo esc_html($pre['item_id']['label']); ?>
                    </td>
                    <td class="" style="<?php echo esc_html($thStyle); ?>">
                        <?php echo esc_html($prefix.$pre['price'].$postfix); ?>
                    </td>
                    <td class="" style="<?php echo esc_html($thStyle); ?>">
                        <?php echo esc_html($pre['qty']); ?>
                    </td>
                    <td class="" style="<?php echo esc_html($thStyle); ?>">
                        <?php echo esc_html($prefix.($pre['price'] * $pre['qty']).$postfix); ?>
                    </td>
                </tr>
                <?php
            } ?>
            <tr>
                <th class="" colspan="4" style="<?php echo esc_html($thStyle); ?>;text-align: right">
                    <?php echo esc_html__('Total','kc-lang'); ?>
                </th>
                <td class=" " style="<?php echo esc_html($thStyle); ?>;">
                    <?php echo esc_html($prefix.$data->total_amount.$postfix); ?>
                </td>
            </tr>
            <tr>
                <th class="" colspan="4" style="<?php echo esc_html($thStyle); ?>;text-align: right">
                    <?php echo esc_html__('Discount','kc-lang'); ?>
                </th>
                <td class="" style="<?php echo esc_html($thStyle); ?>">
                    <?php echo esc_html($prefix.$data->discount.$postfix); ?>
                </td>
            </tr>
            <tr>
                <th colspan="4" style="<?php echo esc_html($thStyle); ?>;">
                    <div class="d-flex align-items-center justify-content-between">
                        <span style="text-align: left">
                        <?php echo esc_html__('Payment Status: ','kc-lang'); ?>
                            <span class="iq-print-badge" style="color: <?php echo $themeColor ?>">
                                <?php echo esc_html(ucfirst($data->payment_status)); ?>
                            </span>
                    </span>
                        <span style="text-align: right">
                        <?php echo esc_html__('Amount due','kc-lang'); ?>
                    </span>
                    </div>
                </th>
                <td class=" " style="<?php echo esc_html($thStyle); ?>">
                    <?php echo esc_html($prefix.$data->actual_amount.$postfix); ?>
                </td>
            </tr>
        </table>
        <?php
    }
}

function kcEncounterPrintTableContent($themeColor, $data, $id,$current_user_role =''){
    $thStyle = "border:1px solid {$themeColor}"
    ?>
    <table>
        <caption style="margin-bottom:20px; text-align:center;caption-side: unset;font-size: 20px;font-weight: bold;">
            <?php echo esc_html__('Prescriptions', 'kc-lang'); ?>
        </caption>
        <tr>
            <th class="" style="<?php echo esc_html($thStyle); ?>">
                <?php echo esc_html__('Name', 'kc-lang'); ?>
            </th>
            <th class="width-33" style="<?php echo esc_html($thStyle); ?>">
                <?php echo esc_html__('Frequency', 'kc-lang'); ?>
            </th>
            <th class="" style="<?php echo esc_html($thStyle); ?>">
                <?php echo esc_html__('Duration', 'kc-lang'); ?>
            </th>
            <th class="" style="<?php echo esc_html($thStyle); ?>">
                <?php echo esc_html__('Instruction', 'kc-lang'); ?>
            </th>
        </tr>
        <?php foreach ($data->prescription as $pre) {
            ?>
            <tr>
                <td class="" style="<?php echo esc_html($thStyle); ?>">
                    <?php echo esc_html($pre->name); ?>
                </td>
                <td class=" width-33" style="<?php echo esc_html($thStyle); ?>">
                    <?php echo esc_html($pre->frequency); ?>
                </td>
                <td class="" style="<?php echo esc_html($thStyle); ?>">
                    <?php echo esc_html($pre->duration) . esc_html__(' day', 'kc-lang'); ?>
                </td>
                <td class="" style="<?php echo esc_html($thStyle); ?>">
                    <?php echo esc_html($pre->instruction); ?>
                </td>
            </tr>
            <?php
        } ?>
    </table>
    <?php
    do_action('kivicare_print_after_prescription_data', $data, $id);
    if (!empty($data->medical_history['show']) && ($data->medical_history['show'] === 'true' || $data->medical_history['show'] === true)) {
        ?>
        <table>
            <caption style="margin-bottom:20px; text-align:center;caption-side: unset;font-size: 20px;font-weight: bold;">
                <?php echo esc_html__('Clinical Detail', 'kc-lang') ?>
            </caption>
            <tr>
                <?php
                if (!empty($data->medical_history['problem']) && $data->medical_history['problem'] != '') {
                    ?>
                    <th style="<?php echo esc_html($thStyle); ?>">
                        <?php echo esc_html__('Problems', 'kc-lang') ?>
                    </th>
                    <?php
                }
                ?>
                <?php
                if (!empty($data->medical_history['observation']) && $data->medical_history['observation'] != '') {
                    ?>
                    <th style="<?php echo esc_html($thStyle); ?>">
                        <?php echo esc_html__('Observations', 'kc-lang') ?>
                    </th>
                    <?php
                }
                ?>
                <?php
                if (!empty($data->medical_history['note']) && $data->medical_history['note'] != '') {
                    ?>
                    <th style="<?php echo esc_html($thStyle); ?>">
                        <?php echo esc_html__('Notes', 'kc-lang') ?>
                    </th>
                    <?php
                }
                ?>
            </tr>
            <?php
            if (!empty($data->medical_history['count']) && $data->medical_history['count'] != 0) {
                for ($i = 0; $i < (int)$data->medical_history['count']; $i++) {
                    ?>
                    <tr>
                        <?php
                        if (!empty($data->medical_history['problem']) && $data->medical_history['problem'] != '') {
                            ?>
                            <td class="" style="<?php echo esc_html($thStyle); ?>">
                                <?php echo esc_html(!empty($data->medical_history['problem'][$i]) ? $data->medical_history['problem'][$i] : ''); ?>
                            </td>
                            <?php
                        }
                        ?>
                        <?php
                        if (!empty($data->medical_history['observation']) && $data->medical_history['observation'] != '') {
                            ?>
                            <td class="" style="<?php echo esc_html($thStyle); ?>">
                                <?php echo esc_html(!empty($data->medical_history['observation'][$i]) ? $data->medical_history['observation'][$i] : ''); ?>
                            </td>
                            <?php
                        }
                        ?>
                        <?php
                        if (!empty($data->medical_history['note']) && $data->medical_history['note'] != '') {
                            ?>
                            <td class="" style="<?php echo esc_html($thStyle); ?>">
                                <?php echo esc_html(!empty($data->medical_history['note'][$i]) ? $data->medical_history['note'][$i] : ''); ?>
                            </td>
                            <?php
                        }
                        ?>
                    </tr>
                    <?php
                }
            }
            ?>
        </table>
        <?php
        do_action('kivicare_print_after_clinical_detail_data', $data, $id);
    }

}

function kcAppointmentPrintContent($themeColor, $appointmentData, $id,$current_user_role =''){
    global $wpdb;
    if(!empty($id)){

        $id = (int)$id;
        if(empty($appointmentData)){
           return;
        }

        $appointment_services = (new KCAppointmentServiceMapping())->get_by([
            'appointment_id' => $id,
        ]);

        $serviceCharges = 0;
        $telemedAppointment = false;
        foreach ($appointment_services as $key => $value) {
            $service_name = kcGetServiceById($value->service_id);
            if(in_array(trim($service_name->name),['telemed','Telemed'])){
                $telemedAppointment = true;
            }
            $service_charges = kcGetServiceCharges([
                'service_id' => $value->service_id,
                'doctor_id' =>  $appointmentData->doctor_id
            ]);
            $serviceCharges = $serviceCharges +$service_charges->charges;
        }

        $serviceName = !empty($appointmentData->all_services_name) ? $appointmentData->all_services_name : '';
        $clinicCurrencyDetails= kcGetClinicCurrenyPrefixAndPostfix();
        if(!empty($serviceCharges) && $serviceCharges != 0){
            $serviceCharges = (!empty($clinicCurrencyDetails['prefix']) ? $clinicCurrencyDetails['prefix'] :'').$serviceCharges.(!empty($clinicCurrencyDetails['postfix']) ? $clinicCurrencyDetails['postfix'] :'');
        }
        $custom_field =  kcGetCustomFields('appointment_module', $id, (int)$appointmentData->doctor_id,true);
        $Appointment_status = '';

        switch ($appointmentData->status){
            case '0':
                $Appointment_status = __('Cancelled','kc-lang');
                break;
            case '1':
                $Appointment_status = __('Booked','kc-lang');
                break;
            case '3':
                $Appointment_status = __('Check Out','kc-lang');
                break;
            case '4':
                $Appointment_status = __('Check In','kc-lang');
                break;
        }

        $payment_mode = __('Manual','kc-lang');
        if(isKiviCareTelemedActive() || isKiviCareGoogleMeetActive() || isKiviCareProActive()){
            if(iskcWooCommerceActive()){
                $order_id = kcAppointmentIsWoocommerceOrder($id);
                if(!empty($order_id)){
                    $payment_mode = get_post_meta( $order_id, '_payment_method_title', true );
                }
            }
        }
        $checkPaymentMode = $wpdb->get_var("SELECT payment_mode FROM {$wpdb->prefix}kc_payments_appointment_mappings WHERE appointment_id={$id}");
        if(!empty($checkPaymentMode)){
            $payment_mode = $checkPaymentMode == 'paypal_rest' ? 'Paypal' : 'Manual';
        }

    }else{
        return;
    }
    do_action('kivicare_print_before_appointment_data', $appointmentData, $id);
    ?>
    <div id="appointmentDetail">
        <table>
            <caption
                    style="margin-bottom:20px; text-align:center;caption-side: unset;font-size: 20px;font-weight: bold;">
                <?php echo esc_html__('Appointment Detail', 'kc-lang') ?>
            </caption>
            <tr>
                <th>
                    <?php echo esc_html__('Appointment Date: ', 'kc-lang'); ?>
                    <span>
                            <?php echo esc_html(!empty($appointmentData->appointment_start_date) ? $appointmentData->appointment_start_date : ''); ?>
                        </span>
                </th>
                <th>
                    <?php echo esc_html__('Appointment Time: ', 'kc-lang'); ?>
                    <span>
                            <?php echo esc_html(!empty($appointmentData->appointment_start_time) ? (kcGetAppointmentTimeFormatOption() == 'on' ? date("H:i", strtotime($appointmentData->appointment_start_time)) :date("g:i a", strtotime($appointmentData->appointment_start_time))) : ''); ?>
                        </span>
                </th>
                <th>
                    <?php echo esc_html__('Appointment Status: ', 'kc-lang'); ?>
                    <span>
                            <?php echo esc_html($Appointment_status); ?>
                        </span>
                </th>
            </tr>
            <tr>
                <th>
                    <?php echo esc_html__('Payment Mode: ', 'kc-lang'); ?>
                    <span>
                            <?php echo esc_html($payment_mode); ?>
                        </span>
                </th>
                <th>
                    <?php echo esc_html__('Service: ', 'kc-lang'); ?>
                    <span>
                            <?php echo esc_html(!empty($serviceName) ? $serviceName : ''); ?>
                        </span>
                </th>
                <th>
                    <?php echo esc_html__('Total Bill Payment: ', 'kc-lang'); ?>
                    <span>
                            <?php echo esc_html(!empty($serviceCharges) ? $serviceCharges : 0); ?>
                        </span>
                </th>
            </tr>
        </table>
        <?php
        do_action('kivicare_print_after_appointment_data', $appointmentData, $id);
        if (kcCheckExtraTabConditionInAppointmentWidget('all') && !empty($custom_field)) {
            ?>
            <table>
                <caption
                        style="margin-bottom:20px; text-align:center;caption-side: unset;font-size: 20px;font-weight: bold;">
                    <?php echo esc_html__('Other Info', 'kc-lang') ?>
                </caption>
                <?php if (kcCheckExtraTabConditionInAppointmentWidget('description')) {
                    ?>
                    <tr style="width=100%">
                        <th>
                            <?php echo esc_html__('Description: ', 'kc-lang'); ?>
                            <span>
                              <?php echo esc_html(!empty($appointmentData->description) ? $appointmentData->description : ''); ?>
                             </span>
                        </th>
                    </tr>
                    <?php
                } ?>
                <?php if(isKiviCareProActive()){
                    ?>
                        <?php
                        foreach ($custom_field as $customKey => $customValue) {
                            if (!empty($customValue['field_data'])) {
                                if($customKey  % 3 == 0) {
                                    ?>
                                    <tr>
                                    <?php
                                }
                                ?>
                                <th>
                                   <?php echo esc_html(!empty($customValue['label']) ? $customValue['label'] . ':' : ''); ?>
                                   <span>
                                   <?php if (!empty($customValue['type']) && $customValue['type'] === 'checkbox') {
                                        foreach ($customValue['options'] as $ckey => $cValue) {
                                            ?>
                                        <input type="checkbox" id="<?php echo esc_html($cValue['id']); ?>"
                                          <?php echo esc_html(in_array($cValue['text'], $customValue['field_data']) ? 'checked' : ''); ?> readonly style="height: unset;margin-right:unset;accent-color:<?php echo esc_html($themeColor); ?>;vertical-align:unset">
                                          <?php echo esc_html($cValue['text']); ?>
                                            <?php
                                        }
                                    } else {
                                        ?>
                                       <?php echo esc_html(!empty($customValue['field_data']) ? $customValue['field_data'] : ''); ?>
                                        <?php
                                    } ?>
                                   </span>
                                </th>
                                <?php
                                if(($customKey + 1 )  % 3 == 0) {
                                ?>
                                </tr>
                                 <?php
                                }
                            }
                        }
                        ?>
                    </div>
                    <?php
                }?>
            </table>
            <?php
        }
        if($telemedAppointment && (isKiviCareGoogleMeetActive() || isKiviCareTelemedActive())){
            $doctor_meeting_type = kcCheckDoctorTelemedType($appointmentData->id);
            if($doctor_meeting_type === 'zoom'){
                $meeting_data = $wpdb->get_row("SELECT * FROM  {$wpdb->prefix}kc_appointment_zoom_mappings  WHERE appointment_id= {$id}");
                $meetingJoinLink = !empty($meeting_data->join_url) ? $meeting_data->join_url : '';
                $meetingStartLink = !empty($meeting_data->start_url) ? $meeting_data->start_url : '';
            }else{
                $meeting_data = $wpdb->get_row("SELECT * FROM  {$wpdb->prefix}wp_kc_appointment_google_meet_mappings  WHERE appointment_id= {$id}");
                $meetingJoinLink = !empty($meeting_data->url) ? $meeting_data->url : '';
                $meetingStartLink = $meetingJoinLink;
            }
            if(!empty($meetingJoinLink) && !empty($meetingStartLink)){
             ?>
            <table style="table-layout: fixed; width: 100%">
                <caption
                        style="margin-bottom:20px; text-align:center;caption-side: unset;font-size: 20px;font-weight: bold;">
                    <?php echo esc_html__('Telemed Meeting Info', 'kc-lang') ?>
                </caption>
                <?php if(!in_array($current_user_role,['kiviCare_patient'])){
                   ?>
                   <tr>
                    <th style="word-wrap: break-word">
                        <?php echo esc_html__('Meeting Start Link: ', 'kc-lang'); ?>
                        <span>
                            <a href="<?php echo esc_url($meetingJoinLink); ?>" target="_blank"><?php echo esc_html($meetingJoinLink); ?></a>
                        </span>
                    </th>
                </tr>
                   <?php
                }
                if(!in_array($current_user_role,['kiviCare_doctor'])){
                  ?>
                  <tr>
                    <th style="word-wrap: break-word">
                        <?php echo esc_html__('Meeting Join Link: ', 'kc-lang'); ?>
                        <span>
                            <a href="<?php echo esc_url($meetingStartLink); ?>" target="_blank"><?php echo esc_html($meetingStartLink); ?></a>
                        </span>
                    </th>
                </tr>
                 <?php
                }
                ?>
            </table>
            <?php
            }
        }
        ?>
    </div>
    <?php
}

function kcWidgetFooterContent($type,$currentActive){
    $currentActive = trim($currentActive);
    if($type == 'kivicare_error_msg_login_register'){
        $buttonText = esc_html__('Register', 'kc-lang');
    }else if($type == 'kivicare_error_msg_confirm'){
        $allPaymentMethod = kcAllPaymentMethodList();
        if(array_key_exists('paymentWoocommerce',$allPaymentMethod)
            || array_key_exists('paymentPaypal',$allPaymentMethod)){
            $buttonText = esc_html__('Next', 'kc-lang');
        }else{
            $buttonText = esc_html__('Confirm', 'kc-lang');
        }
    }else if($type === 'kivicare_payment_mode_confirm'){
        $buttonText = esc_html__('Confirm', 'kc-lang');
    }else{
        $buttonText = esc_html__('Next', 'kc-lang');
    }
    ob_start();
    ?>
    <div class="card-widget-footer">
        <span id="<?php echo esc_html($type); ?>" class="alert alert-popup alert-danger alert-left error" style="display:none">&nbsp;</span>
        <div class="d-flex justify-content-end gap-1-5 " style="margin-left: auto;">
            <?php if ($currentActive !== 'active') { ?>
                <button type="button" class="iq-button iq-button-secondary" id="iq-widget-back-button"
                        data-step="prev"><?php echo esc_html__('Back', 'kc-lang'); ?></button>
            <?php } ?>
            <?php if ($type == 'kivicare_error_msg_confirm_next') { ?>
                <button type="button" id="kivicare_confirm_next"
                        class="iq-button iq-button-primary" ><?php echo esc_html($buttonText); ?></button>
            <?php }else{?>
                <button type="submit" name="submit"  value="submit" data-step="next"
                        class="iq-button iq-button-primary" ><?php echo esc_html($buttonText); ?></button>
            <?php }?>
        </div>
    </div>
    <?php
    return ob_get_clean();
}

function kcAddToCalendarContent($appointment_data){
    global $wpdb;
    $post_table_name = $wpdb->prefix . 'posts';
    $post_name = strtolower(KIVI_CARE_PREFIX . 'default_event_template');
    $post_type = strtolower(KIVI_CARE_PREFIX . 'gcal_tmp');
    $calendar_template = $wpdb->get_row("SELECT * FROM {$post_table_name} WHERE post_name= '{$post_name}' AND post_type ='{$post_type}' AND post_status = 'publish' ", ARRAY_A);
    if(empty($calendar_template)){
        return ;
    }

    $calender_title = $calendar_template['post_title'];
    $calender_content = $calendar_template['post_content'];
    $appointment_id =(int)$appointment_data['id'];
    $appointment = $wpdb->get_row("SELECT * FROM {$wpdb->prefix}kc_appointments WHERE id={$appointment_id}");
    $content_data = kcCommonNotificationData($appointment,[],$appointment_data['appointment_service'],'clinic');
    $calender_content = kcEmailContentKeyReplace($calender_content,  $content_data);
    $calender_title = kcEmailContentKeyReplace($calender_title,  $content_data);

    return [
        "name" => $calender_title,
        "description" => $calender_content,
        "startDate" => $appointment_data['start_date'],
        "endDate" => $appointment_data['end_date'],
        "startTime" => date("H:i", strtotime($appointment_data['start_time'])),
        "endTime" => date("H:i", strtotime($appointment_data['end_time'])),
        "location" => $appointment_data['clinic_address'],
        "timeZone" => get_option('timezone_string'),
        "iCalFileName" => "Reminder-Event",
    ];

}